@@ -41,6 +41,8 @@
 #include <string>
 #include <type_traits>
 
+#include "frequency_monitor.hpp"
+
 #include "testing_matmul.hpp"
 
 #include "type_dispatch.hpp"
@@ -75,22 +77,24 @@ void run_function(const func_map& map, const Arguments& arg, const std::string&
 // Template to dispatch testing_matmul for performance tests
 // the test is marked invalid when (TiA, TiB, To, Tc) not in (H/H/S, B/B/S)
 template <typename TiA,
-          typename TiB = TiA,
-          typename To  = TiB,
-          typename Tc  = To,
-          typename Tci = TiA,
-          typename     = void>
+          typename TiB  = TiA,
+          typename To   = TiB,
+          typename Tc   = To,
+          typename TciA = TiA,
+          typename TciB = TiB,
+          typename      = void>
 struct perf_matmul : hipblaslt_test_invalid
 {
 };
 
-template <typename TiA, typename TiB, typename To, typename Tc, typename Tci>
+template <typename TiA, typename TiB, typename To, typename Tc, typename TciA, typename TciB>
 struct perf_matmul<
     TiA,
     TiB,
     To,
     Tc,
-    Tci,
+    TciA,
+    TciB,
     std::enable_if_t<
         (std::is_same<TiA, hipblasLtHalf>{} && std::is_same<TiB, hipblasLtHalf>{})
         || (std::is_same<TiA, hip_bfloat16>{} && std::is_same<TiB, hip_bfloat16>{})
@@ -98,6 +102,13 @@ struct perf_matmul<
         || (std::is_same<TiA, hipblaslt_f8_fnuz>{} && std::is_same<TiB, hipblaslt_f8_fnuz>{})
         || (std::is_same<TiA, hipblaslt_f8_fnuz>{} && std::is_same<TiB, hipblaslt_bf8_fnuz>{})
         || (std::is_same<TiA, hipblaslt_bf8_fnuz>{} && std::is_same<TiB, hipblaslt_f8_fnuz>{})
+        || (std::is_same<TiA, hipblaslt_bf8_fnuz>{} && std::is_same<TiB, hipblaslt_bf8_fnuz>{})
+#ifdef ROCM_USE_FLOAT8
+        || (std::is_same<TiA, hipblaslt_f8>{} && std::is_same<TiB, hipblaslt_f8>{})
+        || (std::is_same<TiA, hipblaslt_f8>{} && std::is_same<TiB, hipblaslt_bf8>{})
+        || (std::is_same<TiA, hipblaslt_bf8>{} && std::is_same<TiB, hipblaslt_f8>{})
+        || (std::is_same<TiA, hipblaslt_bf8>{} && std::is_same<TiB, hipblaslt_bf8>{})
+#endif
         || (std::is_same<TiA, double>{} && std::is_same<TiB, double>{})
         || (std::is_same<TiA, hipblasLtInt8>{} && std::is_same<TiB, hipblasLtInt8>{})
         || (std::is_same<TiA, hipblaslt_f8_fnuz>{} && std::is_same<TiB, hipblasLtHalf>{})
@@ -106,7 +117,7 @@ struct perf_matmul<
 {
     void operator()(const Arguments& arg)
     {
-        static const func_map map = {{"matmul", testing_matmul<TiA, TiB, To, Tc, Tci>}};
+        static const func_map map = {{"matmul", testing_matmul<TiA, TiB, To, Tc, TciA, TciB>}};
         run_function(map, arg);
     }
 };
@@ -267,9 +278,13 @@ try
     std::string c_type;
     std::string d_type;
     std::string compute_type;
+    std::string compute_input_typeA;
+    std::string compute_input_typeB;
     std::string scale_type;
     std::string bias_type;
     std::string bias_source;
+    std::string scaleAFormat;
+    std::string scaleBFormat;
     std::string initialization;
     std::string filter;
     std::string activation_type;
@@ -367,24 +382,32 @@ try
 
         ("a_type",
          value<std::string>(&a_type), "Precision of matrix A. "
-        "Options: f32_r,f16_r,bf16_r")
+        "Options: f32_r,f16_r,bf16_r,i8_r")
 
         ("b_type",
          value<std::string>(&b_type), "Precision of matrix B. "
-        "Options: f32_r,f16_r,bf16_r")
+        "Options: f32_r,f16_r,bf16_r,i8_r")
 
         ("c_type",
          value<std::string>(&c_type), "Precision of matrix C. "
-         "Options: f32_r,f16_r,bf16_r")
+         "Options: f32_r,f16_r,bf16_r,i8_r")
 
         ("d_type",
          value<std::string>(&d_type), "Precision of matrix D. "
-        "Options: f32_r,f16_r,bf16_r")
+        "Options: f32_r,f16_r,bf16_r,i8_r")
 
         ("compute_type",
          value<std::string>(&compute_type)->default_value("f32_r"), "Precision of computation. "
          "Options: s,f32_r,x,xf32_r,f64_r,i32_r")
 
+        ("compute_input_typeA",
+         value<std::string>(&compute_input_typeA), "Precision of computation input A. "
+         "Options: f32_r, f16_r, bf16_r, f8_r, bf8_r, The default value indicates that the compute_input_typeA has no effect.")
+
+        ("compute_input_typeB",
+         value<std::string>(&compute_input_typeB), "Precision of computation input B. "
+         "Options: f32_r, f16_r, bf16_r, f8_r, bf8_r, The default value indicates that the compute_input_typeA has no effect.")
+
         ("scale_type",
          value<std::string>(&scale_type), "Precision of scalar. "
         "Options: f16_r,bf16_r")
@@ -392,7 +415,7 @@ try
         ("initialization",
          value<std::string>(&initialization)->default_value("hpl"),
          "Initialize matrix data."
-         "Options: rand_int, trig_float, hpl(floating)")
+         "Options: rand_int, trig_float, hpl(floating), special, zero")
 
         ("transA",
          value<char>(&arg.transA)->default_value('N'),
@@ -459,12 +482,12 @@ try
          "Apply bias vector")
 
         ("scaleA",
-         bool_switch(&arg.scaleA)->default_value(false),
-         "Apply scale for A buffer")
+         value<std::string>(&scaleAFormat)->default_value(""),
+         "Apply scale for A buffer. s = scalar, v = vector.")
 
         ("scaleB",
-         bool_switch(&arg.scaleB)->default_value(false),
-         "Apply scale for B buffer")
+         value<std::string>(&scaleBFormat)->default_value(""),
+         "Apply scale for B buffer. s = scalar, v = vector.")
 
         ("scaleAlpha_vector",
          bool_switch(&arg.scaleAlpha_vector)->default_value(false),
@@ -472,11 +495,15 @@ try
 
         ("amaxScaleA",
          bool_switch(&arg.amaxScaleA)->default_value(false),
-         "Apple scale for A buffer by abs max of A buffer")
+         "Apply scale for A buffer by abs max of A buffer")
 
         ("amaxScaleB",
          bool_switch(&arg.amaxScaleB)->default_value(false),
-         "Apple scale for B buffer by abs max of B buffer")
+         "Apply scale for B buffer by abs max of B buffer")
+
+        ("amaxD",
+         bool_switch(&arg.amaxD)->default_value(false),
+         "Output Amax of intermediate D matrix")
 
         ("use_e",
          bool_switch(&arg.use_e)->default_value(false),
@@ -498,9 +525,9 @@ try
          value<int>(&device_id)->default_value(0),
          "Set default device to be used for subsequent program runs")
 
-        ("c_noalias_d",
-         bool_switch(&arg.c_noalias_d)->default_value(false),
-         "C and D are stored in separate memory")
+        ("c_equal_d",
+         bool_switch(&arg.c_equal_d)->default_value(false),
+         "C and D are stored in same memory")
 
         ("workspace",
          value<size_t>(&arg.user_allocated_workspace)->default_value(0),
@@ -539,6 +566,10 @@ try
          valueVec<uint32_t>(&wgm_vector),
          "[Tuning parameter] Set workgroup mapping for a solution, 0 is use solution's default value. (Only support GEMM + api_method mix or cpp)")
 
+        ("flush",
+        value<bool>(&arg.flush)->default_value(false),
+        "Flush icache, only works for gemm.")
+
         ("help,h", "produces this help message")
 
         ("version", "Prints the version number");
@@ -715,6 +746,9 @@ try
         throw std::invalid_argument("Invalid Device ID");
     set_device(device_id);
 
+    FrequencyMonitor& freq_monitor = getFrequencyMonitor();
+    freq_monitor.set_device_id(device_id);
+
     if(datafile)
         return hipblaslt_bench_datafile(filter, any_stride);
 
@@ -758,6 +792,21 @@ try
     if(arg.compute_type == static_cast<hipblasComputeType_t>(0))
         throw std::invalid_argument("Invalid value for --compute_type " + compute_type);
 
+    //The value HIPBLASLT_DATATYPE_INVALID indicates that the compute_input_typeA has no effect.
+    arg.compute_input_typeA = (compute_input_typeA != "")
+                                  ? string_to_hip_datatype(compute_input_typeA)
+                                  : HIPBLASLT_DATATYPE_INVALID;
+    if(arg.compute_input_typeA == HIPBLASLT_DATATYPE_INVALID && compute_input_typeA != "")
+        throw std::invalid_argument("Invalid value for --compute_input_typeA "
+                                    + compute_input_typeA);
+
+    arg.compute_input_typeB = (compute_input_typeB != "")
+                                  ? string_to_hip_datatype(compute_input_typeB)
+                                  : HIPBLASLT_DATATYPE_INVALID;
+    if(arg.compute_input_typeB == HIPBLASLT_DATATYPE_INVALID && compute_input_typeB != "")
+        throw std::invalid_argument("Invalid value for --compute_input_typeB "
+                                    + compute_input_typeB);
+
     if(string_to_hip_datatype(bias_type) == HIPBLASLT_DATATYPE_INVALID && bias_type != ""
        && bias_type != "default")
         throw std::invalid_argument("Invalid value for --bias_type " + bias_type);
@@ -774,6 +823,16 @@ try
 
     arg.bias_source = string_to_hipblaslt_bias_source(bias_source);
 
+    auto scaleString2Enum = [](std::string& s) {
+        if(s == "s")
+            return Arguments::ScalingFormat::Scalar;
+        if(s == "v")
+            return Arguments::ScalingFormat::Vector;
+        return Arguments::ScalingFormat::None;
+    };
+    arg.scaleA = scaleString2Enum(scaleAFormat);
+    arg.scaleB = scaleString2Enum(scaleBFormat);
+
     if(arg.M[0] < 0)
         throw std::invalid_argument("Invalid value for -m " + std::to_string(arg.M[0]));
     if(arg.N[0] < 0)
@@ -786,7 +845,10 @@ try
         throw std::invalid_argument("Invalid value for --function");
 
     if(verify)
-        arg.norm_check = 1;
+    {
+        arg.norm_check     = 1;
+        arg.allclose_check = 1;
+    }
 
     switch(api_method)
     {
@@ -808,7 +870,9 @@ try
     }
 
     arg.norm_check_assert = false;
-    return run_bench_test(arg, filter, any_stride);
+    int status            = run_bench_test(arg, filter, any_stride);
+    freeFrequencyMonitor();
+    return status;
 }
 catch(const std::invalid_argument& exp)
 {
