@@ -41,9 +41,6 @@
 // Predeclare enumerator
 enum hipblaslt_argument : int;
 
-/*! \brief device matches pattern */
-bool gpu_arch_match(const std::string& gpu_arch, const char pattern[4]);
-
 /***************************************************************************
  *! \brief Class used to parse command arguments in both client & gtest    *
  * WARNING: If this data is changed, then hipblaslt_common.yaml must also be *
@@ -52,6 +49,12 @@ bool gpu_arch_match(const std::string& gpu_arch, const char pattern[4]);
 constexpr std::size_t MAX_SUPPORTED_NUM_PROBLEMS{32};
 struct Arguments
 {
+    enum ScalingFormat
+    {
+        None = 0,
+        Scalar,
+        Vector
+    };
     /*************************************************************************
      *                    Beginning Of Arguments                             *
      *************************************************************************/
@@ -97,13 +100,15 @@ struct Arguments
     hipDataType          c_type;
     hipDataType          d_type;
     hipblasComputeType_t compute_type;
+    hipDataType          compute_input_typeA;
+    hipDataType          compute_input_typeB;
     hipDataType          scale_type;
 
     hipblaslt_initialization initialization;
 
-    // the gpu arch string after "gfx" for which the test is valid
-    // '?' is wildcard char, empty string is default as valid on all
-    char gpu_arch[4];
+    // the gpu arch string after "gfx" for which the test is valid,
+    // it represents a regular expression.
+    char gpu_arch[16];
 
     // memory padding for testing write out of bounds
     uint32_t pad;
@@ -117,6 +122,7 @@ struct Arguments
     uint8_t devices;
 
     int8_t norm_check;
+    int8_t allclose_check;
     int8_t unit_check;
     int8_t timing;
 
@@ -130,15 +136,16 @@ struct Arguments
     hipDataType           bias_type;
     hipblaslt_bias_source bias_source;
     bool                  bias_vector;
-    bool                  scaleA;
-    bool                  scaleB;
+    ScalingFormat         scaleA;
+    ScalingFormat         scaleB;
     bool                  scaleC;
     bool                  scaleD;
     bool                  scaleE;
     bool                  scaleAlpha_vector;
     bool                  amaxScaleA;
     bool                  amaxScaleB;
-    bool                  c_noalias_d;
+    bool                  amaxD;
+    bool                  c_equal_d;
     bool                  HMM;
     bool                  use_e;
     bool                  gradient;
@@ -148,6 +155,7 @@ struct Arguments
     bool    use_ext;
     bool    use_ext_setproblem;
     int     algo_method; // 0 for getheuristic, 1 for get all algos, 2 for algo index
+    int     api_method; // 0 for c, 1 for mix, 2 for cpp
     bool    use_user_args;
     int32_t rotating;
     bool    use_gpu_timer;
@@ -160,6 +168,8 @@ struct Arguments
     bool print_solution_found;
     bool print_kernel_info;
 
+    bool flush;
+
     /*************************************************************************
      *                     End Of Arguments                                  *
      *************************************************************************/
@@ -203,6 +213,8 @@ struct Arguments
     OPER(c_type) SEP                 \
     OPER(d_type) SEP                 \
     OPER(compute_type) SEP           \
+    OPER(compute_input_typeA) SEP    \
+    OPER(compute_input_typeB) SEP    \
     OPER(scale_type) SEP             \
     OPER(initialization) SEP         \
     OPER(gpu_arch) SEP               \
@@ -212,6 +224,7 @@ struct Arguments
     OPER(streams) SEP                \
     OPER(devices) SEP                \
     OPER(norm_check) SEP             \
+    OPER(allclose_check) SEP         \
     OPER(unit_check) SEP             \
     OPER(timing) SEP                 \
     OPER(transA) SEP                 \
@@ -230,7 +243,8 @@ struct Arguments
     OPER(scaleAlpha_vector) SEP      \
     OPER(amaxScaleA) SEP             \
     OPER(amaxScaleB) SEP             \
-    OPER(c_noalias_d) SEP            \
+    OPER(amaxD) SEP                  \
+    OPER(c_equal_d) SEP              \
     OPER(HMM) SEP                    \
     OPER(use_e) SEP                  \
     OPER(gradient) SEP               \
@@ -238,13 +252,15 @@ struct Arguments
     OPER(use_ext) SEP                \
     OPER(use_ext_setproblem) SEP     \
     OPER(algo_method) SEP            \
+    OPER(api_method) SEP             \
     OPER(use_user_args) SEP          \
     OPER(rotating) SEP               \
     OPER(use_gpu_timer) SEP          \
     OPER(gsu_vector) SEP             \
     OPER(wgm_vector) SEP             \
     OPER(print_solution_found) SEP   \
-    OPER(print_kernel_info) SEP
+    OPER(print_kernel_info) SEP      \
+    OPER(flush) SEP
 
     // clang-format on
 
