@@ -77,6 +77,7 @@
 #define ASSERT_HALF_EQ(a, b) ASSERT_FLOAT_EQ(float(a), float(b))
 #define ASSERT_BF16_EQ(a, b) ASSERT_FLOAT_EQ(float(a), float(b))
 #define ASSERT_F8_EQ(a, b) ASSERT_FLOAT_EQ(float(a), float(b))
+#define ASSERT_BF8_EQ(a, b) ASSERT_FLOAT_EQ(float(a), float(b))
 
 // Compare float to hip_bfloat16
 // Allow the hip_bfloat16 to match the rounded or truncated value of float
@@ -122,6 +123,32 @@ inline void unit_check_general(
     UNIT_CHECK(M, N, lda, 0, hCPU, hGPU, 1, ASSERT_F8_EQ);
 }
 
+template <>
+inline void unit_check_general(int64_t                   M,
+                               int64_t                   N,
+                               int64_t                   lda,
+                               const hipblaslt_bf8_fnuz* hCPU,
+                               const hipblaslt_bf8_fnuz* hGPU)
+{
+    UNIT_CHECK(M, N, lda, 0, hCPU, hGPU, 1, ASSERT_BF8_EQ);
+}
+
+#ifdef ROCM_USE_FLOAT8
+template <>
+inline void unit_check_general(
+    int64_t M, int64_t N, int64_t lda, const hipblaslt_f8* hCPU, const hipblaslt_f8* hGPU)
+{
+    UNIT_CHECK(M, N, lda, 0, hCPU, hGPU, 1, ASSERT_F8_EQ);
+}
+
+template <>
+inline void unit_check_general(
+    int64_t M, int64_t N, int64_t lda, const hipblaslt_bf8* hCPU, const hipblaslt_bf8* hGPU)
+{
+    UNIT_CHECK(M, N, lda, 0, hCPU, hGPU, 1, ASSERT_BF8_EQ);
+}
+#endif
+
 template <>
 inline void unit_check_general(
     int64_t M, int64_t N, int64_t lda, const hip_bfloat16* hCPU, const hip_bfloat16* hGPU)
@@ -204,6 +231,44 @@ inline void unit_check_general(int64_t                  M,
     UNIT_CHECK(M, N, lda, strideA, hCPU, hGPU, batch_count, ASSERT_F8_EQ);
 }
 
+template <>
+inline void unit_check_general(int64_t                   M,
+                               int64_t                   N,
+                               int64_t                   lda,
+                               int64_t                   strideA,
+                               const hipblaslt_bf8_fnuz* hCPU,
+                               const hipblaslt_bf8_fnuz* hGPU,
+                               int64_t                   batch_count)
+{
+    UNIT_CHECK(M, N, lda, strideA, hCPU, hGPU, batch_count, ASSERT_BF8_EQ);
+}
+
+#ifdef ROCM_USE_FLOAT8
+template <>
+inline void unit_check_general(int64_t             M,
+                               int64_t             N,
+                               int64_t             lda,
+                               int64_t             strideA,
+                               const hipblaslt_f8* hCPU,
+                               const hipblaslt_f8* hGPU,
+                               int64_t             batch_count)
+{
+    UNIT_CHECK(M, N, lda, strideA, hCPU, hGPU, batch_count, ASSERT_F8_EQ);
+}
+
+template <>
+inline void unit_check_general(int64_t              M,
+                               int64_t              N,
+                               int64_t              lda,
+                               int64_t              strideA,
+                               const hipblaslt_bf8* hCPU,
+                               const hipblaslt_bf8* hGPU,
+                               int64_t              batch_count)
+{
+    UNIT_CHECK(M, N, lda, strideA, hCPU, hGPU, batch_count, ASSERT_BF8_EQ);
+}
+#endif
+
 template <>
 inline void unit_check_general<hip_bfloat16, float>(int64_t             M,
                                                     int64_t             N,
