@@ -45,6 +45,10 @@
 
 namespace hipblaslt_ext
 {
+    class GemmInstance;
+    class Gemm;
+    class GroupedGemm;
+
     using HipBufferDeleter = hipError_t (*)(void*);
     using HipBufferPtr     = std::unique_ptr<void, HipBufferDeleter>;
 
@@ -58,7 +62,7 @@ namespace hipblaslt_ext
     };
 
     /*! \ingroup types_module
-     *  \brief hipblasLt extension preference for gemm problems.
+     *  \brief hipblasLt extension preference for gemm problems. (deprecated)
      *
      * \details Currently only supports setting max workspace size.
      */
@@ -85,7 +89,45 @@ namespace hipblaslt_ext
     };
 
     /*! \ingroup types_module
-     *  \brief hipblasLt extension ProblemType for gemm problems.
+     *  \brief hipblasLt extension preference for gemm problems.
+     *
+     * \details Currently only supports setting max workspace size.
+     */
+    class GemmPreferenceV2
+    {
+    public:
+        HIPBLASLT_EXPORT GemmPreferenceV2();
+        HIPBLASLT_EXPORT ~GemmPreferenceV2();
+
+        HIPBLASLT_EXPORT                   GemmPreferenceV2(const GemmPreferenceV2& pref);
+        HIPBLASLT_EXPORT GemmPreferenceV2& operator=(const GemmPreferenceV2& pref);
+
+        HIPBLASLT_EXPORT                   GemmPreferenceV2(GemmPreferenceV2&& pref);
+        HIPBLASLT_EXPORT GemmPreferenceV2& operator=(GemmPreferenceV2&& pref);
+
+        /*! \ingroup library_module
+         *  \brief This function sets the max workspace size.
+         *
+         *  @param[in]
+         *  workspaceBytes  Set the max workspace size in bytes.
+         */
+        HIPBLASLT_EXPORT void setMaxWorkspaceBytes(size_t workspaceBytes);
+
+        /*! \ingroup library_module
+         *  \brief This function returns the set max workspace size.
+         *
+         *  \retval size_t Returns the set max workspace size.
+         */
+        HIPBLASLT_EXPORT const size_t getMaxWorkspaceBytes() const;
+
+    private:
+        friend GemmInstance;
+        class GemmPreferenceImpl;
+        std::unique_ptr<GemmPreferenceImpl> pimpl;
+    };
+
+    /*! \ingroup types_module
+     *  \brief hipblasLt extension ProblemType for gemm problems. (deprecated)
      *
      * \details This structure sets the problem type of a gemm problem.
      */
@@ -101,7 +143,54 @@ namespace hipblaslt_ext
     };
 
     /*! \ingroup types_module
-     *  \brief hipblasLt extension Epilogue for gemm problems.
+     *  \brief hipblasLt extension ProblemType for gemm problems.
+     *
+     * \details This structure sets the problem type of a gemm problem.
+     */
+    class GemmProblemTypeV2
+    {
+    public:
+        HIPBLASLT_EXPORT GemmProblemTypeV2();
+        HIPBLASLT_EXPORT GemmProblemTypeV2(hipblasOperation_t   opA,
+                                           hipblasOperation_t   opB,
+                                           hipDataType          typeA,
+                                           hipDataType          typeB,
+                                           hipDataType          typeC,
+                                           hipDataType          typeD,
+                                           hipblasComputeType_t typeCompute);
+        HIPBLASLT_EXPORT ~GemmProblemTypeV2();
+
+        HIPBLASLT_EXPORT                    GemmProblemTypeV2(const GemmProblemTypeV2& type);
+        HIPBLASLT_EXPORT GemmProblemTypeV2& operator=(const GemmProblemTypeV2& type);
+
+        HIPBLASLT_EXPORT                    GemmProblemTypeV2(GemmProblemTypeV2&& type);
+        HIPBLASLT_EXPORT GemmProblemTypeV2& operator=(GemmProblemTypeV2&& type);
+
+        HIPBLASLT_EXPORT void setOpA(hipblasOperation_t op); //!< Set the A martix transpose.
+        HIPBLASLT_EXPORT void setOpB(hipblasOperation_t op); //!< Set the B matrix transpose.
+        HIPBLASLT_EXPORT void setTypeA(hipDataType type); //!< Set the A matrix datatype.
+        HIPBLASLT_EXPORT void setTypeB(hipDataType type); //!< Set the B matrix datatype.
+        HIPBLASLT_EXPORT void setTypeC(hipDataType type); //!< Set the C matrix datatype.
+        HIPBLASLT_EXPORT void setTypeD(hipDataType type); //!< Set the D matrix datatype.
+        HIPBLASLT_EXPORT void
+            setTypeCompute(hipblasComputeType_t type); //!< Set the compute datatype.
+
+        HIPBLASLT_EXPORT hipblasOperation_t   getOpA() const; //!< The A matrix transpose.
+        HIPBLASLT_EXPORT hipblasOperation_t   getOpB() const; //!< The B matrix transpose.
+        HIPBLASLT_EXPORT hipDataType          getTypeA() const; //!< The A matrix datatype.
+        HIPBLASLT_EXPORT hipDataType          getTypeB() const; //!< The B matrix datatype.
+        HIPBLASLT_EXPORT hipDataType          getTypeC() const; //!< The C matrix datatype.
+        HIPBLASLT_EXPORT hipDataType          getTypeD() const; //!< The D matrix datatype.
+        HIPBLASLT_EXPORT hipblasComputeType_t getTypeCompute() const; //!< The compute datatype.
+    private:
+        friend Gemm;
+        friend GroupedGemm;
+        class GemmProblemTypeImpl;
+        std::unique_ptr<GemmProblemTypeImpl> pimpl;
+    };
+
+    /*! \ingroup types_module
+     *  \brief hipblasLt extension Epilogue for gemm problems. (Deprecated)
      *
      * \details This structure sets the epilogue of a gemm problem.
      */
@@ -117,6 +206,56 @@ namespace hipblaslt_ext
             = 0; //!< The aux batch stride. Only works if mode is set to aux related epilogues.
     };
 
+    /*! \ingroup types_module
+     *  \brief hipblasLt extension Epilogue for gemm problems.
+     *
+     * \details This class sets the epilogue of a gemm problem.
+     */
+    class GemmEpilogueV2
+    {
+    public:
+        HIPBLASLT_EXPORT GemmEpilogueV2();
+        HIPBLASLT_EXPORT ~GemmEpilogueV2();
+
+        HIPBLASLT_EXPORT                 GemmEpilogueV2(const GemmEpilogueV2& epilogue);
+        HIPBLASLT_EXPORT GemmEpilogueV2& operator=(const GemmEpilogueV2& epilogue);
+
+        HIPBLASLT_EXPORT                 GemmEpilogueV2(GemmEpilogueV2&& epilogue);
+        HIPBLASLT_EXPORT GemmEpilogueV2& operator=(GemmEpilogueV2&& epilogue);
+
+        HIPBLASLT_EXPORT void
+            setMode(hipblasLtEpilogue_t mode); //!< Set the mode of epilogue. Default is gemm.
+        HIPBLASLT_EXPORT void setBiasDataType(
+            hipDataType
+                biasDataType); //!< Set the bias datatype. Only works if mode is set to bias related epilogues.
+        HIPBLASLT_EXPORT void setAuxLeadingDimension(
+            int auxLeadingDimension); //!< Set the aux leading dimension. Only works if mode is set to aux related epilogues.
+        HIPBLASLT_EXPORT void setAuxBatchStride(
+            int auxBatchStride); //!< Set the aux batch stride. Only works if mode is set to aux related epilogues.
+        HIPBLASLT_EXPORT void setScalingAType(
+            int scalingAType); //!< 0 is scalar, 1 is vector. Only works if DataTypeA = DataTypeB = FP8.
+        HIPBLASLT_EXPORT void setScalingBType(
+            int scalingBType); //!< 0 is scalar, 1 is vector. Only works if DataTypeA = DataTypeB = FP8.
+
+        HIPBLASLT_EXPORT hipblasLtEpilogue_t
+                                     getMode() const; //!< The mode of epilogue. Default is gemm.
+        HIPBLASLT_EXPORT hipDataType getBiasDataType()
+            const; //!< The bias datatype. Only works if mode is set to bias related epilogues.
+        HIPBLASLT_EXPORT int getAuxLeadingDimension()
+            const; //!< The aux leading dimension. Only works if mode is set to aux related epilogues.
+        HIPBLASLT_EXPORT int getAuxBatchStride()
+            const; //!< The aux batch stride. Only works if mode is set to aux related epilogues.
+        HIPBLASLT_EXPORT int getScalingAType()
+            const; //!< 0 is scalar, 1 is vector. Only works if DataTypeA = DataTypeB = FP8.
+        HIPBLASLT_EXPORT int getScalingBType()
+            const; //!< 0 is scalar, 1 is vector. Only works if DataTypeA = DataTypeB = FP8.
+    private:
+        friend Gemm;
+        friend GroupedGemm;
+        class GemmEpilogueImpl;
+        std::unique_ptr<GemmEpilogueImpl> pimpl;
+    };
+
     struct GemmTuning
     {
         uint8_t splitK = 0; //!< Value of splitK, 0 is off (use the splitK inside the solution).
@@ -124,28 +263,114 @@ namespace hipblaslt_ext
             = 0; //!< Value of workgroup mapping, 0 is off (use the workgroup mapping inside the solution).
     };
 
+    struct GemmTuningV2
+    {
+    public:
+        HIPBLASLT_EXPORT GemmTuningV2();
+        HIPBLASLT_EXPORT ~GemmTuningV2();
+
+        HIPBLASLT_EXPORT               GemmTuningV2(const GemmTuningV2& tuning);
+        HIPBLASLT_EXPORT GemmTuningV2& operator=(const GemmTuningV2& tuning);
+
+        HIPBLASLT_EXPORT               GemmTuningV2(GemmTuningV2&& tuning);
+        HIPBLASLT_EXPORT GemmTuningV2& operator=(GemmTuningV2&& tuning);
+
+        HIPBLASLT_EXPORT void setSplitK(
+            uint16_t
+                splitK); //!< Set the value of splitK, 0 is off (use the splitK inside the solution).
+        HIPBLASLT_EXPORT void setWgm(
+            int16_t
+                wgm); //!< Set the value of workgroup mapping, 0 is off (use the workgroup mapping inside the solution).
+
+        HIPBLASLT_EXPORT uint16_t getSplitK() const; //!< Value of splitK.
+        HIPBLASLT_EXPORT int16_t  getWgm() const; //!< Value of workgroup mapping.
+    private:
+        friend GemmInstance;
+        class GemmTuningImpl;
+        std::unique_ptr<GemmTuningImpl> pimpl;
+    };
+
     /*! \ingroup types_module
-     *  \brief hipblasLt extension Inputs for gemm problems.
+     *  \brief hipblasLt extension Inputs for gemm problems. (Deprecated)
      *
      * \details This structure sets the input pointers of a gemm problem.
      */
     struct GemmInputs
     {
-        void* a     = nullptr; //!< The a matrix input pointer.
-        void* b     = nullptr; //!< The b matrix input pointer.
-        void* c     = nullptr; //!< The c matrix input pointer.
-        void* d     = nullptr; //!< The d matrix input pointer.
-        void* alpha = nullptr; //!< The alpha value.
-        void* beta  = nullptr; //!< The beta value.
+        const void* a     = nullptr; //!< The a matrix input pointer.
+        const void* b     = nullptr; //!< The b matrix input pointer.
+        const void* c     = nullptr; //!< The c matrix input pointer.
+        const void* d     = nullptr; //!< The d matrix input pointer.
+        const void* alpha = nullptr; //!< The alpha value.
+        const void* beta  = nullptr; //!< The beta value.
         // Epilogue inputs
-        void* bias          = nullptr; //!< The bias input pointer.
-        void* scaleA        = nullptr; //!< The Scale A input pointer.
-        void* scaleB        = nullptr; //!< The Scale B input pointer.
-        void* scaleC        = nullptr; //!< The Scale C input pointer.
-        void* scaleD        = nullptr; //!< The Scale D input pointer.
-        void* scaleAux      = nullptr; //!< The Scale AUX input pointer.
-        void* scaleAlphaVec = nullptr; //!< The scaleAlpha vector input pointer.
-        void* aux           = nullptr; //!< The aux input pointer.
+        const void* bias          = nullptr; //!< The bias input pointer.
+        const void* scaleA        = nullptr; //!< The Scale A input pointer.
+        const void* scaleB        = nullptr; //!< The Scale B input pointer.
+        const void* scaleC        = nullptr; //!< The Scale C input pointer.
+        const void* scaleD        = nullptr; //!< The Scale D input pointer.
+        const void* scaleAux      = nullptr; //!< The Scale AUX input pointer.
+        const void* scaleAlphaVec = nullptr; //!< The scaleAlpha vector input pointer.
+        const void* aux           = nullptr; //!< The aux input pointer.
+    };
+
+    /*! \ingroup types_module
+     *  \brief hipblasLt extension Inputs for gemm problems.
+     *
+     * \details This class sets the input pointers of a gemm problem.
+     */
+    class GemmInputsV2
+    {
+    public:
+        HIPBLASLT_EXPORT GemmInputsV2();
+        HIPBLASLT_EXPORT ~GemmInputsV2();
+
+        HIPBLASLT_EXPORT               GemmInputsV2(const GemmInputsV2& input);
+        HIPBLASLT_EXPORT GemmInputsV2& operator=(const GemmInputsV2& input);
+
+        HIPBLASLT_EXPORT               GemmInputsV2(GemmInputsV2&& input);
+        HIPBLASLT_EXPORT GemmInputsV2& operator=(GemmInputsV2&& input);
+
+        HIPBLASLT_EXPORT void setA(const void* a); //!< Set the a matrix input pointer.
+        HIPBLASLT_EXPORT void setB(const void* b); //!< Set the b matrix input pointer.
+        HIPBLASLT_EXPORT void setC(const void* c); //!< Set the c matrix input pointer.
+        HIPBLASLT_EXPORT void setD(const void* d); //!< Set the d matrix input pointer.
+        HIPBLASLT_EXPORT void setAlpha(const void* alpha); //!< Set the alpha value.
+        HIPBLASLT_EXPORT void setBeta(const void* beta); //!< Set the beta value.
+        HIPBLASLT_EXPORT void setBias(const void* bias); //!< Set the bias input pointer.
+        HIPBLASLT_EXPORT void setScaleA(const void* scaleA); //!< Set the Scale A input pointer.
+        HIPBLASLT_EXPORT void setScaleB(const void* scaleB); //!< Set the Scale B input pointer.
+        HIPBLASLT_EXPORT void setScaleC(const void* scaleC); //!< Set the Scale C input pointer.
+        HIPBLASLT_EXPORT void setScaleD(const void* scaleD); //!< Set the Scale D input pointer.
+        HIPBLASLT_EXPORT void
+            setScaleAux(const void* scaleAux); //!< Set the Scale AUX input pointer.
+        HIPBLASLT_EXPORT void setScaleAlphaVec(
+            const void* scaleAlphaVec); //!< Set the scaleAlpha vector input pointer.
+        HIPBLASLT_EXPORT void setAux(const void* aux); //!< Set the aux input pointer.
+        HIPBLASLT_EXPORT void setAmaxD(const void* amaxD); //!< Set the AmaxD input pointer.
+
+        HIPBLASLT_EXPORT const void* getA() const; //!< The a matrix input pointer.
+        HIPBLASLT_EXPORT const void* getB() const; //!< The b matrix input pointer.
+        HIPBLASLT_EXPORT const void* getC() const; //!< The c matrix input pointer.
+        HIPBLASLT_EXPORT const void* getD() const; //!< The d matrix input pointer.
+        HIPBLASLT_EXPORT const void* getAlpha() const; //!< The alpha value.
+        HIPBLASLT_EXPORT const void* getBeta() const; //!< The beta value.
+        HIPBLASLT_EXPORT const void* getBias() const; //!< The bias input pointer.
+        HIPBLASLT_EXPORT const void* getScaleA() const; //!< The Scale A input pointer.
+        HIPBLASLT_EXPORT const void* getScaleB() const; //!< The Scale B input pointer.
+        HIPBLASLT_EXPORT const void* getScaleC() const; //!< The Scale C input pointer.
+        HIPBLASLT_EXPORT const void* getScaleD() const; //!< The Scale D input pointer.
+        HIPBLASLT_EXPORT const void* getScaleAux() const; //!< The Scale AUX input pointer.
+        HIPBLASLT_EXPORT const void*
+            getScaleAlphaVec() const; //!< The scaleAlpha vector input pointer.
+        HIPBLASLT_EXPORT const void* getAux() const; //!< The aux input pointer
+        HIPBLASLT_EXPORT const void* getAmaxD() const; //!< The AmaxD input pointer
+
+    private:
+        friend Gemm;
+        friend GroupedGemm;
+        class GemmInputsImpl;
+        std::unique_ptr<GemmInputsImpl> pimpl;
     };
 
     /*! \ingroup types_module
@@ -177,7 +402,7 @@ namespace hipblaslt_ext
         int8_t   beta[16]; //!< The beta value.
         // Epilogue inputs
         void* scaleA; //!< The scaleA input pointer.
-        void* scaleB; //!< The scaleA input pointer.
+        void* scaleB; //!< The scaleB input pointer.
         void* scaleC; //!< The scaleC input pointer.
         void* scaleD; //!< The scaleD input pointer.
         void* scaleAlphaVec; //!< The scaleAlpha vector input pointer.
@@ -191,7 +416,7 @@ namespace hipblaslt_ext
             strideE2; //!< The aux batch stride. Only works if mode is set to aux related epilogues.
         float act0; //!< The activation value 1. Some activations might use it.
         float act1; //!< The activation value 2.
-        int activationType; //!< The activation type.  Only works if mode is set to activation related epilogues.
+        int   activationType; //!< The activation type.  Only works if mode is set to activation related epilogues.
     } __attribute__((packed));
 
     /*! \ingroup types_module
@@ -200,14 +425,14 @@ namespace hipblaslt_ext
     class GemmInstance
     {
     public:
-        HIPBLASLT_EXPORT virtual ~GemmInstance();
+        HIPBLASLT_EXPORT virtual ~GemmInstance(){};
         HIPBLASLT_EXPORT               GemmInstance(const GemmInstance& rhs) = delete;
         HIPBLASLT_EXPORT GemmInstance& operator=(const GemmInstance& rhs)    = delete;
         HIPBLASLT_EXPORT               GemmInstance(GemmInstance&& rhs) noexcept;
         HIPBLASLT_EXPORT GemmInstance& operator=(GemmInstance&& rhs) noexcept;
 
         /*! \ingroup library_module
-        *  \brief Retrieve the possible algorithms
+        *  \brief Retrieve the possible algorithms (deprecated)
         *
         *  \details
         *  This function retrieves the possible algorithms for the matrix multiply
@@ -234,6 +459,34 @@ namespace hipblaslt_ext
                              const GemmPreference&                          pref,
                              std::vector<hipblasLtMatmulHeuristicResult_t>& heuristicResults);
 
+        /*! \ingroup library_module
+        *  \brief Retrieve the possible algorithms
+        *
+        *  \details
+        *  This function retrieves the possible algorithms for the matrix multiply
+        * operation hipblasLtMatmul() function with the given data and compute tpye.
+        * The output is placed in heuristicResult in the order of increasing
+        * estimated compute time.
+        *
+        *  @param[in]
+        *  requestedAlgoCount  number of requested algorithms.
+        *  @param[in]
+        *  pref hipblasLt extension preference for gemm problems.
+        *  @param[out]
+        *  heuristicResults    The algorithm heuristic vector.
+        *
+        *  \retval HIPBLAS_STATUS_SUCCESS           If query was successful. Inspect
+        * heuristicResults.size > 0, but may heuristicResults.size < requestedAlgoCount
+        * state for the status of the results. \retval HIPBLAS_STATUS_NOT_SUPPORTED
+        * If no heuristic function available for current configuration.
+        * \retval HIPBLAS_STATUS_INVALID_VALUE If no solution is found.
+        */
+        HIPBLASLT_EXPORT
+        hipblasStatus_t
+            algoGetHeuristic(const int                                      requestedAlgoCount,
+                             const GemmPreferenceV2&                        pref,
+                             std::vector<hipblasLtMatmulHeuristicResult_t>& heuristicResults);
+
         /*! \ingroup library_module
         *  \brief Check if the algorithm supports the problem. (For hipblaslt extension API)
         *
@@ -253,6 +506,29 @@ namespace hipblaslt_ext
         HIPBLASLT_EXPORT
         hipblasStatus_t isAlgoSupported(hipblasLtMatmulAlgo_t& algo, size_t& workspaceSizeInBytes);
 
+        /*! \ingroup library_module
+        *  \brief Check if the algorithm supports the problem. (deprecated)
+        *
+        *  \details
+        *  This function updates the problem saved inside the algorithm if the problem is
+        * supported. The required workspaceSizeInBytes is also returned.
+        *
+        *  @param[in]
+        *  algo The algorithm heuristic.
+        *  @param[in]
+        *  tuning The tuning parameters.
+        *  @param[out]
+        *  workspaceSizeInBytes Return the required workspace size.
+        *
+        *  \retval HIPBLAS_STATUS_SUCCESS           If query was successful. The problem is
+        * supported by the algorithm.
+        * results. \retval HIPBLAS_STATUS_INVALID_VALUE     The problem is not supported.
+        */
+        HIPBLASLT_EXPORT
+        hipblasStatus_t isAlgoSupported(hipblasLtMatmulAlgo_t& algo,
+                                        GemmTuning&            tuning,
+                                        size_t&                workspaceSizeInBytes);
+
         /*! \ingroup library_module
         *  \brief Check if the algorithm supports the problem. (For hipblaslt extension API)
         *
@@ -273,7 +549,7 @@ namespace hipblaslt_ext
         */
         HIPBLASLT_EXPORT
         hipblasStatus_t isAlgoSupported(hipblasLtMatmulAlgo_t& algo,
-                                        GemmTuning&            tuning,
+                                        GemmTuningV2&          tuning,
                                         size_t&                workspaceSizeInBytes);
 
         /*! \ingroup library_module
@@ -309,7 +585,7 @@ namespace hipblaslt_ext
                                    hipStream_t                  stream      = 0);
 
         /*! \ingroup library_module
-        *  \brief Create kernel arguments from a given hipblaslt_ext::GemmInstance.
+        *  \brief Create kernel arguments from a given hipblaslt_ext::GemmInstance. (deprecated)
         *
         *  \details
         *  This function creates kernel arguments from a given hipblaslt_ext::GemmInstance
@@ -345,22 +621,67 @@ namespace hipblaslt_ext
                                    bool                         useUserArgs = true,
                                    hipStream_t                  stream      = 0);
 
+        /*! \ingroup library_module
+        *  \brief Create kernel arguments from a given hipblaslt_ext::GemmInstance.
+        *
+        *  \details
+        *  This function creates kernel arguments from a given hipblaslt_ext::GemmInstance
+        *  then saves the arguments inside the instance.
+        *
+        *  @param[in]
+        *  algo                    Handle for matrix multiplication algorithm to be
+        * used. See hipblaslt.h::hipblasLtMatmulAlgo_t . When NULL, an implicit heuristics query
+        * with default search preferences will be performed to determine actual
+        * algorithm to use.
+        *  @param[in]
+        *  tuning                  Structure with user tuning parameters. Note that not every algo
+        * supports user tuning parameters. Will return HIPBLAS_STATUS_INVALID_VALUE if not supported.
+        * be 0).
+        *  @param[in]
+        *  workspace               Pointer to the workspace buffer allocated in the GPU
+        * memory. Pointer must be 16B aligned (that is, lowest 4 bits of address must
+        * be 0).
+        *  @param[in]
+        *  useUserArgs                Use user args, this does not affect vanilla gemm.
+        * (May be deprecated in the future)
+        *  @param[in]
+        *  stream                  The HIP stream where all the GPU work will be
+        * submitted. (May be deprecated in the future)
+        *
+        *  \retval HIPBLAS_STATUS_SUCCESS           If the operation completed
+        * successfully. \retval HIPBLAS_STATUS_INVALID_VALUE If the gemm_count = 0.
+        */
+        HIPBLASLT_EXPORT
+        hipblasStatus_t initialize(const hipblasLtMatmulAlgo_t& algo,
+                                   GemmTuningV2&                tuning,
+                                   void*                        workspace,
+                                   bool                         useUserArgs = true,
+                                   hipStream_t                  stream      = 0);
+
         /*! \ingroup library_module
         *  \brief Execute the kernel arguments stored inside the hipblaslt_ext::GemmInstance.
         *
         *  @param[in]
         *  stream                  The HIP stream where all the GPU work will be
+        *  @param[in]
+        *  start                   The HIP event which will record the start of the kernel
+        *  @param[in]
+        *  stop                    The HIP event which will record the end of the kernel
         * submitted.
         *
         *  \retval HIPBLAS_STATUS_SUCCESS           If the operation completed
         * successfully.
         */
         HIPBLASLT_EXPORT
-        hipblasStatus_t run(hipStream_t stream);
+        hipblasStatus_t
+            run(hipStream_t stream, hipEvent_t start = nullptr, hipEvent_t stop = nullptr);
 
         HIPBLASLT_EXPORT GemmType getGemmType();
         HIPBLASLT_EXPORT size_t   getGemmCount();
 
+        HIPBLASLT_EXPORT std::string getSolutionName();
+        HIPBLASLT_EXPORT std::string getKernelName();
+
     protected:
         /*! \ingroup library_module
         *  \brief Constructor of GemmInstance.
@@ -456,7 +777,7 @@ namespace hipblaslt_ext
         HIPBLASLT_EXPORT Gemm& operator=(Gemm&&) noexcept;
 
         /*! \ingroup library_module
-        *  \brief Sets the problem for a gemm problem.
+        *  \brief Sets the problem for a gemm problem. (Deprecated)
         *
         *  \details
         *  This function sets the problem with m, n, k, batch_count. It uses the problem type sets
@@ -497,6 +818,40 @@ namespace hipblaslt_ext
         *  from the constructor.
         *
         *  @param[in]
+        *  m,n,k                      The problem size.
+        *  @param[in]
+        *  batch_count                The batch count.
+        *  @param[in]
+        *  epilogue                   The class that controls the epilogue.
+        *  @param[in]
+        *  inputs                     The inputs of the problem.
+        *
+        *  \retval HIPBLAS_STATUS_SUCCESS           If the operation completed
+        * successfully. \retval HIPBLAS_STATUS_EXECUTION_FAILED  If HIP reported an
+        * execution error from the device. \retval HIPBLAS_STATUS_ARCH_MISMATCH     If
+        * the configured operation cannot be run using the selected device. \retval
+        * HIPBLAS_STATUS_NOT_SUPPORTED     If the current implementation on the
+        * selected device doesn't support the configured operation. \retval
+        * HIPBLAS_STATUS_INVALID_VALUE     If the parameters are unexpectedly NULL, in
+        * conflict or in an impossible configuration.
+        *  \retval HIBLAS_STATUS_NOT_INITIALIZED    If hipBLASLt handle has not been
+        * initialized.
+        */
+        HIPBLASLT_EXPORT hipblasStatus_t setProblem(int64_t         m,
+                                                    int64_t         n,
+                                                    int64_t         k,
+                                                    int64_t         batch_count,
+                                                    GemmEpilogueV2& epilogue,
+                                                    GemmInputsV2&   inputs);
+
+        /*! \ingroup library_module
+        *  \brief Sets the problem for a gemm problem. (Deprecated)
+        *
+        *  \details
+        *  This function sets the problem with m, n, k, batch_count. It uses the problem type sets
+        *  from the constructor.
+        *
+        *  @param[in]
         *  m,n,k                            The problem size.
         *  @param[in]
         *  batch_count                      The batch count.
@@ -538,6 +893,55 @@ namespace hipblaslt_ext
                                                     GemmInputs&      inputs,
                                                     GemmProblemType& problemtype);
 
+        /*! \ingroup library_module
+        *  \brief Sets the problem for a gemm problem.
+        *
+        *  \details
+        *  This function sets the problem with m, n, k, batch_count. It uses the problem type sets
+        *  from the constructor.
+        *
+        *  @param[in]
+        *  m,n,k                            The problem size.
+        *  @param[in]
+        *  batch_count                      The batch count.
+        *  @param[in]
+        *  lda,ldb,ldc,ldd                  The leading dimensions of the matrix.
+        *  @param[in]
+        *  strideA,strideB,strideC,strideD  The batch stride of the matrix.
+        *  @param[in]
+        *  epilogue                         The structure that controls the epilogue.
+        *  @param[in]
+        *  inputs                           The inputs of the problem.
+        *  @param[in]
+        *  problemtype                      The structure that sets the problem type of a gemm problem.
+        *
+        *  \retval HIPBLAS_STATUS_SUCCESS           If the operation completed
+        * successfully. \retval HIPBLAS_STATUS_EXECUTION_FAILED  If HIP reported an
+        * execution error from the device. \retval HIPBLAS_STATUS_ARCH_MISMATCH     If
+        * the configured operation cannot be run using the selected device. \retval
+        * HIPBLAS_STATUS_NOT_SUPPORTED     If the current implementation on the
+        * selected device doesn't support the configured operation. \retval
+        * HIPBLAS_STATUS_INVALID_VALUE     If the parameters are unexpectedly NULL, in
+        * conflict or in an impossible configuration.
+        *  \retval HIBLAS_STATUS_NOT_INITIALIZED    If hipBLASLt handle has not been
+        * initialized.
+        */
+        HIPBLASLT_EXPORT hipblasStatus_t setProblem(int64_t            m,
+                                                    int64_t            n,
+                                                    int64_t            k,
+                                                    int64_t            batch_count,
+                                                    int64_t            lda,
+                                                    int64_t            ldb,
+                                                    int64_t            ldc,
+                                                    int64_t            ldd,
+                                                    int64_t            strideA,
+                                                    int64_t            strideB,
+                                                    int64_t            strideC,
+                                                    int64_t            strideD,
+                                                    GemmEpilogueV2&    epilogue,
+                                                    GemmInputsV2&      inputs,
+                                                    GemmProblemTypeV2& problemtype);
+
         /*! \ingroup library_module
         *  \brief Sets the gemm problem from hipblasLt structures
         *
@@ -583,7 +987,8 @@ namespace hipblaslt_ext
                                                     void*                   D,
                                                     hipblasLtMatrixLayout_t matD);
 
-        HIPBLASLT_EXPORT GemmProblemType getProblemTypes();
+        HIPBLASLT_EXPORT GemmProblemType   getProblemTypes();
+        HIPBLASLT_EXPORT GemmProblemTypeV2 getProblemTypesV2();
     };
 
     /*! \ingroup types_module
@@ -665,7 +1070,7 @@ namespace hipblaslt_ext
                                               std::vector<hipblasLtMatrixLayout_t>& matD);
 
         /*! \ingroup library_module
-        *  \brief Sets the problem for a gemm problem.
+        *  \brief Sets the problem for a gemm problem. (Deprecated)
         *
         *  \details
         *  This function sets the problem with m, n, k, batch_count. It uses the problem type sets
@@ -706,6 +1111,40 @@ namespace hipblaslt_ext
         *  from the constructor.
         *
         *  @param[in]
+        *  m,n,k                      The problem size in vector.
+        *  @param[in]
+        *  batch_count                The batch count in vector.
+        *  @param[in]
+        *  epilogue                   The structure in vector that controls the epilogue.
+        *  @param[in]
+        *  inputs                     The inputs in vector of the problem.
+        *
+        *  \retval HIPBLAS_STATUS_SUCCESS           If the operation completed
+        * successfully. \retval HIPBLAS_STATUS_EXECUTION_FAILED  If HIP reported an
+        * execution error from the device. \retval HIPBLAS_STATUS_ARCH_MISMATCH     If
+        * the configured operation cannot be run using the selected device. \retval
+        * HIPBLAS_STATUS_NOT_SUPPORTED     If the current implementation on the
+        * selected device doesn't support the configured operation. \retval
+        * HIPBLAS_STATUS_INVALID_VALUE     If the parameters are unexpectedly NULL, in
+        * conflict or in an impossible configuration.
+        *  \retval HIBLAS_STATUS_NOT_INITIALIZED    If hipBLASLt handle has not been
+        * initialized.
+        */
+        HIPBLASLT_EXPORT hipblasStatus_t setProblem(std::vector<int64_t>&        m,
+                                                    std::vector<int64_t>&        n,
+                                                    std::vector<int64_t>&        k,
+                                                    std::vector<int64_t>&        batch_count,
+                                                    std::vector<GemmEpilogueV2>& epilogue,
+                                                    std::vector<GemmInputsV2>&   inputs);
+
+        /*! \ingroup library_module
+        *  \brief Sets the problem for a gemm problem. (Deprecated)
+        *
+        *  \details
+        *  This function sets the problem with m, n, k, batch_count. It uses the problem type sets
+        *  from the constructor.
+        *
+        *  @param[in]
         *  m,n,k                            The problem size in vector.
         *  @param[in]
         *  batch_count                      The batch count in vector.
@@ -748,6 +1187,56 @@ namespace hipblaslt_ext
                                                     std::vector<GemmInputs>&   inputs,
                                                     GemmProblemType&           problemtype);
 
+        /*! \ingroup library_module
+        *  \brief Sets the problem for a gemm problem.
+        *
+        *  \details
+        *  This function sets the problem with m, n, k, batch_count. It uses the problem type sets
+        *  from the constructor.
+        *
+        *  @param[in]
+        *  m,n,k                            The problem size in vector.
+        *  @param[in]
+        *  batch_count                      The batch count in vector.
+        *  @param[in]
+        *  lda,ldb,ldc,ldd                  The leading dimensions in vector of the matrix.
+        *  @param[in]
+        *  strideA,strideB,strideC,strideD  The batch stride in vector of the matrix.
+        *  @param[in]
+        *  epilogue                         The structure in vector that controls the epilogue.
+        *  @param[in]
+        *  inputs                           The inputs in vector of the problem.
+        *  @param[in]
+        *  problemtype                      The structure that sets the problem type
+        * of a gemm problem.
+        *
+        *  \retval HIPBLAS_STATUS_SUCCESS           If the operation completed
+        * successfully. \retval HIPBLAS_STATUS_EXECUTION_FAILED  If HIP reported an
+        * execution error from the device. \retval HIPBLAS_STATUS_ARCH_MISMATCH     If
+        * the configured operation cannot be run using the selected device. \retval
+        * HIPBLAS_STATUS_NOT_SUPPORTED     If the current implementation on the
+        * selected device doesn't support the configured operation. \retval
+        * HIPBLAS_STATUS_INVALID_VALUE     If the parameters are unexpectedly NULL, in
+        * conflict or in an impossible configuration.
+        *  \retval HIBLAS_STATUS_NOT_INITIALIZED    If hipBLASLt handle has not been
+        * initialized.
+        */
+        HIPBLASLT_EXPORT hipblasStatus_t setProblem(std::vector<int64_t>&        m,
+                                                    std::vector<int64_t>&        n,
+                                                    std::vector<int64_t>&        k,
+                                                    std::vector<int64_t>&        batch_count,
+                                                    std::vector<int64_t>&        lda,
+                                                    std::vector<int64_t>&        ldb,
+                                                    std::vector<int64_t>&        ldc,
+                                                    std::vector<int64_t>&        ldd,
+                                                    std::vector<int64_t>&        strideA,
+                                                    std::vector<int64_t>&        strideB,
+                                                    std::vector<int64_t>&        strideC,
+                                                    std::vector<int64_t>&        strideD,
+                                                    std::vector<GemmEpilogueV2>& epilogue,
+                                                    std::vector<GemmInputsV2>&   inputs,
+                                                    GemmProblemTypeV2&           problemtype);
+
         /*! \ingroup library_module
         *  \brief Sets the grouped gemm problem from hipblasLt structures
         *
@@ -795,6 +1284,7 @@ namespace hipblaslt_ext
                        std::vector<hipblasLtMatrixLayout_t>& matD);
 
         HIPBLASLT_EXPORT std::vector<GemmProblemType> getProblemTypes();
+        HIPBLASLT_EXPORT std::vector<GemmProblemTypeV2> getProblemTypesV2();
 
         /*! \ingroup library_module
         *  \brief A helper function to initialize DeviceUserArguments using the set problem(s)
@@ -840,7 +1330,8 @@ namespace hipblaslt_ext
      *  This function retrieves the possible algorithms for the matrix multiply
      * operation hipblasLtMatmul() function with the given data and compute tpye.
      * The output is placed in heuristicResults in the order of increasing
-     * estimated compute time.
+     * estimated compute time. It should use matmulIsAlgoSupported() to check if
+     * the algorithm support the problem before execute hipblasLtMatmul().
      *
      *  @param[in]
      *  handle                  Pointer to the allocated hipBLASLt handle for the
@@ -928,7 +1419,9 @@ namespace hipblaslt_ext
      *  This function retrieves the possible algorithms for the matrix multiply
      * operation hipblasLtMatmul() function with the given index.
      * The output is placed in heuristicResult in the order of increasing
-     * estimated compute time.
+     * estimated compute time. A recorded solution index cannot be used across
+     * different verison of library. It should use matmulIsAlgoSupported() to
+     * check if the algorithm support the problem before execute hipblasLtMatmul().
      *
      *  @param[in]
      *  handle                  Pointer to the allocated hipBLASLt handle for the
@@ -942,7 +1435,7 @@ namespace hipblaslt_ext
      * heuristicResults.size() > 0.state for the status of the
      * results. \retval HIPBLAS_STATUS_NOT_SUPPORTED     If no heuristic function
      * available for current configuration. \retval HIPBLAS_STATUS_INVALID_VALUE If
-     * no solution is found.
+     * query indexes are all out of bound of solution map.
      */
     HIPBLASLT_EXPORT
     hipblasStatus_t
@@ -1003,4 +1496,12 @@ namespace hipblaslt_ext
      */
     HIPBLASLT_EXPORT
     hipblasStatus_t copyMatmul(hipblasLtMatmulDesc_t src, hipblasLtMatmulDesc_t dst);
+
+    HIPBLASLT_EXPORT
+    int matmulIsTuned(hipblasLtHandle_t       handle,
+                      hipblasLtMatmulDesc_t   matmulDesc,
+                      hipblasLtMatrixLayout_t Adesc,
+                      hipblasLtMatrixLayout_t Bdesc,
+                      hipblasLtMatrixLayout_t Cdesc,
+                      hipblasLtMatrixLayout_t Ddesc);
 } // End of namespace hipblasltext
