@@ -46,15 +46,32 @@
 
 #include "hipblaslt/hipblaslt-export.h"
 #include "hipblaslt/hipblaslt-version.h"
+#ifndef LEGACY_HIPBLAS_DIRECT
+#include <hipblas-common/hipblas-common.h>
+#else
 #include <hipblas/hipblas.h>
+#endif
 
 #include <memory>
+#include <regex>
 #include <vector>
 
 #include <hip/hip_bfloat16.h>
 #include <hip/hip_complex.h>
 #include <hip/hip_runtime.h>
 #include <hip/hip_runtime_api.h>
+#include <hip/hip_version.h>
+
+#if HIP_VERSION_MAJOR == 6 && HIP_VERSION_MINOR > 2 \
+    && HIP_VERSION_PATCH > 42130 //tmp before gfx94 use hip f8 header
+#define ROCM_USE_FLOAT8 1
+#else
+#undef ROCM_USE_FLOAT8
+#endif
+
+#if defined(__HIPCC__)
+#include <hip/hip_fp8.h>
+#endif
 
 #if defined(__HIP_PLATFORM_AMD__)
 #include "hipblaslt-types.h"
@@ -157,7 +174,12 @@ typedef enum {
   HIPBLASLT_MATMUL_DESC_EPILOGUE_AUX_LD = 11,           /**<The leading dimension of the epilogue auxiliary buffer pointer in the device memory. Data Type:int64_t */
   HIPBLASLT_MATMUL_DESC_EPILOGUE_AUX_BATCH_STRIDE = 12, /**<The batch stride of the epilogue auxiliary buffer pointer in the device memory. Data Type:int64_t */
   HIPBLASLT_MATMUL_DESC_POINTER_MODE = 13,              /**<Specifies alpha and beta are passed by reference, whether they are scalars on the host or on the device, or device vectors. Default value is: HIPBLASLT_POINTER_MODE_HOST (i.e., on the host). Data Type: int32_t based on hipblasLtPointerMode_t*/
-  HIPBLASLT_MATMUL_DESC_MAX = 101
+  HIPBLASLT_MATMUL_DESC_AMAX_D_POINTER = 14,           /**<Device pointer to the memory location that on completion will be set to the maximum of absolute values in the output matrix. Data Type:void* /const void* */
+  HIPBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_A_EXT = 100,     /**<Compute input A types. Defines the data type used for the input A of matrix multiply. */
+  HIPBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_B_EXT,           /**<Compute input B types. Defines the data type used for the input B of matrix multiply. */
+  HIPBLASLT_MATMUL_DESC_A_SCALE_POINTER_VEC_EXT,        /**<Equivalent to HIPBLASLT_MATMUL_DESC_A_SCALE_POINTER but in vector. Default value: NULL Type: void* /const void* */
+  HIPBLASLT_MATMUL_DESC_B_SCALE_POINTER_VEC_EXT,        /**<Equivalent to HIPBLASLT_MATMUL_DESC_B_SCALE_POINTER but in vector. Default value: NULL Type: void* /const void* */
+  HIPBLASLT_MATMUL_DESC_MAX,
 } hipblasLtMatmulDescAttributes_t;
 
 /*! \ingroup types_module
@@ -303,11 +325,15 @@ typedef struct _hipblasLtMatmulAlgo_t{
 } hipblasLtMatmulAlgo_t;
 
 /*! \ingroup types_module
+ *  \struct hipblasLtMatmulHeuristicResult_t
  *  \brief Description of the matrix multiplication algorithm
  *
  *  \details
  *  This is a descriptor that holds the configured matrix multiplication algorithm descriptor and its runtime properties.
  *  This structure can be trivially serialized and later restored for use with the same version of hipBLASLt library to save on selecting the right configuration again.
+ *  @param algo \ref hipblasLtMatmulAlgo_t struct
+ *  @param workspaceSize Actual size of workspace memory required
+ *  @param state Result status. Other fields are valid only if, after call to hipblasLtMatmulAlgoGetHeuristic(), this member is set to HIPBLAS_STATUS_SUCCESS
  */
 typedef struct _hipblasLtMatmulHeuristicResult_t{
   hipblasLtMatmulAlgo_t algo;                      /**<Algo struct*/
@@ -339,7 +365,7 @@ hipblasStatus_t hipblasLtGetArchName(char** archName);
  * opaque structure holding the hipBLASLt library context. It allocates light
  * hardware resources on the host and device, and must be called prior to making
  * any other hipBLASLt library calls. The hipBLASLt library context is tied to
- * the current CUDA device. To use the library on multiple devices, one
+ * the current ROCm device. To use the library on multiple devices, one
  * hipBLASLt handle should be created for each device.
  *
  *  @param[out]
