@@ -26,6 +26,7 @@
 
 #include "hipblaslt.h"
 #include "exceptions.hpp"
+#include "hipblaslt-ext-op.h"
 #include "hipblaslt_internal.hpp"
 
 #include <hip/hip_runtime_api.h>
@@ -127,12 +128,6 @@ extern "C" {
 hipblasStatus_t hipblasLtCreate(hipblasLtHandle_t* handle)
 try
 {
-    // TODO: Synchronizer size pass into predicate SynchronizerSizeCheck
-    // 1K just for small size now, need to cal corner case if support all situations
-    void* d_Synchronizer = nullptr;
-    CHECK_HIP_ERROR(hipMalloc(&d_Synchronizer, 16 * 1024 * sizeof(int)));
-    CHECK_HIP_ERROR(hipMemset(d_Synchronizer, 0, sizeof(int) * 16 * 1024));
-
     // Check if handle is valid
     if(handle == nullptr)
     {
@@ -142,6 +137,11 @@ try
     int             deviceId;
     hipError_t      err;
     hipblasStatus_t retval = HIPBLAS_STATUS_SUCCESS;
+    // TODO: Synchronizer size pass into predicate SynchronizerSizeCheck
+    // 1K just for small size now, need to cal corner case if support all situations
+    void* d_Synchronizer = nullptr;
+    CHECK_HIP_ERROR(hipMalloc(&d_Synchronizer, 16 * 40960 * sizeof(int)));
+    CHECK_HIP_ERROR(hipMemset(d_Synchronizer, 0, sizeof(int) * 16 * 40960));
 
     err = hipGetDevice(&deviceId);
     if(err == hipSuccess)
@@ -399,22 +399,26 @@ hipblasStatus_t hipblasLtMatmul(hipblasLtHandle_t            handle,
                                 hipStream_t                  stream)
 try
 {
-    return RocBlasLtStatusToHIPStatus(rocblaslt_matmul((rocblaslt_handle)handle,
-                                                       (rocblaslt_matmul_desc)matmul_descr,
-                                                       alpha,
-                                                       A,
-                                                       (rocblaslt_matrix_layout)matA,
-                                                       B,
-                                                       (rocblaslt_matrix_layout)matB,
-                                                       beta,
-                                                       C,
-                                                       (rocblaslt_matrix_layout)matC,
-                                                       D,
-                                                       (rocblaslt_matrix_layout)matD,
-                                                       (const rocblaslt_matmul_algo*)algo,
-                                                       workspace,
-                                                       workspaceSizeInBytes,
-                                                       stream));
+    hipblasStatus_t return_status = HIPBLAS_STATUS_SUCCESS;
+
+    return_status = RocBlasLtStatusToHIPStatus(rocblaslt_matmul((rocblaslt_handle)handle,
+                                                                (rocblaslt_matmul_desc)matmul_descr,
+                                                                alpha,
+                                                                A,
+                                                                (rocblaslt_matrix_layout)matA,
+                                                                B,
+                                                                (rocblaslt_matrix_layout)matB,
+                                                                beta,
+                                                                C,
+                                                                (rocblaslt_matrix_layout)matC,
+                                                                D,
+                                                                (rocblaslt_matrix_layout)matD,
+                                                                (const rocblaslt_matmul_algo*)algo,
+                                                                workspace,
+                                                                workspaceSizeInBytes,
+                                                                stream));
+
+    return return_status;
 }
 catch(...)
 {
