@@ -33,7 +33,11 @@
 #define _ROCBLASLT_TYPES_H_
 
 #include <hip/hip_bfloat16.h>
+#ifndef LEGACY_HIPBLAS_DIRECT
+#include <hipblas-common/hipblas-common.h>
+#else
 #include <hipblas/hipblas.h>
+#endif
 #include <hipblaslt/hipblaslt.h>
 #include <memory>
 #include <stddef.h>
@@ -178,7 +182,9 @@ typedef enum rocblaslt_epilogue_
 typedef enum rocblaslt_pointer_mode_
 {
     rocblaslt_pointer_mode_host   = 0, /**< scalar pointers are in host memory. */
-    rocblaslt_pointer_mode_device = 1 /**< scalar pointers are in device memory. */
+    rocblaslt_pointer_mode_device = 1, /**< scalar pointers are in device memory. */
+    rocblaslt_pointer_mode_alpha_device_vector_beta_host
+    = 4 /** alpha pointer targets a device memory vector of length equal to the number of rows of matrix D, and beta is a single value in host memory. */
 } rocblaslt_pointer_mode;
 
 /*! \ingroup types_module
@@ -248,12 +254,31 @@ typedef enum rocblaslt_status_
  */
 typedef enum rocblaslt_compute_type_
 {
-    rocblaslt_compute_f32 = 2, /**< 32-bit floating-point precision. */
+    rocblaslt_compute_f16          = 0, /**< 16-bit floating-point precision. */
+    rocblaslt_compute_f16_pedantic = 1, /**< compute will be exactly 16-bit precision */
+    rocblaslt_compute_f32          = 2, /**< 32-bit floating-point precision. */
+    rocblaslt_compute_f32_pedantic = 3, /**< compute will be exactly 32-bit precision */
     rocblaslt_compute_f32_fast_f16
     = 4, /**< F16 compute for 16-bit input and 32-bit output matrices */
+    rocblaslt_compute_f32_fast_bf16
+    = 5, /**< BF16 compute for 16-bit input and 32-bit output matrices */
     rocblaslt_compute_f32_fast_xf32 = 6, /**< XF32 compute for 32-bit input and output matrices */
     rocblaslt_compute_f64           = 7, /**< 64-bit floating-point precision. */
+    rocblaslt_compute_f64_pedantic  = 8, /**< compute will be exactly 64-bit precision */
     rocblaslt_compute_i32           = 9, /**< 32-bit integer precision. */
+    rocblaslt_compute_i32_pedantic  = 10, /**< compute will be exactly 32-bit integer precision */
+    rocblaslt_compute_f32_fast_f8_fnuz  = 100, /**< 32-bit input can use fp8 compute */
+    rocblaslt_compute_f32_fast_bf8_fnuz = 101, /**< 32-bit input can use bf8 compute */
+    rocblaslt_compute_f32_fast_f8bf8_fnuz
+    = 102, /**< 32-bit input can use fp8 for A and bf8 for B compute */
+    rocblaslt_compute_f32_fast_bf8f8_fnuz
+    = 103, /**< 32-bit input can use bf8 for A and fp8 for B compute */
+    rocblaslt_compute_f32_fast_f8_ocp  = 104, /**< 32-bit input can use fp8 compute */
+    rocblaslt_compute_f32_fast_bf8_ocp = 105, /**< 32-bit input can use bf8 compute */
+    rocblaslt_compute_f32_fast_f8bf8_ocp
+    = 106, /**< 32-bit input can use fp8 for A and bf8 for B compute */
+    rocblaslt_compute_f32_fast_bf8f8_ocp
+    = 107, /**< 32-bit input can use bf8 for A and fp8 for B compute */
 } rocblaslt_compute_type;
 
 /*! \ingroup types_module
@@ -317,7 +342,12 @@ typedef enum rocblaslt_matmul_desc_attributes_
     ROCBLASLT_MATMUL_DESC_EPILOGUE_AUX_LD            = 11,
     ROCBLASLT_MATMUL_DESC_EPILOGUE_AUX_BATCH_STRIDE  = 12,
     ROCBLASLT_MATMUL_DESC_POINTER_MODE               = 13,
-    ROCBLASLT_MATMUL_DESC_MAX                        = 101
+    ROCBLASLT_MATMUL_DESC_AMAX_D_POINTER             = 14,
+    ROCBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_A_EXT   = 100,
+    ROCBLASLT_MATMUL_DESC_COMPUTE_INPUT_TYPE_B_EXT,
+    ROCBLASLT_MATMUL_DESC_A_SCALE_POINTER_VEC_EXT,
+    ROCBLASLT_MATMUL_DESC_B_SCALE_POINTER_VEC_EXT,
+    ROCBLASLT_MATMUL_DESC_MAX,
 } rocblaslt_matmul_desc_attributes;
 
 /*! \ingroup types_module
@@ -397,6 +427,18 @@ namespace rocblaslt
         rocblaslt_compute_type type_compute;
     };
 
+    class RocGemmProblemTypeV2
+    {
+    public:
+        hipblasOperation_t     op_a;
+        hipblasOperation_t     op_b;
+        hipDataType            type_a;
+        hipDataType            type_b;
+        hipDataType            type_c;
+        hipDataType            type_d;
+        rocblaslt_compute_type type_compute;
+    };
+
     struct RocGemmEpilogue
     {
         rocblaslt_epilogue mode           = ROCBLASLT_EPILOGUE_DEFAULT;
@@ -405,12 +447,30 @@ namespace rocblaslt
         int                aux_stride     = 0;
     };
 
+    class RocGemmEpilogueV2
+    {
+    public:
+        rocblaslt_epilogue mode           = ROCBLASLT_EPILOGUE_DEFAULT;
+        hipDataType        bias_data_type = HIPBLASLT_DATATYPE_INVALID;
+        int                aux_ld         = 0;
+        int                aux_stride     = 0;
+        int                scaling_a_type = 0;
+        int                scaling_b_type = 0;
+    };
+
     struct RocTuning
     {
         uint8_t gsu = 0;
         uint8_t wgm = 0;
     };
 
+    class RocTuningV2
+    {
+    public:
+        uint16_t gsu = 0;
+        int16_t  wgm = 0;
+    };
+
     struct RocGemmInputs
     {
         void* a     = nullptr;
@@ -430,6 +490,26 @@ namespace rocblaslt
         void* aux           = nullptr;
     };
 
+    struct RocGemmInputsV2
+    {
+        void* a     = nullptr;
+        void* b     = nullptr;
+        void* c     = nullptr;
+        void* d     = nullptr;
+        void* alpha = nullptr;
+        void* beta  = nullptr;
+        // Epilogue inputs
+        void* bias          = nullptr;
+        void* scaleA        = nullptr;
+        void* scaleB        = nullptr;
+        void* scaleC        = nullptr;
+        void* scaleD        = nullptr;
+        void* scaleE        = nullptr;
+        void* scaleAlphaVec = nullptr;
+        void* aux           = nullptr;
+        void* amaxD         = nullptr;
+    };
+
     class RocGemm
     {
     public:
