@@ -124,6 +124,10 @@ inline rocblaslt_status validateMatmulDescrArgs(rocblaslt_handle       handle,
         if(compute_type != rocblaslt_compute_i32)
             return rocblaslt_status_invalid_value;
         break;
+    case HIP_R_8I:
+        if(compute_type != rocblaslt_compute_i32)
+            return rocblaslt_status_invalid_value;
+        break;
     default:
         return rocblaslt_status_invalid_value;
         break;
@@ -135,23 +139,25 @@ inline rocblaslt_status validateMatmulDescrArgs(rocblaslt_handle       handle,
 /*******************************************************************************
  * Validate Matmul Arguments
  ******************************************************************************/
-inline rocblaslt_status validateMatmulArgs(int64_t     m,
-                                           int64_t     n,
-                                           int64_t     k,
-                                           const void* alpha,
-                                           const void* a,
-                                           const void* b,
-                                           const void* beta,
-                                           const void* c,
-                                           const void* d,
-                                           int         num_batches_a  = 1,
-                                           int         num_batches_b  = 1,
-                                           int         num_batches_c  = 1,
-                                           int         num_batches_d  = 1,
-                                           int64_t     batch_stride_a = 0,
-                                           int64_t     batch_stride_b = 0,
-                                           int64_t     batch_stride_c = 0,
-                                           int64_t     batch_stride_d = 0)
+inline rocblaslt_status validateMatmulArgs(int64_t                       m,
+                                           int64_t                       n,
+                                           int64_t                       k,
+                                           const void*                   alpha,
+                                           const void*                   a,
+                                           const void*                   b,
+                                           const void*                   beta,
+                                           const void*                   c,
+                                           const void*                   d,
+                                           int                           num_batches_a  = 1,
+                                           int                           num_batches_b  = 1,
+                                           int                           num_batches_c  = 1,
+                                           int                           num_batches_d  = 1,
+                                           int64_t                       batch_stride_a = 0,
+                                           int64_t                       batch_stride_b = 0,
+                                           int64_t                       batch_stride_c = 0,
+                                           int64_t                       batch_stride_d = 0,
+                                           const rocblaslt_pointer_mode& pointermode
+                                           = rocblaslt_pointer_mode_host)
 {
     // sizes must not be negative
     if(batch_stride_a < 0 || batch_stride_b < 0 || batch_stride_c < 0 || batch_stride_d < 0)
@@ -179,8 +185,10 @@ inline rocblaslt_status validateMatmulArgs(int64_t     m,
     if(!beta)
         return rocblaslt_status_invalid_pointer;
 
+    // Update for the valid case: ((alpha_in_host && alpha=0) && (A=NULL || B=NULL))
+    bool alpha_A_B_violation = (!alpha || ((pointermode || (*((float*)alpha))) && (!a || !b)));
     // pointers must be valid
-    if(n && ((k && (!a || !b || !alpha)) || !c || !d))
+    if(n && ((k && alpha_A_B_violation) || !c || !d))
         return rocblaslt_status_invalid_pointer;
 
     return rocblaslt_status_continue;
@@ -197,6 +205,8 @@ inline rocblaslt_status rocblaslt_epilogue_valid_args(const rocblaslt_epilogue&
                                                       const void*               original_bias,
                                                       const void*  original_scaleAlphaVec,
                                                       const void*  alpha,
+                                                      const bool   isScaleAVec,
+                                                      const bool   isScaleBVec,
                                                       void*&       E,
                                                       int64_t&     lde,
                                                       int64_t&     batch_stride_e,
@@ -211,10 +221,13 @@ inline rocblaslt_status rocblaslt_epilogue_valid_args(const rocblaslt_epilogue&
     bias_type = original_bias_type; // == HIPBLASLT_DATATYPE_INVALID ? d_type : original_bias_type;
     gradient  = is_grad_enabled(epilogue);
 
+    bias = nullptr;
     if(is_bias_enabled(epilogue))
+    {
+        if(original_bias == nullptr)
+            status = rocblaslt_status_invalid_pointer;
         bias = (void*)original_bias;
-    else
-        bias = nullptr;
+    }
 
     if(original_scaleAlphaVec)
         scaleAlphaVec = (void*)original_scaleAlphaVec; //pointer mode alpha vector pass by alpha
@@ -233,6 +246,11 @@ inline rocblaslt_status rocblaslt_epilogue_valid_args(const rocblaslt_epilogue&
     batch_stride_e = original_stride_e > 0 ? original_stride_e : original_lde * num_cols_e;
     if(E != nullptr && ((lde < num_rows_e) || (batch_stride_e < (num_cols_e * num_rows_e))))
         status = rocblaslt_status_invalid_value;
+    if(isScaleAVec != isScaleBVec)
+    {
+        log_error(__func__, "Scale A and Scale B must be both scalar or vector.");
+        status = rocblaslt_status_invalid_value;
+    }
     return status;
 }
 
@@ -324,7 +342,8 @@ inline rocblaslt_status rocblaslt_matmul_valid_args(const rocblaslt_matmul_desc
                                      batch_stride_a,
                                      batch_stride_b,
                                      batch_stride_c,
-                                     batch_stride_d);
+                                     batch_stride_d, 
+                                     matmul_descr->pointermode);
 
     if(status == rocblaslt_status_continue)
     {
@@ -332,8 +351,10 @@ inline rocblaslt_status rocblaslt_matmul_valid_args(const rocblaslt_matmul_desc
              && matD->type == HIP_R_32F)
            && compute_type == rocblaslt_compute_f32_fast_xf32)
             status = rocblaslt_status_not_implemented;
-        if(!(matA->type == HIP_R_8I && matB->type == HIP_R_8I && matC->type == HIP_R_32I
-             && matD->type == HIP_R_32I)
+        if(!((matA->type == HIP_R_8I && matB->type == HIP_R_8I && matC->type == HIP_R_32I
+              && matD->type == HIP_R_32I)
+             || (matA->type == HIP_R_8I && matB->type == HIP_R_8I && matC->type == HIP_R_8I
+                 && matD->type == HIP_R_8I))
            && compute_type == rocblaslt_compute_i32)
             status = rocblaslt_status_not_implemented;
     }
@@ -350,6 +371,8 @@ inline rocblaslt_status rocblaslt_matmul_valid_args(const rocblaslt_matmul_desc
                                                matmul_descr->bias,
                                                alphaVecPtr,
                                                alpha,
+                                               matmul_descr->isScaleAVec,
+                                               matmul_descr->isScaleBVec,
                                                E,
                                                lde,
                                                batch_stride_e,
