@@ -42,6 +42,7 @@
 #include "handle.h"
 //#include "tuple_helper.hpp"
 #include "utility.hpp"
+#include <Tensile/Contractions.hpp>
 #include <Tensile/DataTypes.hpp>
 #include <atomic>
 
@@ -124,8 +125,11 @@ struct RocblasltContractionProblem
     const void*        scaleD;
     const void*        scaleE;
     const void*        scaleAlphaVec;
+    bool               isScaleAVec;
+    bool               isScaleBVec;
     hipDataType        bias_type;
     rocblaslt_epilogue epilogue;
+    void*              amaxD;
     void*              workspace;
     size_t             workspaceSize;
 
@@ -177,8 +181,11 @@ struct RocblasltContractionProblem
                                 const void*            scaleD,
                                 const void*            scaleE,
                                 const void*            scaleAlphaVec,
+                                bool                   isScaleAVec,
+                                bool                   isScaleBVec,
                                 hipDataType            bias_type,
                                 rocblaslt_epilogue     epilogue,
+                                void*                  amaxD,
                                 void*                  workspace,
                                 size_t                 workspaceSize,
                                 hipStream_t            stream,
@@ -231,8 +238,11 @@ struct RocblasltContractionProblem
         , scaleD(scaleD)
         , scaleE(scaleE)
         , scaleAlphaVec(scaleAlphaVec)
+        , isScaleAVec(isScaleAVec)
+        , isScaleBVec(isScaleBVec)
         , bias_type(bias_type)
         , epilogue(epilogue)
+        , amaxD(amaxD)
         , workspace(workspace)
         , workspaceSize(workspaceSize)
         , stream(stream)
@@ -243,18 +253,34 @@ struct RocblasltContractionProblem
             // FIXME: Currently the default bias_type is set to match the yamls' configuration, should add the default type when the yamls are fixed.
             if(this->compute_type == rocblaslt_compute_i32)
             {
-                this->bias_type = HIP_R_32F;
+                this->bias_type = HIP_R_32I;
             }
             else if(this->compute_type == rocblaslt_compute_f32_fast_xf32)
             {
                 this->bias_type = HIP_R_32F;
             }
             else if((this->a_type == HIP_R_8F_E4M3_FNUZ || this->a_type == HIP_R_8F_E5M2_FNUZ)
-                    && (this->b_type == HIP_R_8F_E4M3_FNUZ || this->b_type == HIP_R_8F_E5M2_FNUZ)
-                    && (this->d_type == HIP_R_8F_E4M3_FNUZ || this->d_type == HIP_R_8F_E5M2_FNUZ))
+                    && (this->b_type == HIP_R_8F_E4M3_FNUZ || this->b_type == HIP_R_8F_E5M2_FNUZ))
             {
-                this->bias_type = HIP_R_32F;
+                if(this->d_type == HIP_R_32F || this->d_type == HIP_R_16BF)
+                    this->bias_type = HIP_R_16BF;
+                else if(this->d_type == HIP_R_16F)
+                    this->bias_type = HIP_R_16F;
+                else //more default cases once support C != D
+                    this->bias_type = HIP_R_16F;
             }
+#ifdef ROCM_USE_FLOAT8
+            else if((this->a_type == HIP_R_8F_E4M3 || this->a_type == HIP_R_8F_E5M2)
+                    && (this->b_type == HIP_R_8F_E4M3 || this->b_type == HIP_R_8F_E5M2))
+            {
+                if(this->d_type == HIP_R_32F || this->d_type == HIP_R_16BF)
+                    this->bias_type = HIP_R_16BF;
+                else if(this->d_type == HIP_R_16F)
+                    this->bias_type = HIP_R_16F;
+                else //more default cases once support C != D
+                    this->bias_type = HIP_R_16F;
+            }
+#endif
             else
             {
                 this->bias_type = this->d_type;
@@ -291,10 +317,11 @@ rocblaslt_status groupedGemmCreate(std::vector<RocblasltContractionProblem>& pro
                                    std::shared_ptr<void>&                    gemmData,
                                    size_t&                                   gemmCount);
 
+template <typename Tuning>
 rocblaslt_status makeArgument(rocblaslt_handle             handle,
                               const rocblaslt::RocGemmType gemmType,
                               const rocblaslt_matmul_algo& algo,
-                              const rocblaslt::RocTuning*  tuning,
+                              const Tuning*                tuning,
                               void*                        workspace,
                               bool                         useUserArgs,
                               hipStream_t                  stream,
@@ -304,7 +331,9 @@ rocblaslt_status makeArgument(rocblaslt_handle             handle,
 rocblaslt_status runKernelFromInvocation(rocblaslt_handle       handle,
                                          rocblaslt::RocGemmType gemmType,
                                          std::shared_ptr<void>  gemmData,
-                                         hipStream_t            stream);
+                                         hipStream_t            stream,
+                                         hipEvent_t             start = nullptr,
+                                         hipEvent_t             stop  = nullptr);
 
 rocblaslt_status getDeviceUserArgumentsValuesFromContractionProblem(rocblaslt_handle       handle,
                                                                     rocblaslt::RocGemmType gemmType,
@@ -326,6 +355,14 @@ rocblaslt_status runKernelFromDeviceUserArguments(rocblaslt_handle             h
                                                   void*                        workspace,
                                                   hipStream_t                  stream);
 
+std::string getKernelNameFromData(rocblaslt_handle             handle,
+                                  const rocblaslt::RocGemmType gemmType,
+                                  std::shared_ptr<void>        gemmData);
+
+std::string getSolutionNameFromData(rocblaslt_handle             handle,
+                                    const rocblaslt::RocGemmType gemmType,
+                                    std::shared_ptr<void>        gemmData);
+
 std::string getKernelNameFromAlgoIndex(rocblaslt_handle handle, const rocblaslt_matmul_algo& algo);
 
 std::string getSolutionNameFromAlgoIndex(rocblaslt_handle             handle,
@@ -356,6 +393,12 @@ rocblaslt_status getAllSolutions(std::vector<RocblasltContractionProblem>&
                                  std::vector<rocblaslt_matmul_heuristic_result>& heuristicResults,
                                  size_t                                          maxWorkSpaceBytes);
 
+rocblaslt_status getAllSolutions(std::shared_ptr<void>                           gemmData,
+                                 rocblaslt_handle                                handle,
+                                 rocblaslt::RocGemmType                          gemmType,
+                                 std::vector<rocblaslt_matmul_heuristic_result>& heuristicResults,
+                                 size_t                                          maxWorkSpaceBytes);
+
 rocblaslt_status
     getSolutionsFromIndex(rocblaslt_handle                                handle,
                           std::vector<int>&                               solutionIndex,
@@ -368,13 +411,21 @@ rocblaslt_status isSolutionSupported(rocblaslt_handle             handle,
                                      rocblaslt_matmul_algo*       algo,
                                      size_t*                      workspaceSizeInBytes);
 
+template <typename Tuning>
 rocblaslt_status isSolutionSupported(rocblaslt_handle              handle,
                                      const rocblaslt::RocGemmType& gemmType,
                                      std::shared_ptr<void>         gemmData,
                                      rocblaslt_matmul_algo&        algo,
-                                     const rocblaslt::RocTuning*   tuning,
+                                     const Tuning*                 tuning,
                                      size_t&                       workspaceSizeInBytes);
 
+std::vector<std::shared_ptr<Tensile::ContractionSolution>>
+    getBestRawSolutions(RocblasltContractionProblem const& prob,
+                        rocblaslt_handle                   handle,
+                        std::shared_ptr<void>              gemmData,
+                        int                                requestedAlgoCount,
+                        size_t                             maxWorkSpaceBytes);
+
 /*******************************************************************************
  * getBestSolutions() calls finTopSolutions from Tensile and converts to       *
  * rocblaslt_matmul_heuristic_result                                           *
@@ -413,6 +464,12 @@ inline Tensile::DataType hipDataType_to_tensile_type(hipDataType type)
         return Tensile::DataType::Float8;
     case HIP_R_8F_E5M2_FNUZ:
         return Tensile::DataType::BFloat8;
+#ifdef ROCM_USE_FLOAT8
+    case HIP_R_8F_E4M3:
+        return Tensile::DataType::Float8;
+    case HIP_R_8F_E5M2:
+        return Tensile::DataType::BFloat8;
+#endif
     case HIP_R_8I:
         return Tensile::DataType::Int8;
     case HIP_R_32I:
