@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -41,6 +41,7 @@ class StoreState:
     class StoreConstConfig:
         def __init__(self, kernelWriter, kernel, ss, gwvw, edge, beta, atomic):
             self.gwvw = gwvw
+            self.lsu = kernel["LocalSplitU"]
 
             if ss.optSingleColVgpr:
                 # use one vgpr (allocated in ss.sharedColDVgprs) for all addressing
@@ -98,11 +99,16 @@ class StoreState:
             # exception: data vgpr cannot be shared if UseInitialStridesCD is enabled and card enable EccHalf,
             #            since each buffer_load_short would overwrite undefined 16bit as zero.
             self.halfDataRegPerVI = gwvw*self.numVgprsPerDataPerVI == 0.5 and not (kernel["ProblemType"]["UseInitialStridesCD"] and kernelWriter.states.archCaps["HasEccHalf"]) and not (kernel["ProblemType"]["DestDataType"].numRegisters() == 0.25)
+            # indicates the VGPRs index offset from LSU Reduction.
+            # Used for multi-batch/Edge cases.
+            self.lsuStartVgprOffset = 0
 
     # StoreState constructor:
     def __init__(self, kernelWriter, kernel, gwvw, edge, beta, atomic, elements):
         self.kernelWriter = kernelWriter
         self.kernel = kernel
+        self.lsu = kernel["LocalSplitU"]
+        self.lsuStartVgprOffset = 0
 
         self.isReset = False
         #--
@@ -182,6 +188,10 @@ class StoreState:
             numCols = len([e for e in elements if e[0] == 0 and e[2] == 0]) # count #elements with row d1=v1==0
             self.numAddrVgpr = numCols
             self.sharedColDVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColDVgprs for packed elements")
+            if kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel":
+                self.sharedGSUSyncVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedGSUSyncVgprs for packed elements")
+            else:
+                self.sharedGSUSyncVgprs = None
             if kernel["GroupLoadStore"] and kernel["ProblemType"]["UseBeta"]:
                 self.sharedColCVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColCVgprs for packed elements")
             else:
@@ -194,16 +204,27 @@ class StoreState:
                 self.sharedColEVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColEVgprs for packed elements")
             else:
                 self.sharedColEVgprs = None
-            if kernel["ProblemType"]["UseScaleAlphaVec"] and (kernel["GlobalSplitU"] == 1):
+            if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+                self.sharedColScaleAVecVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColScaleAVecVgprs for packed elements")
+                self.sharedColScaleBVecVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColScaleBVecVgprs for packed elements")
+            else:
+                self.sharedColScaleAVecVgprs = None
+                self.sharedColScaleBVecVgprs = None
+            if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
                 self.sharedColScaleAlphaVecVgprs = kernelWriter.vgprPool.checkOut(self.numAddrVgpr, "sharedColScaleAlphaVecVgprs for packed elements")
             else:
                 self.sharedColScaleAlphaVecVgprs = None
         elif self.optSingleColVgpr:
             self.numAddrVgpr = 1
             self.sharedColDVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColDVgprs")
+            if kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel":
+                self.sharedGSUSyncVgprs = kernelWriter.vgprPool.checkOut(1, "sharedGSUSyncVgprs")
+            else:
+                self.sharedGSUSyncVgprs = None
             self.singleColBiasAddrUpdated = False
             self.singleColEAddrUpdated    = False
             self.singleColDAddrUpdated    = False
+            self.singleColTDAddrUpdated   = False
             self.singleColCAddrUpdated    = False
             if kernel["ProblemType"]["UseBeta"]:
                 self.sharedColCVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColCVgprs")
@@ -217,7 +238,13 @@ class StoreState:
                 self.sharedColEVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColEVgprs for packed elements")
             else:
                 self.sharedColEVgprs = None
-            if kernel["ProblemType"]["UseScaleAlphaVec"] and (kernel["GlobalSplitU"] == 1):
+            if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+                self.sharedColScaleAVecVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColScaleAVecVgprs for packed elements")
+                self.sharedColScaleBVecVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColScaleBVecVgprs for packed elements")
+            else:
+                self.sharedColScaleAVecVgprs = None
+                self.sharedColScaleBVecVgprs = None
+            if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
                 self.sharedColScaleAlphaVecVgprs = kernelWriter.vgprPool.checkOut(1, "sharedColScaleAlphaVecVgprs for packed elements")
             else:
                 self.sharedColScaleAlphaVecVgprs = None
@@ -225,8 +252,11 @@ class StoreState:
             self.numAddrVgpr = 0
             self.sharedColEVgprs    = None
             self.sharedColDVgprs    = None
+            self.sharedGSUSyncVgprs = None
             self.sharedColCVgprs    = None
             self.sharedColBiasVgprs = None
+            self.sharedColScaleAVecVgprs = None
+            self.sharedColScaleBVecVgprs = None
             self.sharedColScaleAlphaVecVgprs = None
 
         # For detecting when we are running first batch
@@ -249,9 +279,14 @@ class StoreState:
                 numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
                 self.numVgprsPerElement += numVgprs * gwvw  # Loaded data
 
-        if kernel["ProblemType"]["UseScaleAlphaVec"] and (kernel["GlobalSplitU"] == 1):
+        if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+            self.numVgprsPerElement += self.cfg.numVgprsPerAddr * 2  # ScaleAVec + ScaleBVec address
+            numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
+            self.numVgprsPerElement += numVgprs * gwvw + (numVgprs * (2 if gwvw >= 2 else 1)) # Loaded data
+
+        if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
             self.numVgprsPerElement += self.cfg.numVgprsPerAddr  # ScaleAlphaVec address
-            numVgprs = int(ceil(kernel["ProblemType"]["DataType"].numRegisters()))
+            numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
             self.numVgprsPerElement += numVgprs * gwvw  # Loaded data
         # Calculate align
         self.align = 1
@@ -272,14 +307,9 @@ class StoreState:
     #
     # Also create an AddrCalc for each memory operation.
     ##############################################################################
-    def setupStoreElementsForBatch(self, kernel, gwvw, batchElements, batchElementSgprs, isOptNLL):
-
-        self.elementAddr     = []
-        self.elementDataE    = []
-        self.elementData     = []  # VGPR to use for element data, needed for atomic or beta
-        self.elementDataBias = []
-        self.elementDataScaleAlphaVec = []
-        self.elementMask     = []  # SGPR to use for element mask
+    def getStoreElementsInfoForBatch(self, kernel, batchElements):
+        self.elementCoord0 = []
+        self.elementCoord1 = []
         self.elementSumIdx = []
 
         kw = self.kernelWriter
@@ -290,21 +320,7 @@ class StoreState:
             matrixInstBM = 1                                                if (kernel["MatrixInstM"] == 4) else kernel["MatrixInstBM"]
             matrixInstBN = 1                                                if (kernel["MatrixInstN"] == 4) else kernel["MatrixInstBN"]
 
-        biasVgprMap = {}
-        scaleAlphaVecVgprMap = {}
-        lastData = 0
         for elementIdx in range(0, len(batchElements)):
-            # Create the AddrCalc for each memory load/store
-            # This is the control code that sets up the dest, source, offsets, etc and
-            # identifies cases where the AddrCalc is a new row and therefore needs some
-            # additional math.  Each AddrCalc contains isolated state sufficient to
-            # perform any needed range checks and address calculations for the element.
-            #
-            # The AddrCalc creation code here maintains state across elements (including
-            # across write batches) to remove replicated calculations.
-            #
-            # Later the AddrCalc::emitAddressSetupCode will emit the necessary code
-            # Also allocate VGPR resources here, if needed.
 
             element = batchElements[elementIdx]
             (d1,d0,vc1,vc0) = element
@@ -327,8 +343,15 @@ class StoreState:
                 coordOffset1 += bIdx1 * matrixInstN
                 coordOffset1 += wtIdex * matrixInstN *  matrixInstBN * kernel["MIWaveGroup"][1]
                 coordOffset1  = coordOffset1 * vectorWidth + vc1
+            else: # mac instruction
+                if kernel["LocalSplitU"] > 1:
+                    strideD1 = (kernel["NumThreads"]*kernel["VectorWidthB"]//kernel["MacroTile0"])
+                else:
+                    strideD1 = (kernel["SubGroup1"] * kernel["VectorWidthB"])
+                coordOffset1 = d1 * strideD1 + vc1
 
             newCoord1 = (self.firstBatch and elementIdx==0) or (coordOffset1 != self.lastCoordOffset1)
+            self.elementCoord1.append(coordOffset1)
 
             # gpr and offset assignments for element
             coordOffset0 = 0
@@ -348,13 +371,65 @@ class StoreState:
                 coordOffset0 += bIdx0 * matrixInstM
                 coordOffset0 += wtIdex * matrixInstM * matrixInstBM * kernel["MIWaveGroup"][0]
                 coordOffset0  = coordOffset0 * vectorWidth + vc0
+            else: # mac instruction
+                coordOffset0 = d0 * kernel["SubGroup0"]*kernel["VectorWidthA"] + vc0
+            self.elementCoord0.append(coordOffset0)
+
+        return self.elementCoord0, self.elementCoord1
+
+    def setupStoreElementsForBatch(self, kernel, gwvw, batchElements, batchElementSgprs, isOptNLL, factorDim):
+
+        self.elementAddr              = []
+        self.elementDataE             = []
+        self.elementData              = []  # VGPR to use for element data, needed for atomic or beta
+        self.elementDataBias          = []
+        self.elementDataScaleAVec     = []
+        self.elementDataScaleBVec     = []
+        self.elementDataScaleAlphaVec = []
+        self.elementMask              = []  # SGPR to use for element mask
+        self.elementSumIdx            = []
+        self.elementCoord0            = []
+        self.elementCoord1            = []
+
+        #generate elementCoord0/1
+        self.getStoreElementsInfoForBatch(kernel, batchElements)
+        kw = self.kernelWriter
+
+        biasVgprMap = {}
+        scaleAVecVgprMap = {}
+        scaleBVecVgprMap = {}
+        scaleAlphaVecVgprMap = {}
+        lastData = 0
+
+        for elementIdx in range(0, len(batchElements)):
+            # Create the AddrCalc for each memory load/store
+            # This is the control code that sets up the dest, source, offsets, etc and
+            # identifies cases where the AddrCalc is a new row and therefore needs some
+            # additional math.  Each AddrCalc contains isolated state sufficient to
+            # perform any needed range checks and address calculations for the element.
+            #
+            # The AddrCalc creation code here maintains state across elements (including
+            # across write batches) to remove replicated calculations.
+            #
+            # Later the AddrCalc::emitAddressSetupCode will emit the necessary code
+            # Also allocate VGPR resources here, if needed.
+
+            element      = batchElements[elementIdx]
+            coordOffset0 = self.elementCoord0[elementIdx]
+            coordOffset1 = self.elementCoord1[elementIdx]
+            newCoord1    = (self.firstBatch and elementIdx==0) or (coordOffset1 != self.lastCoordOffset1)
+
+            (d1,d0,vc1,vc0) = element
 
             if self.optSingleColVgpr:
                 # use same address vgpr for all
                 addrEVgpr    = self.sharedColEVgprs
                 addrDVgpr    = self.sharedColDVgprs
+                addrGSUSyncVgprs = self.sharedGSUSyncVgprs
                 addrCVgpr    = self.sharedColCVgprs
                 addrBiasVgpr = self.sharedColBiasVgprs
+                addrScaleAVecVgpr = self.sharedColScaleAVecVgprs
+                addrScaleBVecVgpr = self.sharedColScaleBVecVgprs
                 addrScaleAlphaVecVgpr = self.sharedColScaleAlphaVecVgprs
             elif self.optSharedColVgpr:
                 if kernel["EnableMatrixInstruction"]:
@@ -364,6 +439,10 @@ class StoreState:
                 assert (modf(elementCol)[0] < 0.001)
                 elementCol   = trunc(elementCol)
                 addrDVgpr    = self.sharedColDVgprs+elementCol
+                if kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel":
+                    addrGSUSyncVgprs = self.sharedGSUSyncVgprs+elementCol
+                else:
+                    addrGSUSyncVgprs = None
                 addrCVgpr    = self.sharedColCVgprs+elementCol
                 if self.useBias != DataDirection.NONE:
                     addrBiasVgpr = self.sharedColBiasVgprs+elementCol
@@ -375,14 +454,27 @@ class StoreState:
                     addrEVgpr = None
                 #print ("d0=", d0, "vc0=", vc0, "elementCol=", elementCol)
 
+                if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and (kernel["GlobalSplitU"] == 1):
+                    addrScaleAVecVgpr = self.sharedColScaleAVecVgprs+elementCol
+                    addrScaleBVecVgpr = self.sharedColScaleBVecVgprs+elementCol
+                else:
+                    addrScaleAVecVgpr = None
+                    addrScaleBVecVgpr = None
+
                 if kernel["ProblemType"]["UseScaleAlphaVec"] and (kernel["GlobalSplitU"] == 1):
                     addrScaleAlphaVecVgpr = self.sharedColScaleAlphaVecVgprs+elementCol
                 else:
                     addrScaleAlphaVecVgpr = None
+
             else:
                 # allocate new VGPR for each element:
                 addrDVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
                     int(ceil(self.cfg.numVgprsPerAddr)), "writeDBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
+                if kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel":
+                    addrGSUSyncVgprs = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
+                        int(ceil(self.cfg.numVgprsPerAddr)), "writeDBatch-addr for ei=%u"%(elementIdx), preventOverflow=0)
+                else:
+                    addrGSUSyncVgprs = None
                 if kernel["GroupLoadStore"] and kernel["ProblemType"]["UseBeta"]:
                     addrCVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
                         int(ceil(self.cfg.numVgprsPerAddr)), "loadCBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
@@ -399,13 +491,21 @@ class StoreState:
                 else:
                     addrEVgpr = None
 
-                if kernel["ProblemType"]["UseScaleAlphaVec"] and (kernel["GlobalSplitU"] == 1):
+                if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+                    addrScaleAVecVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
+                        int(ceil(self.cfg.numVgprsPerAddr)), "loadScaleAVecBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
+                    addrScaleBVecVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
+                        int(ceil(self.cfg.numVgprsPerAddr)), "loadScaleAVecBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
+                else:
+                    addrScaleAVecVgpr = None
+                    addrScaleBVecVgpr = None
+
+                if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
                     addrScaleAlphaVecVgpr = kw.vgprPool.checkOutAligned(self.cfg.numVgprsPerAddr, \
                         int(ceil(self.cfg.numVgprsPerAddr)), "loadScaleAlphaVecBatch-addr for ei=%u"%(elementIdx), preventOverflow=not isOptNLL)
                 else:
                     addrScaleAlphaVecVgpr = None
-
-            self.elementAddr.append(AddrCalculation(kw, self, addrCVgpr, addrDVgpr, addrEVgpr, addrBiasVgpr, addrScaleAlphaVecVgpr, element, coordOffset0, \
+            self.elementAddr.append(AddrCalculation(kw, self, addrCVgpr, addrDVgpr, addrGSUSyncVgprs, addrEVgpr, addrBiasVgpr, addrScaleAVecVgpr, addrScaleBVecVgpr, addrScaleAlphaVecVgpr, element, coordOffset0, \
               self.kernelWriter.vgprs.coord1, coordOffset1, coordOffset1 - self.lastCoordOffset1, newCoord1))
             # if numVgprsPerDataPerVI == 0.5, then two consecutive elements
             # should have same data pointer, next should move.
@@ -441,13 +541,15 @@ class StoreState:
             self.elementData.append(data)
 
             if self.useBias == DataDirection.READ:
-                if coordOffset0 in biasVgprMap:
-                    dataBias = biasVgprMap[coordOffset0]
+                coordOffset = coordOffset0 if factorDim == 0 else coordOffset1
+                if coordOffset in biasVgprMap:
+                    dataBias = biasVgprMap[coordOffset]
                 else:
+                    gwvw = self.cfg.gwvw if factorDim == 0 else min(self.cfg.gwvw, 2)
                     numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
-                    dataBias = kw.vgprPool.checkOutAligned(int(numVgprs*self.cfg.gwvw), \
-                                  int(ceil(numVgprs*self.cfg.gwvw)), "bias data for ei=%u"%elementIdx, preventOverflow=False)
-                    biasVgprMap[coordOffset0] = dataBias
+                    dataBias = kw.vgprPool.checkOutAligned(int(numVgprs*gwvw), \
+                                int(ceil(numVgprs*gwvw)), "bias data for ei=%u"%elementIdx, preventOverflow=False)
+                    biasVgprMap[coordOffset] = dataBias
             else:
                 dataBias = 0
             self.elementDataBias.append(dataBias)
@@ -461,14 +563,38 @@ class StoreState:
                 dataE = 0
             self.elementDataE.append(dataE)
 
-            if kernel["ProblemType"]["UseScaleAlphaVec"] and (kernel["GlobalSplitU"] == 1):
-                if coordOffset0 in scaleAlphaVecVgprMap:
-                    dataScaleAlphaVec = scaleAlphaVecVgprMap[coordOffset0]
+            if (kernel["ProblemType"]["UseScaleAB"] == "Vector") and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+                if coordOffset0 in scaleAVecVgprMap:
+                    dataScaleAVec = scaleAVecVgprMap[coordOffset0]
                 else:
                     numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
-                    dataScaleAlphaVec = kw.vgprPool.checkOutAligned(int(numVgprs*self.cfg.gwvw), \
-                                  int(ceil(numVgprs*self.cfg.gwvw)), "scaleAlphaVec data for ei=%u"%elementIdx, preventOverflow=False)
-                    scaleAlphaVecVgprMap[coordOffset0] = dataScaleAlphaVec
+                    dataScaleAVec = kw.vgprPool.checkOutAligned(int(numVgprs*self.cfg.gwvw), \
+                                  int(ceil(numVgprs*self.cfg.gwvw)), "scaleAVec data for ei=%u"%elementIdx, preventOverflow=False)
+                    scaleAVecVgprMap[coordOffset0] = dataScaleAVec
+                if coordOffset1 in scaleBVecVgprMap:
+                    dataScaleBVec = scaleBVecVgprMap[coordOffset1]
+                else:
+                    gwvw = min(self.cfg.gwvw, 2)
+                    numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
+                    dataScaleBVec = kw.vgprPool.checkOutAligned(int(numVgprs*gwvw), \
+                                  int(ceil(numVgprs*gwvw)), "scaleBVec data for ei=%u"%elementIdx, preventOverflow=False)
+                    scaleBVecVgprMap[coordOffset1] = dataScaleBVec
+            else:
+                dataScaleAVec = 0
+                dataScaleBVec = 0
+            self.elementDataScaleAVec.append(dataScaleAVec)
+            self.elementDataScaleBVec.append(dataScaleBVec)
+
+            if kernel["ProblemType"]["UseScaleAlphaVec"] and ((kernel["GlobalSplitU"] == 1) or (kernel["_GlobalAccumulation"] == "MultipleBufferSingleKernel")):
+                coordOffset = coordOffset0 if factorDim == 0 else coordOffset1
+                gwvw = self.cfg.gwvw if factorDim == 0 else (2 if self.cfg.gwvw >= 2 else 1)
+                if coordOffset in scaleAlphaVecVgprMap:
+                    dataScaleAlphaVec = scaleAlphaVecVgprMap[coordOffset]
+                else:
+                    numVgprs = int(ceil(kernel["ProblemType"]["ComputeDataType"].numRegisters()))
+                    dataScaleAlphaVec = kw.vgprPool.checkOutAligned(int(numVgprs*gwvw), \
+                                  int(ceil(numVgprs*gwvw)), "scaleAlphaVec data for ei=%u"%elementIdx, preventOverflow=False)
+                    scaleAlphaVecVgprMap[coordOffset] = dataScaleAlphaVec
             else:
                 dataScaleAlphaVec = 0
             self.elementDataScaleAlphaVec.append(dataScaleAlphaVec)
@@ -483,7 +609,10 @@ class StoreState:
             #print "Edge=", edge, element
             sumIdx = 0
             if kernel["LocalSplitU"] > 1:
-                sumIdx = kw.states.c.startVgprValu + vc0 + d1*kernel["VectorWidthA"]
+                if len(self.elementSumIdx) == 0:
+                    sumIdx = kw.states.c.startVgprValu
+                else:
+                    sumIdx = self.elementSumIdx[-1] + self.cfg.numVgprPerValuC * self.cfg.gwvw
             else:
                 bestVw                  = kernel["VectorWidthA"]
                 elementsLoadedPerVw     = kernel["NumThreads"] * bestVw
@@ -506,7 +635,7 @@ class StoreState:
         if self.kernelWriter.states.serializedStore is False:
             return # early exit; currently only serializedStore==True checks out C-tile from register pool
 
-        if len(self.elementSumIdx) > 0:
+        if len(self.elementSumIdx) > 0 and self.lsu == 1:
             for i in self.elementSumIdx:
                 self.kernelWriter.vgprPool.checkIn(i * self.cfg.numVgprPerValuC)
                 # print("checked in vgpr %u"%i)
@@ -521,6 +650,7 @@ class StoreState:
                 self.singleColBiasAddrUpdated = False
                 self.singleColEAddrUpdated    = False
                 self.singleColDAddrUpdated    = False
+                self.singleColTDAddrUpdated    = False
                 self.singleColCAddrUpdated    = False
             else:
                 pass # Nothing to reset
@@ -532,14 +662,26 @@ class StoreState:
 
         if (self.sharedColEVgprs != None):
             self.kernelWriter.vgprPool.checkIn(self.sharedColEVgprs)
+            self.sharedColEVgprs = None
         if (self.sharedColDVgprs != None):
             self.kernelWriter.vgprPool.checkIn(self.sharedColDVgprs)
-            if (self.sharedColCVgprs != self.sharedColDVgprs):
+            if (self.sharedColCVgprs != self.sharedColDVgprs and self.sharedColCVgprs != None):
                 self.kernelWriter.vgprPool.checkIn(self.sharedColCVgprs)
+                self.sharedColCVgprs = None
+            self.sharedColDVgprs = None
+        if (self.sharedGSUSyncVgprs != None):
+            self.kernelWriter.vgprPool.checkIn(self.sharedGSUSyncVgprs)
         if (self.sharedColBiasVgprs != None):
             self.kernelWriter.vgprPool.checkIn(self.sharedColBiasVgprs)
-
+            self.sharedColBiasVgprs = None
+        if (self.sharedColScaleAVecVgprs != None):
+            self.kernelWriter.vgprPool.checkIn(self.sharedColScaleAVecVgprs)
+            self.sharedColScaleAVecVgprs = None
+        if (self.sharedColScaleBVecVgprs != None):
+            self.kernelWriter.vgprPool.checkIn(self.sharedColScaleBVecVgprs)
+            self.sharedColScaleBVecVgprs = None
         if (self.sharedColScaleAlphaVecVgprs != None):
             self.kernelWriter.vgprPool.checkIn(self.sharedColScaleAlphaVecVgprs)
+            self.sharedColScaleAlphaVecVgprs = None
         self.checkInTempVgprC()
         self.resetState()
