@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -118,7 +118,8 @@ globalParameters["UnrollLoopEfficiencyEnable"] = False   # if True split(S) MAC&
 ########################################
 globalParameters["CMakeBuildType"] = "Release"            # whether benchmark clients and library client should be release or debug
 globalParameters["PrintSolutionRejectionReason"] = False  # when a solution is marked as invalid, print why
-globalParameters["LibraryFormat"] = "yaml"                # set library backend (either yaml or msgpack)
+globalParameters["LogicFormat"] = "yaml"                  # set library backend (yaml, or json)
+globalParameters["LibraryFormat"] = "yaml"                # set library backend (yaml, or msgpack)
 globalParameters["EmbedLibrary"] = None                   # whether library should be embedded or not
 
 # True/False: CSV will/won't export WinnerGFlops, WinnerTimeUS, WinnerIdx, WinnerName.
@@ -172,6 +173,7 @@ globalParameters["CMakeCFlags"] = ""              # pass flags to cmake
 globalParameters["DebugKernel"] = False           # assembly only, kernel gets buffer for debug "printing"; kernel writes data to memory, gets coppied to host and printed
 globalParameters["LibraryPrintDebug"] = False     # solutions will print enqueue info when enqueueing a kernel
 globalParameters["SaveTemps"] = False             # Generate intermediate results of hip kernels
+globalParameters["KeepBuildTmp"] = False          # If true, do not remove artifacts in build_tmp
 
 # debug for assembly
 globalParameters["EnableAsserts"] = False         # Enable assembly debug assert
@@ -189,6 +191,7 @@ globalParameters["PrintTensorC"] = 0          # Print TensorC.  0x1=after init;
 globalParameters["PrintTensorD"] = 0          # Print TensorD.  0x1=after init; 0x2=after copy-back; 0x3=both
 globalParameters["PrintTensorRef"] = 0          # Print reference tensor.  0x1=after init; 0x2=after copy-back; 0x3=both
 globalParameters["PrintTensorBias"] = 0          # Print TensorBias after initialization
+globalParameters["PrintTensorAmaxD"] = 0          # Print AmaxD after validation
 globalParameters["PrintIndexAssignments"] = 0      # Print the tensor index assignment info
 globalParameters["PrintWinnersOnly"] = False      # Only print the solutions which become the fastest
 globalParameters["PrintCodeCommands"] = False  # print the commands used to generate the code objects (asm,link,hip-clang, etc)
@@ -209,7 +212,7 @@ globalParameters["MergeFiles"] = True             # F=store every solution and k
 globalParameters["NumMergedFiles"] = 1            # The number of files that kernels should be split between when merging
 
 globalParameters["MaxFileName"] = 64              # If a file name would be longer than this, shorten it with a hash.
-globalParameters["SupportedISA"] = [(8,0,3), (9,0,0), (9,0,6), (9,0,8), (9,0,10), (9,4,0), (9,4,1), (9,4,2), (10,1,0), (10,1,1), (10,1,2), (10,3,0), (11,0,0), (11,0,1), (11,0,2)] # assembly kernels writer supports these architectures
+globalParameters["SupportedISA"] = [(8,0,3), (9,0,0), (9,0,6), (9,0,8), (9,0,10), (9,4,0), (9,4,1), (9,4,2), (10,1,0), (10,1,1), (10,1,2), (10,3,0), (11,0,0), (11,0,1), (11,0,2), (12,0,0), (12,0,1)] # assembly kernels writer supports these architectures
 
 globalParameters["GenerateManifestAndExit"] = False               # Output manifest file with list of expected library objects and exit
 globalParameters["NewClient"] = 2                                 # Old client deprecated: NewClient must be set to 2.
@@ -226,7 +229,7 @@ globalParameters["LibraryUpdateComment"] = False                  # Include solu
 globalParameters["CurrentISA"] = (0,0,0)
 globalParameters["ROCmAgentEnumeratorPath"] = None      # /opt/rocm/bin/rocm_agent_enumerator
 globalParameters["ROCmSMIPath"] = None                  # /opt/rocm/bin/rocm-smi
-globalParameters["AssemblerPath"] = None                # /opt/rocm/hip/bin/hipcc
+globalParameters["AssemblerPath"] = None                # /opt/rocm/llvm/bin/clang++
 globalParameters["WorkingPath"] = os.getcwd()           # path where tensile called from
 globalParameters["IndexChars"] =  "IJKLMNOPQRSTUVWXYZ"  # which characters to use for C[ij]=Sum[k] A[ik]*B[jk]
 globalParameters["ScriptPath"] = os.path.dirname(os.path.realpath(__file__))            # path to Tensile/Tensile.py
@@ -240,7 +243,8 @@ else:
   globalParameters["RuntimeLanguage"] = "HIP"
 
 globalParameters["CodeObjectVersion"] = "default"
-globalParameters["CxxCompiler"] = "hipcc"
+globalParameters["CxxCompiler"] = "amdclang++" if os.name != "nt" else "clang++"
+globalParameters["CCompiler"] = "amdclang" if os.name != "nt" else "clang"
 globalParameters["Architecture"] = "all"
 
 # might be deprecated
@@ -273,6 +277,11 @@ globalParameters["LazyLibraryLoading"] = False # Load library and code object fi
 globalParameters["UseUserArgs"] = False
 
 globalParameters["RotatingBufferSize"] = 0 # Size in MB
+globalParameters["RotatingMode"] = 0 # Default is 0, allocated in order A0B0C0D0..ANBNCNDN. 1 is in order A0 pad B0 pad .... AN pad BN pad.
+                                     # Mode 0 requires memcpy everytime when the problem changes to reset the data, but mode 1 doesn't.
+
+globalParameters["BuildIdKind"] = "sha1"
+globalParameters["ValidateLibrary"] = False
 
 # Save a copy - since pytest doesn't re-run this initialization code and YAML files can override global settings - odd things can happen
 defaultGlobalParameters = deepcopy(globalParameters)
@@ -287,7 +296,8 @@ architectureMap = {
   'gfx941':'aquavanjaram', 'gfx941:xnack+':'aquavanjaram', 'gfx941:xnack-':'aquavanjaram',
   'gfx942':'aquavanjaram', 'gfx942:xnack+':'aquavanjaram', 'gfx942:xnack-':'aquavanjaram',
   'gfx1010':'navi10', 'gfx1011':'navi12', 'gfx1012':'navi14', 'gfx1030':'navi21',
-  'gfx1100':'navi31', 'gfx1101':'navi32', 'gfx1102':'navi33'
+  'gfx1100':'navi31', 'gfx1101':'navi32', 'gfx1102':'navi33',
+  'gfx1200':'gfx1200', 'gfx1201':'gfx1201',
 }
 
 def getArchitectureName(gfxName):
@@ -311,14 +321,39 @@ internalParameters = {
 
 # These parameters are used in ContractionSolutions for user arguments support.
 defaultInternalSupportParams = {
+  "KernArgsVersion": 2,
   # Information about user input internal kernel argument support
   # Change this to False if the CustomKernel does not support.
   "SupportUserGSU": True,
   # This is a little different from GSU because GSU is already a parameter,
   # but WGM is not.
-  "SupportCustomWGM": True
+  "SupportCustomWGM": True,
+  "SupportCustomStaggerU": True,
+  # Use GG as G's backend
+  "UseUniversalArgs": True
 }
 
+def supportedCompiler(compiler: str) -> bool:
+  """ Determines if compiler is supported by Tensile.
+
+      Args:
+          The name of a compiler to test for support.
+
+      Return:
+          If supported True; otherwise, False.
+  """
+  isSupported = (compiler == "hipcc")
+  if os.name == "nt":
+    isSupported = (isSupported or compiler == "clang++")
+  else:
+    isSupported = (isSupported or compiler == "amdclang++")
+
+  if not isSupported: printWarning(f"{compiler} is unsupported for os {os.name}")
+
+  return isSupported
+
+
+
 ################################################################################
 # Enumerate Valid Solution Parameters
 ################################################################################
@@ -343,7 +378,7 @@ validMacroTileSides = [1, 2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 6, 12, 24, 4
 validMacroTiles = []
 validISA = [(0,0,0)]
 validISA.extend(globalParameters["SupportedISA"])
-depthUs = list(range(2,512+1,1))
+depthUs = list(range(2,1024+1,1))
 for i in validMacroTileSides:
   for j in validMacroTileSides:
     validMacroTiles.append([i, j])
@@ -383,8 +418,12 @@ validSMFMA["H"] = [[32,32,16,1], [16,16,32,1]]
 validSMFMA["B"] = [[32,32,16,1], [16,16,32,1]]
 validSMFMA["4xi8"] = [[32,32,32,1], [16,16,64,1]]
 validSMFMA["I8"] = validSMFMA["4xi8"]
+validSMFMA["F8"] = [[32,32,32,1], [16,16,64,1]]
+validSMFMA["B8"] = validSMFMA["F8"]
+validSMFMA["F8B8"] = validSMFMA["F8"]
+validSMFMA["B8F8"] = validSMFMA["F8"]
 validSMFMA["_format9"] = []
-for SMFMA in [validSMFMA["H"], validSMFMA["B"], validSMFMA["4xi8"]]:
+for SMFMA in [validSMFMA["H"], validSMFMA["B"], validSMFMA["4xi8"], validSMFMA["F8"]]:
   for MI in SMFMA:
     for bm in range(int(math.log(MI[3],2))+1):
       for tt0 in range(1,validTT+1):
@@ -406,26 +445,31 @@ validMatrixInstructions = validMatrixInstructions + validSparseMatrixInstruction
 # This is used in SolutionStruct.py::checkIfSupportedGEMMType()
 validGEMMTypes = [ ('H','H','H'), ('S','S','S'), ('D','D','D'), ('C','C','C'), ('Z','Z','Z'), \
                    ('H','H','S'), ('H','S','S'), \
-                   ('B','B','S'), ('B','S','S'), \
+                   ('B','B','S'), ('B','S','S'), ('B','H','S'), \
                    ('I8','I','I'), ('4xi8','I','I'), ('I8','I8','I'), \
-                   ('I8','I','S'), ('I8','I8','S'), ('I8', 'H', 'S'), \
+                   ('I8','I','S'), ('I8','I8','S'), ('I8', 'H', 'S'), ('I8', 'B', 'S'), \
                    ('F8','S','S'), ('B8','S','S'), \
                    ('F8B8','S','S'), ('B8F8', 'S', 'S'), \
+                   ('F8','H','S'), ('B8','H','S'), \
+                   ('F8B8','H','S'), ('B8F8','H','S'), ('B8','B','S'), \
+                   ('H','F8','S'), ('F8','B','S'), ('F8B8','B','S'), ('B8F8','B','S'), \
+                   # in/out are both R8
                    ('F8','F8','S'), ('B8','B8','S'), \
                    ('F8B8','B8','S'), ('B8F8', 'B8', 'S'), \
-                   ('F8','H','S'), ('B8','H','S'), \
-                   ('F8B8','H','S'), ('B8F8','H','S'),
-                   ('H','F8','S') ]
+                   ('F8','B8','S'), ('B8','F8','S'), \
+                   ('F8B8','F8','S'), ('B8F8', 'F8', 'S') ]
 
 # All HPA types are listed here (HPA=T). The name of the library logic files for these types is:
-# *_TiToTc_BH*.yaml where Ti, Tc, and To are the data types of A/B, C/D, and computation, respectively.
+# *_TiToTc_BH*.yaml where Ti, To, and Tc are the data types of A/B, C/D, and computation, respectively.
 # The name of the library logic files for non-HPA (HPA=F) types is: *_TiB*.yaml.
-HPATypes = [ ('H','S','S'), ('H','H','S'), ('B','B','S'), ('B','S','S'), ('I8','I','I'), \
-             ('4xi8','I','I'), ('I8','I','S'), ('I8','I8','S'), ('I8', 'H', 'S'), \
+HPATypes = [ ('H','S','S'), ('H','H','S'), ('B','B','S'), ('B','S','S'), ('B','H','S'), ('I8','I','I'), \
+             ('4xi8','I','I'), ('I8','I','S'), ('I8','I8','S'), ('I8', 'H', 'S'), ('I8', 'B', 'S'),\
              ('F8','S','S'), ('B8','S','S'), ('F8B8','S','S'), ('B8F8', 'S', 'S'), \
-             ('F8B8','B8','S'), ('B8F8', 'B8', 'S'), \
              ('F8','H','S'), ('B8','H','S'), ('F8B8','H','S'), ('B8F8','H','S'), \
-             ('F8','F8', 'S'), ('B8', 'B8', 'S'), ('H','F8','S') ]
+             ('H','F8','S'), ('F8','B','S'), ('F8B8','B','S'), \
+             # in/out are both R8
+             ('F8','F8','S'), ('B8','B8','S'), ('F8B8','B8','S'), ('B8F8', 'B8', 'S'), \
+             ('F8','B8','S'), ('B8','F8','S'), ('F8B8','F8','S'), ('B8F8', 'F8', 'S') ]
 
 validParameters = {
     # 0: Global read is along parallel direction in thread level,
@@ -461,6 +505,10 @@ validParameters = {
     "WaveSeparateGlobalReadA":    [ 0, 1, 2 ],
     "WaveSeparateGlobalReadB":    [ 0, 1, 2 ],
 
+    # Add an unrolled loop and NGLL loop with swapped GRA and GRB order.
+    # which may change the tlb thrashing behavior.
+    "UnrollLoopSwapGlobalReadOrder": [0, 1],
+
     # PrefetchGlobalRead = 1:
     # Requires 2X LDS space, and VGPRs for buffering data on way into LDS
     #   prefetch / double-buffer reads from global memory -> vgprs -> lds.
@@ -499,7 +547,8 @@ validParameters = {
     # choose how to do GlobalSplitU
     # 1: use atomic operation to accumulate on one buffer
     # 2: each GSU group write to each own buffer and accumulate by another kernel
-    "GlobalSplitUAlgorithm":      ["SingleBuffer", "MultipleBuffer"],
+    # 3: each GSU group write to each own buffer and accumulate by same kernel
+    "GlobalSplitUAlgorithm":      ["SingleBuffer", "MultipleBuffer", "MultipleBufferSingleKernel"],
 
     # don't create a whole copy of the Unroll loop with loads removed - instead
     # use buffer limits to suppress global loads and ignore unnecessary ds_reads
@@ -715,7 +764,7 @@ validParameters = {
     # StaggerUStride will be internally increased so it is an integer multiple of DepthU*BpeAB.
     # (the implementation requires this - the unroll iteration accesses data in steps of
     # DepthU*BPE
-    "StaggerUStride":               [16,32,64,128,256,512,1024],
+    "StaggerUStride":               [-1,16,32,64,128,256,512,1024,2048],
 
     # How the tile assignment (wg0, wg1, wg2) controls the initial StaggerU offset:
     # 0: Use wg0
@@ -726,7 +775,18 @@ validParameters = {
     #    to a different bank since all workgroups still start at same point.
     "StaggerUMapping":       [0,1,2,3,4],
 
+    # GSU Workgroup Coalesced Ordering
+    # False: {(wg0,wg1,wg2,wgn)|(wg0,wg1,wg2,wgn)|...|(wg0,wg1,wg2,wgn)}
+    # True:  {(wg0,wg0,wg0)|(wg1,wg1,wg1)|(wg2,wg2,wg2)|...|(wgn,wgn,wgn)}
+    "GlobalSplitUCoalesced":        [False, True],
 
+    # GSU Workgroup Mapping
+    # False: wg issued order = {(wg0,wg1,wg2,wgn),(wg0,wg1,wg2,wgn)|...|(wg0,wg1,wg2,wgn)}
+    #   -> workgroups do the summation by tile -> slower GR but faster GW
+    # True:  wg issused oder = {(wg0,wg0,wg0)|(wg1,wg1,wg1)|(wg2,wg2,wg2)|...|(wgn,wgn,wgn)}
+    #   -> workgroups split up the summation -> faster GR but slower GW
+    "GlobalSplitUWorkGroupMappingRoundRobin":        [False, True],
+    
     # 0=don't use magic div (source only)
     # 1=magic div alg #1.  Slightly faster but limited range (if magic number is 2^32)
     # 2=magic div alg#2.  Slightly slower but handles all unsigned ints up to 2^32
@@ -756,7 +816,10 @@ validParameters = {
     #
     # Formula for wgSerial:
     # wgSerial = wg0 + (wg1 % WorkGroupMapping) * nwg0
-    "WorkGroupMapping":           list(range(0,1024+1)),  # change a workgroup's id so that the all the workgroups on the gpu at a time are hitting L2 cache the best
+    "WorkGroupMapping":           list(range(-1024,1024+1)),  # change a workgroup's id so that the all the workgroups on the gpu at a time are hitting L2 cache the best
+    "WorkGroupMappingXCC":        list(range(1,1024+1)),  # change a workgroup's id so that contiguous workgroup can map on same XCC
+    "WorkGroupMappingXCCGroup":   list(range(0,1024+1)),  # change a workgroup's id so that contiguous workgroup can map on same XCC, remap workgroup in a group of WGMXCCG.
+
     "MaxOccupancy":               list(range(1, 40+1)),       # wg / CU; if cache thrashing is hurting performance, this allocates extra lds to artificially limit occupancy
     "WorkGroup":                  validWorkGroups,      # ( wg0 x wg1 x LocalSplitU ) dimensions of the workgroup which will operate on a tile and share lds
 
@@ -852,9 +915,6 @@ validParameters = {
     # Only support for kernel whose totalVgpr counts less than 256 and gcn that has control bit ACC_CD.
     "MIArchVgpr":               [False, True],
 
-    # alternate implementation for fp16 HPA MFMA
-    "Fp16AltImpl": [False, True],
-
     # Controls desired width (#elements) for loads from global memory -> LDS.
     # and eliminates the pointer unshift logic
     # -1 : Set GlobalReadVectorWidth =  VectorWidth
@@ -933,9 +993,9 @@ validParameters = {
     # is added (readOffset aware of the pad and adjusts offset value based on this parameter value).
     # Only support LdsBlockSizePerPad >= unrollDepth * BPE
     # 0 means disable LdsBlockSizePerPad
-    "LdsBlockSizePerPadA":         [-1, 0, 64, 128, 256, 512, 1024],
-    "LdsBlockSizePerPadB":         [-1, 0, 64, 128, 256, 512, 1024],
-    "LdsBlockSizePerPadMetadata":  [-1, 0, 64, 128, 256, 512, 1024],
+    "LdsBlockSizePerPadA":         [-1, 0, 64, 128, 256, 512, 1024, 2048],
+    "LdsBlockSizePerPadB":         [-1, 0, 64, 128, 256, 512, 1024, 2048],
+    "LdsBlockSizePerPadMetadata":  [-1, 0, 64, 128, 256, 512, 1024, 2048],
 
     # Transpose LDS format. Local store in coalesced dimension , same as optimized global fetch dimension . applicable only in TLU=0 case for miSIMD(s)
     # -1 : keep LDS layout same as global fetch dimension for both A and B
@@ -947,16 +1007,16 @@ validParameters = {
     "TransposeLDS":                [-1, 1, 0, 2],
 
     # add gls or slc after global memory read/writes to change caching, not caching the writes is promising and improved performance a tiny bit
-    # 1: glc, 2: slc, 3: glc+slc
+    # 0: none, 1: glc, 2: slc, 3: glc slc
     # For gfx940, sets sc0/sc1 bits
-    # 0: none, 1: sc0, 2: sc1, 3: sc0 sc1
-    "NonTemporalE":               list(range(0,4)),
-    "NonTemporalD":               list(range(0,4)),
-    "NonTemporalC":               list(range(0,4)),
-    "NonTemporalA":               list(range(0,4)),
-    "NonTemporalB":               list(range(0,4)),
-    "NonTemporalMetadata":        list(range(0,4)),
-    "NonTemporal":                list(range(-1,4)),
+    # 0: none, 1: sc0, 2: sc1, 3: sc0 sc1, 4: nt, 5: nt sc0, 6: nt sc1, 7: nt sc0 sc1
+    "NonTemporalE":               list(range(0,8)),
+    "NonTemporalD":               list(range(0,8)),
+    "NonTemporalC":               list(range(0,8)),
+    "NonTemporalA":               list(range(0,8)),
+    "NonTemporalB":               list(range(0,8)),
+    "NonTemporalMetadata":        list(range(0,8)),
+    "NonTemporal":                list(range(-1,8)),
 
     # Group together unroll iterations inside the unroll loop.
     # For example, InnerUnroll=2 will fetch LDS for two unroll iterations
@@ -1012,6 +1072,12 @@ validParameters = {
 
     # 4:2 Structured Sparse A Matrix, 0=Non Sparse, 1=Sparse Matrix A, 2=Sparse Matrix B
     "Sparse":                      [0, 1, 2],
+
+    # in mix mode F8 need to convert to F16, do this before(0) ds or after(1) ds
+    "ConvertAfterDS":              [False, True],
+
+    # Force disable shadow init to release more sgpr in preloop
+    "ForceDisableShadowInit":      [False, True],
     }
 
 
@@ -1037,6 +1103,7 @@ defaultBenchmarkCommonParameters = [
     {"WaveSeparateGlobalReadA":   [ 0 ] },
     {"WaveSeparateGlobalReadB":   [ 0 ] },
     {"WaveSeparateGlobalReadMetadata":   [ 0 ] },
+    {"UnrollLoopSwapGlobalReadOrder":    [ 0 ] },
     {"PrefetchGlobalRead":        [ 1 ] },
     {"PrefetchLocalRead":         [ 1 ] },
     {"ClusterLocalRead":          [ 1 ] },
@@ -1069,11 +1136,15 @@ defaultBenchmarkCommonParameters = [
     {"MagicDivAlg":               [ 2 ] },
     {"GlobalSplitU":              [ 1 ] },
     {"GlobalSplitUAlgorithm":     [ "MultipleBuffer" ] },
+    {"GlobalSplitUCoalesced":     [ False ] },
+    {"GlobalSplitUWorkGroupMappingRoundRobin":     [ False ] },
     {"Use64bShadowLimit":         [ 1 ] },
     {"NumLoadsCoalescedA":        [ 1 ] },
     {"NumLoadsCoalescedB":        [ 1 ] },
     {"WorkGroup":                 [ [16,16,1]] },
     {"WorkGroupMapping":          [ 8 ] },
+    {"WorkGroupMappingXCC":       [ 1 ] },
+    {"WorkGroupMappingXCCGroup":  [ 0 ] },
     {"ThreadTile":                [ [4,4] ] },
     {"WavefrontSize":             [ 64 ]},
     {"MatrixInstruction":         [ [] ] },
@@ -1086,7 +1157,7 @@ defaultBenchmarkCommonParameters = [
     {"NonTemporalB":              [ 0 ] },
     {"NonTemporalMetadata":       [ 0 ] },
     {"NonTemporal":               [ -1 ] },
-    {"PreloadKernArgs":           [ False ] },
+    {"PreloadKernArgs":           [ True ] },
     {"CustomKernelName":          [ "" ] },
     {"NoReject":                  [ False ]},
     {"MinVgprNumber":             [0]},
@@ -1101,8 +1172,10 @@ defaultBenchmarkCommonParameters = [
     {"ActivationFused":           [ True  ] },
     {"ActivationFuncCall":        [ True  ] },
     {"ActivationAlt":             [ False ] },
-    {"WorkGroupReduction":        [ False ] }
-    ]
+    {"WorkGroupReduction":        [ False ] },
+    {"ConvertAfterDS":            [ False ] },
+    {"ForceDisableShadowInit":    [ False ] }
+]
 
 # dictionary of defaults comprised of default option for each parameter
 defaultSolution = {}
@@ -1123,6 +1196,7 @@ defaultProblemType = {
     "DataTypeA":                0,                # A data type can specified by a variety of ways, such as "s", as listed in SolutionStructs.py::DataType
     "DataTypeB":                0,                # B data type can specified by a variety of ways, such as "s", as listed in SolutionStructs.py::DataType
     "DataTypeE":                0,                # E data type can specified by a variety of ways, such as "s", as listed in SolutionStructs.py::DataType
+    "DataTypeAmaxD":            0,                # AmaxD data type can specified by a variety of ways, such as "s", as listed in SolutionStructs.py::DataType
     "DestDataType":             0,                # destination data types can specified by a variety of ways, such as "s", as listed in SolutionStructs.py::DataType
     "ComputeDataType":          0,                # compute data types can specified by a variety of ways, such as "s", as listed in SolutionStructs.py::DataType
     "F32XdlMathOp":             0,                # reducing intermediate precision from f32 to a specific type, such as "x", as listed in SolutionStructs.py::DataType.
@@ -1130,11 +1204,11 @@ defaultProblemType = {
     "UseBeta":                  True,             # =True use beta parameter (asm will check for B=0 and optimize the write for that), =False don't use beta parameter
     "UseE":                     False,            # =True use output E to output gemm results before activation
     "Gradient":                 False,            # =True set globalWriteElements to gradient mode
-    "UseBias":                  False,            # =True use bias vector
+    "UseBias":                  0,                # =1 support bias vector on M direction, =2 support bias vector on N direction, =3 support bias vector on both M,N direction
     "BiasSrc":                  "D",              # This parameter is used in gradient + bias. Support A, B, D.
-    "UseScaleAB":               False,            # =True use scaleA, scaleB
+    "UseScaleAB":               "",               # Support "", "Scalar", and "Vector"
     "UseScaleCD":               False,            # =True use scaleC, scaleD
-    "UseScaleAlphaVec":         False,            # =True use scaleAlpha vector
+    "UseScaleAlphaVec":         0,                # =1 support alpha vector on M direction, =2 support bias vector on N direction, =3 support alpha vector on both M,N direction
     "HighPrecisionAccumulate":  False,            # f32 += f16*f16
     "SilentHighPrecisionAccumulate": False,       # Keep kernel names the same for HPA mode.  Useful for testing.
 
@@ -1143,6 +1217,8 @@ defaultProblemType = {
     "ComplexConjugateA":        False,            # complex data should be conjugated for "C" transpose case
     "ComplexConjugateB":        False,
 
+    "StochasticRounding":       False,            # By default, IEEE RNE rounding
+
     # for OperationType == GEMM
     "TransposeA":               False,            # =True means transA="T" or "C", =False means transA = "N"
     "TransposeB":               True,
@@ -1203,14 +1279,13 @@ defaultProblemType = {
     # Tile aware solution selection
     "TileAwareSelection":       False,
 
-    # FP16 Alternate Implementation
-    "Fp16AltImpl":              False,
-
     # Activation
     "Activation":               False,
     "ActivationNoGuard":        False,
+    # AmaxD
+    "OutputAmaxD":              False,
     # For kernels putting arguments in workspaces instead of kernel arguments, they can choose to support user arguments input instead.
-    "SupportUserArgs":          False
+    "SupportUserArgs":          True
     }
 
 defaultProblemSizes = [{"Range": [ [2880], 0, 0 ]}]
@@ -1398,6 +1473,25 @@ def printTable(rows):
       print(pad, cell, sep='', end=' ')
     print()
 
+def checkParametersAreValid(param, validParams):
+    """Ensures paramaters in params exist and have valid values as specified by validParames"""
+    (name, values) = param
+    if name == "ProblemSizes":
+        return
+    elif name == "InternalSupportParams":
+        return
+
+    if name not in validParams:
+        printExit("Invalid parameter name: {}\nValid parameters are {}." \
+                .format(name, sorted(validParameters.keys())))
+
+    for value in values:
+        if validParams[name] != -1 and value not in validParams[name]:
+            msgBase = "Invalid parameter value: {} = {}\nValid values for {} are {}{}."
+            msgExt = " (only first 32 combos printed)\nRefer to Common.py for more info" \
+                    if len(validParams[name])>32 else ""
+            printExit(msgBase.format(name, value, name, validParams[name][:32], msgExt))
+
 def printCapTable(parameters):
   import itertools
   archs = [(0,0,0)] + parameters["SupportedISA"]
@@ -1419,10 +1513,13 @@ def printCapTable(parameters):
   printTable([headerRow] + asmCapRows + archCapRows)
 
 def which(p):
-    exes = [p+x for x in ['', '.exe', '.bat']]
-    system_path = os.environ['PATH'].split(os.pathsep)
-    if p == 'hipcc' and 'CMAKE_CXX_COMPILER' in os.environ and os.path.isfile(os.environ['CMAKE_CXX_COMPILER']):
+    if supportedCompiler(p) and 'CMAKE_CXX_COMPILER' in os.environ and os.path.isfile(os.environ['CMAKE_CXX_COMPILER']):
         return os.environ['CMAKE_CXX_COMPILER']
+    if os.name == "nt":
+        exes = [p+x for x in ['.exe', '', '.bat']]  # bat may be front end for file with no extension
+    else:
+        exes = [p+x for x in ['', '.exe', '.bat']]
+    system_path = os.environ['PATH'].split(os.pathsep)
     for dirname in system_path+[globalParameters["ROCmBinPath"]]:
         for exe in exes:
             candidate = os.path.join(os.path.expanduser(dirname), exe)
@@ -1470,6 +1567,8 @@ def assignGlobalParameters( config ):
   globalParameters["CmakeCxxCompiler"] = None
   if "CMAKE_CXX_COMPILER" in os.environ:
     globalParameters["CmakeCxxCompiler"] = os.environ.get("CMAKE_CXX_COMPILER")
+  if "CMAKE_C_COMPILER" in os.environ:
+    globalParameters["CmakeCCompiler"] = os.environ.get("CMAKE_C_COMPILER")
 
   globalParameters["ROCmBinPath"] = os.path.join(globalParameters["ROCmPath"], "bin")
 
@@ -1481,14 +1580,27 @@ def assignGlobalParameters( config ):
 
   if "CxxCompiler" in config:
     globalParameters["CxxCompiler"] = config["CxxCompiler"]
+    # Pair the CCompiler with CxxCompiler
+    if globalParameters["CxxCompiler"] == "hipcc":
+       globalParameters["CCompiler"] = "hipcc"
+    else:
+        if supportedCompiler(globalParameters["CxxCompiler"]):
+          globalParameters["CCompiler"] = "clang" if os.name == "nt" else "amdclang"
+        else: # unkown c++ compiler so set c compile rto be the same
+          globalParameters["CCompiler"] = globalParameters["CxxCompiler"]
+
+  if "CCompiler" in config:
+    globalParameters["CCompiler"] = config["CCompiler"]
 
   if "TENSILE_ROCM_ASSEMBLER_PATH" in os.environ:
     globalParameters["AssemblerPath"] = os.environ.get("TENSILE_ROCM_ASSEMBLER_PATH")
-  elif globalParameters["AssemblerPath"] is None and globalParameters["CxxCompiler"] == "hipcc":
+  elif globalParameters["AssemblerPath"] is None and supportedCompiler(globalParameters["CxxCompiler"]):
     if os.name == "nt":
       globalParameters["AssemblerPath"] = locateExe(globalParameters["ROCmBinPath"], "clang++.exe")
     else:
-      globalParameters["AssemblerPath"] = locateExe(os.path.join(globalParameters["ROCmPath"], "llvm/bin"), "clang++")
+      bin_path = "llvm/bin" if globalParameters["CxxCompiler"] == "hipcc" else "bin"
+      compiler = "clang++" if globalParameters["CxxCompiler"] == "hipcc" else "amdclang++"
+      globalParameters["AssemblerPath"] = locateExe(os.path.join(globalParameters["ROCmPath"], bin_path), compiler)
 
   globalParameters["ROCmSMIPath"] = locateExe(globalParameters["ROCmBinPath"], "rocm-smi")
   globalParameters["ROCmLdPath"]  = locateExe(os.path.join(globalParameters["ROCmPath"], "llvm/bin"), "ld.lld")
@@ -1506,6 +1618,9 @@ def assignGlobalParameters( config ):
   if "ROCmAgentEnumeratorPath" in config:
     globalParameters["ROCmAgentEnumeratorPath"] = config["ROCmAgentEnumeratorPath"]
 
+  if "KeepBuildTmp" in config:
+      globalParameters["KeepBuildTmp"] = config["KeepBuildTmp"]
+
   # read current gfx version
   returncode = detectGlobalCurrentISA()
   if globalParameters["CurrentISA"] == (0,0,0):
@@ -1515,11 +1630,6 @@ def assignGlobalParameters( config ):
       globalParameters["CurrentISA"] = (9,0,6)
       printWarning("Failed to detect ISA so forcing (gfx906) on windows")
 
-  # TODO Remove this when rocm-smi supports gfx940
-  if globalParameters["CurrentISA"] in [(9,4,0), (9,4,1), (9,4,2)]:
-    printWarning("HardwareMonitor currently disabled for gfx940")
-    globalParameters["HardwareMonitor"] = False
-
   globalParameters["AsmCaps"] = {}
   globalParameters["ArchCaps"] = {}
   globalParameters["AsmBugs"] = {}
@@ -1550,6 +1660,8 @@ def assignGlobalParameters( config ):
   # Due to platform.linux_distribution() being deprecated, just try to run dpkg regardless.
   # The alternative would be to install the `distro` package.
   # See https://docs.python.org/3.7/library/platform.html#platform.linux_distribution
+
+  # The following try except block computes the hipcc version
   try:
     if os.name == "nt":
       compileArgs = ['perl'] + [which('hipcc')] + ['--version']
