@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -34,6 +34,7 @@ class KernelWriterConversion(KernelWriterBase):
     super().__init__()
 
     self.state["ProblemType"] = deepcopy(state["ProblemType"])
+    self.state["GenPGRPostKernels"] = state["GenPGRPostKernels"]
     self.state["_GlobalAccumulation"] = state["_GlobalAccumulation"]
     self.state["ActivationFused"] = state["ActivationFused"]
     self.state["GlobalSplitU"] = state["GlobalSplitU"]
@@ -76,6 +77,13 @@ class KernelWriterConversion(KernelWriterBase):
       for idx, arch in enumerate(self.supportedArchs):
         self.supportedArchs[idx] = gfxArch(''.join(map(str, arch)))
 
+    self.gsuKernels = [self.state["GlobalSplitU"]]
+    if self.state["GenPGRPostKernels"]:
+      pgrgsu = int(self.state["GlobalSplitU"] / 2)
+      while pgrgsu > 1:
+        self.gsuKernels.append(pgrgsu)
+        pgrgsu = int(pgrgsu / 2)
+
   def functionArgument(self):
     kStr = ""
 
@@ -118,10 +126,13 @@ class KernelWriterConversion(KernelWriterBase):
       kStr += "  " + scalePtrStr + " * " + "ScaleC;" + self.endLine
       kStr += "  " + scalePtrStr + " * " + "ScaleD;" + self.endLine
 
+    enableFactorDim = False
     # interface: ScaleAlphaVec GSU>1 GSUA "MUL"
     if self.state["ProblemType"]["UseScaleAlphaVec"]:
       scaleAlphaVecPtrStr = self.state["ProblemType"]["ComputeDataType"].toDevice(self.language)
       kStr += "  " + scaleAlphaVecPtrStr + " * " + "ScaleAlphaVec;" + self.endLine
+      if self.state["ProblemType"]["UseScaleAlphaVec"] == 3:
+        enableFactorDim = True
 
     # alpha & beta
     kStr += "  %s alpha;%s" % (self.state["ProblemType"]["ComputeDataType"].toDevice(self.language), self.endLine)
@@ -155,6 +166,8 @@ class KernelWriterConversion(KernelWriterBase):
         (not self.state["ProblemType"]["Gradient"] or \
           (self.state["ProblemType"]["Gradient"] and (self.state["ProblemType"]["BiasSrc"] == "A" or self.state["ProblemType"]["BiasSrc"] == "B"))):
       kStr += "  unsigned int strideBias;%s" % (self.endLine)
+      if self.state["ProblemType"]["UseBias"] == 3:
+        enableFactorDim = True
 
     # sizes
     for i in range(0, self.state["ProblemType"]["NumIndicesC"]):
@@ -165,6 +178,9 @@ class KernelWriterConversion(KernelWriterBase):
     kStr += "  unsigned char gsu;%s" % (self.endLine)
     kStr += "  unsigned char reserved[3];%s" % (self.endLine)
 
+    if enableFactorDim:
+      kStr += "  unsigned int factorDim;%s" % (self.endLine)
+
     # argument structure end
     kStr += "};" + self.endLine
 
@@ -398,10 +414,17 @@ class KernelWriterConversion(KernelWriterBase):
     if self.state["ProblemType"]["UseBias"] and \
        (not self.state["ProblemType"]["Gradient"] or \
          (self.state["ProblemType"]["Gradient"] and (self.state["ProblemType"]["BiasSrc"] == "A" or self.state["ProblemType"]["BiasSrc"] == "B"))):
+
+      id_str = "id0"
+      if self.state["ProblemType"]["UseBias"] == 3:
+        id_str = "idb"
+        kStr += "  %s idb = ( arg.factorDim == 0 ? (%s)id0 : id1);%s" % (self.uint64Str, self.uint64Str, self.endLine)
+      elif self.state["ProblemType"]["UseBias"] == 2:
+        id_str = "id1"
       if problemType["NumIndicesC"] > 2:
-        kStr += "  %s idxBias = GLOBAL_BIAS((%s)id0, id2);%s" % (self.uint64Str, self.uint64Str, self.endLine)
+        kStr += "  %s idxBias = GLOBAL_BIAS((%s)%s, id2);%s" % (self.uint64Str, self.uint64Str, id_str, self.endLine)
       else:
-        kStr += "  %s idxBias = id0;%s" % (self.uint64Str, self.uint64Str, self.endLine)
+        kStr += "  %s idxBias = %s;%s" % (self.uint64Str, id_str, self.endLine)
 
 
     ########################################
@@ -424,7 +447,7 @@ class KernelWriterConversion(KernelWriterBase):
     kStr += "  " + destTypeStr + " result[NUM_ELEMENT_LOAD];" + self.endLine
 
     #Load scaleAB
-    if self.state["ProblemType"]["UseScaleAB"]:
+    if self.state["ProblemType"]["UseScaleAB"] == "Scalar":
       kStr += "  " + intermediateDataType + " scaleA_data, scaleB_data;" + self.endLine
       kStr += "  " + "scaleA_data = arg.ScaleA == nullptr ? 1 : *(arg.ScaleA);" + self.endLine
       kStr += "  " + "scaleB_data = arg.ScaleB == nullptr ? 1 : *(arg.ScaleB);" + self.endLine
@@ -502,9 +525,12 @@ class KernelWriterConversion(KernelWriterBase):
         if globalParameters["AsmCaps"][archTuple]['v_pk_add_f32']:
           canPKF32Arch.append(arch)
       defineStr = []
-      defineStr = "#if defined(__%s__)"%getGfxName(canPKF32Arch[0])
-      for arch in canPKF32Arch[1:]:
-        defineStr += "|| defined(__%s__)"%getGfxName(arch)
+      if len(canPKF32Arch) > 0:
+        defineStr = "#if defined(__%s__)"%getGfxName(canPKF32Arch[0])
+        for arch in canPKF32Arch[1:]:
+          defineStr += "|| defined(__%s__)"%getGfxName(arch)
+      else:
+        defineStr = "#if 0"
       # PGR=2
       kStr += "  %s temp[NUM_GSU];" % loadTypeStr + self.endLine
       for gsuIdx in range(self.state["GlobalSplitU"]):
@@ -589,9 +615,19 @@ class KernelWriterConversion(KernelWriterBase):
     resultStr = "result"
 
     #scaleAB
-    if self.state["ProblemType"]["UseScaleAB"]:
+    if self.state["ProblemType"]["UseScaleAB"] == "Scalar":
       kStr += "  arg.alpha = arg.alpha*scaleA_data*scaleB_data;%s" % (self.endLine)
-    kStr += self.endLine
+      kStr += self.endLine
+    elif self.state["ProblemType"]["UseScaleAB"] == "Vector":
+      kStr += "  if(arg.ScaleA != nullptr) {" + self.endLine
+      for vIdx in range(self.num_dword_load):
+        kStr += "    %s[%d] *= (%s)arg.ScaleA[id0+%d];%s" % (accumStr, vIdx, intermediateDataType, vIdx, self.endLine)
+      kStr += "  }" + self.endLine
+      kStr += "  if(arg.ScaleB != nullptr) {" + self.endLine
+      for vIdx in range(self.num_dword_load):
+        kStr += "      %s[%d] *= (%s)arg.ScaleB[id1];%s" % (accumStr, vIdx, intermediateDataType, self.endLine)
+      kStr += "  }" + self.endLine
+      kStr += self.endLine
 
     #alpha
     for vIdx in range(self.num_dword_load):
@@ -600,9 +636,25 @@ class KernelWriterConversion(KernelWriterBase):
 
     if self.state["ProblemType"]["UseScaleAlphaVec"]:
       kStr += "  if(arg.ScaleAlphaVec != nullptr){" + self.endLine
-      for vIdx in range(self.num_dword_load):
-        kStr += "  %s[%d] *= (%s)arg.ScaleAlphaVec[id0+%d];%s" % (accumStr, vIdx, intermediateDataType, vIdx, self.endLine)
-      kStr += "  }" + self.endLine
+
+      if self.state["ProblemType"]["UseScaleAlphaVec"] == 3:
+        kStr += "    if(arg.factorDim == 0){" + self.endLine
+        for vIdx in range(self.num_dword_load):
+          kStr += "      %s[%d] *= (%s)arg.ScaleAlphaVec[id0+%d];%s" % (accumStr, vIdx, intermediateDataType, vIdx, self.endLine)
+        kStr += "    }else{" + self.endLine
+        for vIdx in range(self.num_dword_load):
+          kStr += "      %s[%d] *= (%s)arg.ScaleAlphaVec[id1];%s" % (accumStr, vIdx, intermediateDataType, self.endLine)
+        kStr += "    }" + self.endLine
+        kStr += "  }" + self.endLine
+      elif self.state["ProblemType"]["UseBias"] == 2:
+        for vIdx in range(self.num_dword_load):
+          kStr += "    %s[%d] *= (%s)arg.ScaleAlphaVec[id1];%s" % (accumStr, vIdx, intermediateDataType, self.endLine)
+        kStr += "  }" + self.endLine
+      else:
+        for vIdx in range(self.num_dword_load):
+          kStr += "    %s[%d] *= (%s)arg.ScaleAlphaVec[id0+%d];%s" % (accumStr, vIdx, intermediateDataType, vIdx, self.endLine)
+        kStr += "  }" + self.endLine
+
       kStr += self.endLine
 
     #scaleC
@@ -620,9 +672,24 @@ class KernelWriterConversion(KernelWriterBase):
     #Bias
     if self.state["ProblemType"]["UseBias"] and (not self.state["ProblemType"]["Gradient"]):
       kStr += "  if(arg.Bias != 0){" + self.endLine
-      for vIdx in range(self.num_dword_load):
-        kStr += "    %s[%d] += (%s)arg.Bias[idxBias+%d];%s" % (accumStr, vIdx, intermediateDataType, vIdx, self.endLine)
-      kStr += "  }" + self.endLine
+      if self.state["ProblemType"]["UseBias"] == 3:
+        kStr += "    if(arg.factorDim == 0){" + self.endLine
+        for vIdx in range(self.num_dword_load):
+          kStr += "      %s[%d] += (%s)arg.Bias[idxBias+%d];%s" % (accumStr, vIdx, intermediateDataType, vIdx, self.endLine)
+        kStr += "    }else{" + self.endLine
+        for vIdx in range(self.num_dword_load):
+          kStr += "      %s[%d] += (%s)arg.Bias[idxBias];%s" % (accumStr, vIdx, intermediateDataType, self.endLine)
+        kStr += "    }" + self.endLine
+        kStr += "  }" + self.endLine
+      elif self.state["ProblemType"]["UseBias"] == 2:
+        for vIdx in range(self.num_dword_load):
+          kStr += "    %s[%d] += (%s)arg.Bias[idxBias];%s" % (accumStr, vIdx, intermediateDataType, self.endLine)
+        kStr += "  }" + self.endLine
+      else:
+        for vIdx in range(self.num_dword_load):
+          kStr += "    %s[%d] += (%s)arg.Bias[idxBias+%d];%s" % (accumStr, vIdx, intermediateDataType, vIdx, self.endLine)
+        kStr += "  }" + self.endLine
+
       kStr += self.endLine
 
     #Handle E
@@ -735,6 +802,12 @@ class KernelWriterConversion(KernelWriterBase):
         name += "_BiasSrc%s"%(self.state["ProblemType"]["BiasSrc"])
       else:
         name += "_Bias%s"%self.state["ProblemType"]["BiasDataType"].toChar()
+
+    factorDim =  0 if self.state["ProblemType"]["Gradient"] else self.state["ProblemType"]["UseBias"]
+    factorDim =  max(factorDim, self.state["ProblemType"]["UseScaleAlphaVec"])
+    if factorDim > 1:
+        name += "_FD%s"%("N" if factorDim == 2 else "MN")
+
     if self.state["ProblemType"]["UseE"]:
       if self.state["ProblemType"]["Gradient"]:
         name += "_Grad%s"%self.state["ProblemType"]["DataTypeE"].toChar()
@@ -748,7 +821,10 @@ class KernelWriterConversion(KernelWriterBase):
         name += "_%s"%str(self.state["ProblemType"]["ActivationType"]).upper()
       name += self.state["ProblemType"]["ActivationComputeDataType"].toChar()
       name += ("ng" if self.state["ProblemType"]["ActivationNoGuard"] else "")
-    name += "_ScaleAB" if self.state["ProblemType"]["UseScaleAB"] else ""
+    if self.state["ProblemType"]["UseScaleAB"] == "Scalar":
+      name += "_ScaleAB"
+    elif self.state["ProblemType"]["UseScaleAB"] == "Vector":
+      name += "_ScaleABVec"
     name += "_ScaleCD" if self.state["ProblemType"]["UseScaleCD"] else ""
     name += "_ScaleAlphaVec" if self.state["ProblemType"]["UseScaleAlphaVec"] else ""
     name += "_PostGSU" + str(self.state["GlobalSplitU"])
@@ -775,9 +851,21 @@ class KernelWriterConversion(KernelWriterBase):
                                                                       self.state["ProblemType"]["ActivationType"])
       fileString += "\n"
 
-    fileString += self.functionArgument()
-    fileString += self.functionSignature()
-    fileString += ";\n"
+    backupGSU    = self.state["GlobalSplitU"]
+    backupUnroll = self.state["UnrollOnly"]
+    for gsu in self.gsuKernels:
+      for toggle in [True, False]:
+        self.state["GlobalSplitU"] = gsu
+        self.state["ProblemType"]["GroupedGemm"] = toggle
+        self.kernelName = self.getKernelName()
+        fileString += self.functionArgument()
+        fileString += self.functionSignature()
+        fileString += ";\n"
+      if not self.state["UnrollOnly"]:
+        self.state["UnrollOnly"] = True
+    self.state["GlobalSplitU"] = backupGSU
+    self.state["UnrollOnly"] = backupUnroll
+
 
     return fileString
 
@@ -789,8 +877,19 @@ class KernelWriterConversion(KernelWriterBase):
       fileString += "#include \"%s.h\"\n" % self.kernelName
       fileString += "\n"
 
-    fileString += self.functionSignature()
-    fileString += self.kernelBody()
+    backupGSU    = self.state["GlobalSplitU"]
+    backupUnroll = self.state["UnrollOnly"]
+    for gsu in self.gsuKernels:
+      for toggle in [True, False]:
+        self.state["GlobalSplitU"] = gsu
+        self.state["ProblemType"]["GroupedGemm"] = toggle
+        self.kernelName = self.getKernelName()
+        fileString += self.functionSignature()
+        fileString += self.kernelBody()
+      if not self.state["UnrollOnly"]:
+        self.state["UnrollOnly"] = True
+    self.state["GlobalSplitU"] = backupGSU
+    self.state["UnrollOnly"] = backupUnroll
 
     return (0, fileString)
 
