@@ -1,6 +1,6 @@
 ################################################################################
 #
-# Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+# Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -23,12 +23,30 @@
 ################################################################################
 
 from .Common import printExit, printWarning, versionIsCompatible
+from .CustomKernels import getCustomKernelConfig
 from .SolutionStructs import Solution, ProblemSizes, ProblemType
 from . import __version__
 from . import Common
 from . import SolutionLibrary
+from .CustomYamlLoader import load_yaml_stream
 
 from typing import NamedTuple, List
+import os
+import sys
+
+try:
+    import orjson as json
+except ImportError:
+    try:
+        import ujson as json
+        printWarning("orjson not installed. Fallback to ujson.")
+    except ImportError:
+        try:
+            import simplejson as json
+            printWarning("orjson, ujson not installed. Fallback to simplejson.")
+        except ImportError:
+            import json
+            printWarning("orjson, ujson, simplejson not installed. Fallback to json.")
 
 try:
     import yaml
@@ -56,10 +74,12 @@ def write(filename_noExt, data, format="yaml"):
     """Writes data to file with specified format; extension is appended based on format."""
     if format == "yaml":
         writeYAML(filename_noExt + ".yaml", data)
+    elif format == "json":
+        writeJson(filename_noExt + ".json", data)
     elif format == "msgpack":
         writeMsgPack(filename_noExt + ".dat", data)
     else:
-        printExit("Unrecognized format {}".format(format))
+        printExit("Unrecognized write format {}".format(format))
 
 
 def writeYAML(filename, data, **kwargs):
@@ -75,6 +95,11 @@ def writeYAML(filename, data, **kwargs):
     with open(filename, "w") as f:
         yaml.dump(data, f, **kwargs)
 
+def writeJson(filename, data):
+    """Writes data to file in json format."""
+    with open(filename, "w") as f:
+        json_object = json.dumps(data, option=json.OPT_INDENT_2).decode("utf-8") if 'orjson' in sys.modules else json.dumps(data, indent=2)
+        f.write(json_object)
 
 def writeMsgPack(filename, data):
     """Writes data to file in Message Pack format."""
@@ -88,7 +113,7 @@ def writeSolutions(filename, problemSizes, biasTypeArgs, activationArgs, solutio
     solutionStates = []
 
     if cache:
-        solYaml = readYAML(filename)
+        solYaml = read(filename)
         if biasTypeArgs and activationArgs:
             solutionStates = solYaml[4:]
         elif biasTypeArgs or activationArgs:
@@ -107,6 +132,8 @@ def writeSolutions(filename, problemSizes, biasTypeArgs, activationArgs, solutio
                     solutionState["ProblemType"]["DataTypeB"].value
             solutionState["ProblemType"]["DataTypeE"] = \
                     solutionState["ProblemType"]["DataTypeE"].value
+            solutionState["ProblemType"]["DataTypeAmaxD"] = \
+                    solutionState["ProblemType"]["DataTypeAmaxD"].value
             solutionState["ProblemType"]["DestDataType"] = \
                     solutionState["ProblemType"]["DestDataType"].value
             solutionState["ProblemType"]["ComputeDataType"] = \
@@ -119,6 +146,9 @@ def writeSolutions(filename, problemSizes, biasTypeArgs, activationArgs, solutio
                     solutionState["ProblemType"]["ActivationType"].value
             solutionState["ProblemType"]["F32XdlMathOp"] = \
                 solutionState["ProblemType"]["F32XdlMathOp"].value
+            if "DataTypeMetadata" in solutionState["ProblemType"]:
+                solutionState["ProblemType"]["DataTypeMetadata"] = \
+                    solutionState["ProblemType"]["DataTypeMetadata"].value
             solutionStates.append(solutionState)
     # write dictionaries
     with open(filename, "w") as f:
@@ -142,16 +172,30 @@ def writeSolutions(filename, problemSizes, biasTypeArgs, activationArgs, solutio
 ###############################
 # Reading and parsing functions
 ###############################
+def read(filename, customizedLoader=False):
+    name, extension = os.path.splitext(filename)
+    if extension == ".yaml":
+        return load_yaml_stream(filename, yamlLoader) if customizedLoader else readYAML(filename)
+    if extension == ".json":
+        return readJson(filename)
+    else:
+        printExit("Unrecognized read format {}".format(extension))
+
 def readYAML(filename):
     """Reads and returns YAML data from file."""
     with open(filename, "r") as f:
         data = yaml.load(f, yamlLoader)
     return data
 
+def readJson(filename):
+    """Reads and returns JSON data from file."""
+    with open(filename, "r") as f:
+        data = json.loads(f.read())
+    return data
 
 def parseSolutionsFile(filename):
     """Wrapper function to read and parse a solutions file."""
-    return parseSolutionsData(readYAML(filename), filename)
+    return parseSolutionsData(read(filename), filename)
 
 
 def parseSolutionsData(data, srcFile="?"):
@@ -201,7 +245,7 @@ class LibraryLogic(NamedTuple):
 
 def parseLibraryLogicFile(filename, archs=None):
     """Wrapper function to read and parse a library logic file."""
-    return parseLibraryLogicData(readYAML(filename), filename, archs)
+    return parseLibraryLogicData(read(filename, True), filename, archs)
 
 
 def parseLibraryLogicData(data, srcFile="?", archs=None):
@@ -222,9 +266,6 @@ def parseLibraryLogicData(data, srcFile="?", archs=None):
     if "CUCount" not in data:
         data["CUCount"] = None
 
-    if "Fp16AltImpl" not in data:
-        data["Fp16AltImpl"] = False
-
     if not versionIsCompatible(data["MinimumRequiredVersion"]):
         printWarning("Version = {} in library logic file {} does not match Tensile version = {}" \
                 .format(srcFile, data["MinimumRequiredVersion"], __version__) )
@@ -242,6 +283,13 @@ def parseLibraryLogicData(data, srcFile="?", archs=None):
         # force redo the deriving of parameters, make sure old version logic yamls can be validated
         solutionState["AssignedProblemIndependentDerivedParameters"] = False
         solutionState["AssignedDerivedParameters"] = False
+        if solutionState["CustomKernelName"]:
+            isp = {}
+            if "InternalSupportParams" in solutionState:
+                isp = solutionState["InternalSupportParams"]
+            customConfig = getCustomKernelConfig(solutionState["CustomKernelName"], isp)
+            for key, value in customConfig.items():
+                solutionState[key] = value
         solutionObject = Solution(solutionState)
         if solutionObject["ProblemType"] != problemType:
             printExit(f"ProblemType in library logic file {srcFile} doesn't match solution: {problemType} != {solutionObject['ProblemType']}")
@@ -249,7 +297,7 @@ def parseLibraryLogicData(data, srcFile="?", archs=None):
 
     solutions = [solutionStateToSolution(solutionState) for solutionState in data["Solutions"]]
 
-    newLibrary = SolutionLibrary.MasterSolutionLibrary.FromOriginalState(data, solutions)
+    newLibrary, _ = SolutionLibrary.MasterSolutionLibrary.FromOriginalState(data, solutions)
 
     return LibraryLogic(data["ScheduleName"], data["ArchitectureName"], problemType, solutions, \
             data.get("ExactLogic"), newLibrary, srcFile)
@@ -283,13 +331,10 @@ def parseLibraryLogicList(data, srcFile="?"):
     if len(data) > 10 and data[10]:
         rv["PerfMetric"] = data[10]
 
-    if len(data) > 11 and data[11]:
-        rv["Fp16AltImpl"] = True
-
     # library logic fields
     libraryType = None
-    if len(data) > 12 and data[12]:
-        libraryType = data[12]
+    if len(data) > 11 and data[11]:
+        libraryType = data[11]
     else:
         printExit("Library logic file {} is missing required field matching property." \
                 .format(srcFile))
@@ -364,6 +409,8 @@ def createLibraryLogic(schedulePrefix, architectureName, deviceNames, libraryTyp
             problemTypeState["DataTypeB"].value
     problemTypeState["DataTypeE"] = \
             problemTypeState["DataTypeE"].value
+    problemTypeState["DataTypeAmaxD"] = \
+            problemTypeState["DataTypeAmaxD"].value
     problemTypeState["DestDataType"] = \
             problemTypeState["DestDataType"].value
     problemTypeState["ComputeDataType"] = \
@@ -376,6 +423,9 @@ def createLibraryLogic(schedulePrefix, architectureName, deviceNames, libraryTyp
             problemTypeState["ActivationType"].value
     problemTypeState["F32XdlMathOp"] = \
             problemTypeState["F32XdlMathOp"].value
+    if "DataTypeMetadata" in problemTypeState:
+        problemTypeState["DataTypeMetadata"] = \
+                problemTypeState["DataTypeMetadata"].value
     data.append(problemTypeState)
     # solutions
     solutionList = []
@@ -390,6 +440,8 @@ def createLibraryLogic(schedulePrefix, architectureName, deviceNames, libraryTyp
                 solutionState["ProblemType"]["DataTypeB"].value
         solutionState["ProblemType"]["DataTypeE"] = \
                 solutionState["ProblemType"]["DataTypeE"].value
+        solutionState["ProblemType"]["DataTypeAmaxD"] = \
+                solutionState["ProblemType"]["DataTypeAmaxD"].value
         solutionState["ProblemType"]["DestDataType"] = \
                 solutionState["ProblemType"]["DestDataType"].value
         solutionState["ProblemType"]["ComputeDataType"] = \
@@ -402,6 +454,9 @@ def createLibraryLogic(schedulePrefix, architectureName, deviceNames, libraryTyp
                 solutionState["ProblemType"]["ActivationType"].value
         solutionState["ProblemType"]["F32XdlMathOp"] = \
                 solutionState["ProblemType"]["F32XdlMathOp"].value
+        if "DataTypeMetadata" in solutionState["ProblemType"]:
+            solutionState["ProblemType"]["DataTypeMetadata"] = \
+                    solutionState["ProblemType"]["DataTypeMetadata"].value
         solutionList.append(solutionState)
 
     if tileSelection:
@@ -417,6 +472,8 @@ def createLibraryLogic(schedulePrefix, architectureName, deviceNames, libraryTyp
                     solutionState["ProblemType"]["DataTypeB"].value
             solutionState["ProblemType"]["DataTypeE"] = \
                     solutionState["ProblemType"]["DataTypeE"].value
+            solutionState["ProblemType"]["DataTypeAmaxD"] = \
+                    solutionState["ProblemType"]["DataTypeAmaxD"].value
             solutionState["ProblemType"]["DestDataType"] = \
                     solutionState["ProblemType"]["DestDataType"].value
             solutionState["ProblemType"]["ComputeDataType"] = \
@@ -429,6 +486,9 @@ def createLibraryLogic(schedulePrefix, architectureName, deviceNames, libraryTyp
                     solutionState["ProblemType"]["ActivationType"].value
             solutionState["ProblemType"]["F32XdlMathOp"] = \
                 solutionState["ProblemType"]["F32XdlMathOp"].value
+            if "DataTypeMetadata" in solutionState["ProblemType"]:
+                solutionState["ProblemType"]["DataTypeMetadata"] = \
+                    solutionState["ProblemType"]["DataTypeMetadata"].value
             solutionList.append(solutionState)
 
     data.append(solutionList)
@@ -456,6 +516,5 @@ def createLibraryLogic(schedulePrefix, architectureName, deviceNames, libraryTyp
         data.append(None)
 
     data.append(logicTuple[7]) # PerfMetric
-    data.append(logicTuple[8]) # Fp16AltImpl
     data.append(libraryType) # LibraryType
     return data
