@@ -594,18 +594,19 @@ class SoftmaxKernelGenerator:
             self.vgpr_pool.checkIn(local_offset_byte_offset_reg_idx)
         return mod
 
-def kernel_rodata(name: str):
-    return f'''
-.rodata
-.p2align 6
-.amdhsa_kernel {name}
-.amdhsa_user_sgpr_kernarg_segment_ptr 1
-.amdhsa_system_sgpr_workgroup_id_x 1
-.amdhsa_accum_offset 8
-.amdhsa_next_free_vgpr .amdgcn.next_free_vgpr
-.amdhsa_next_free_sgpr .amdgcn.next_free_sgpr
-.end_amdhsa_kernel
-'''
+def kernel_rodata(name: str, gfx_arch: Tuple[int, int, int]):
+    header = ""
+    header += f'.rodata\n'
+    header += f'.p2align 6\n'
+    header += f'.amdhsa_kernel {name}\n'
+    header += f'.amdhsa_user_sgpr_kernarg_segment_ptr 1\n'
+    header += f'.amdhsa_system_sgpr_workgroup_id_x 1\n'
+    if gfx_arch == (9, 0, 10) or (gfx_arch > (9, 4) and gfx_arch < (10, 0, 0)):
+        header += f'.amdhsa_accum_offset 8\n'
+    header += f'.amdhsa_next_free_vgpr .amdgcn.next_free_vgpr\n'
+    header += f'.amdhsa_next_free_sgpr .amdgcn.next_free_sgpr\n'
+    header += f'.end_amdhsa_kernel\n'
+    return header
 
 @dataclass
 class KernelArgument:
@@ -704,7 +705,7 @@ if __name__ == '__main__':
     meta.update_args_offsets()
     k_str = '\n'.join([kernel_header(func_name, arch),
                        str(kernel_body),
-                       kernel_rodata(func_name),
+                       kernel_rodata(func_name, isa),
                        meta_str((meta,))])
 
     with open(output_path, 'w') as f:
