@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,6 +32,7 @@
 #include <Tensile/hip/HipUtils.hpp>
 
 #include "ClientProblemFactory.hpp"
+#include "Rotating.hpp"
 
 #include <cstddef>
 #include <random>
@@ -338,6 +339,23 @@ namespace Tensile
                                m_groupedOffsets,
                                problem,
                                hipMemcpyDeviceToDevice);
+                    if(m_rotatingMode == 1 && m_rotatingBuffer > 0)
+                    {
+                        auto mem = m_rm->getRotatingMemory();
+                        // init mode 1 rotating data
+                        for(size_t j = 1; j < mem.size(); j++)
+                            for(size_t i = 0; i < m_vdata.size(); i++)
+                            {
+                                auto& desc = problem.tensors()[i];
+                                auto  it   = m_vdata[i].pristine.find(desc.dataType());
+                                if(it != m_vdata[i].pristine.end())
+                                {
+                                    auto& p = it->second;
+                                    if(i <= ContractionProblemGemm::TENSOR::METADATA)
+                                        HIP_CHECK_EXC(hipMemcpy(mem[j][i].data.get(), p.gpuInput.current.get(), mem[j][i].size, hipMemcpyDeviceToDevice));
+                                }
+                            }
+                    }
                     m_gpuInit = true;
                 }
                 initializeGPUBatchedInputs(problem);
@@ -351,7 +369,8 @@ namespace Tensile
             std::vector<std::shared_ptr<ProblemInputs>>
                 prepareRotatingGPUOutput(int32_t                        maxRotatingBufferNum,
                                          ContractionProblem const*      problem,
-                                         std::shared_ptr<ProblemInputs> inputs);
+                                         std::shared_ptr<ProblemInputs> inputs,
+                                         hipStream_t                    stream);
 
             template <typename S>
             void initArray(DataType dataType, InitMode initMode, void* array, S descriptor)
@@ -983,9 +1002,9 @@ namespace Tensile
             /// cannot be used with problem dependent data.
             bool m_problemDependentData = false;
 
-            int64_t               m_rotatingBuffer        = 0;
-            int64_t               m_rotatingAllocatedSize = 0;
-            std::shared_ptr<void> m_rotatingPointer       = nullptr;
+            int64_t               m_rotatingBuffer          = 0;
+            std::shared_ptr<RotatingMemory> m_rm;
+            int32_t                         m_rotatingMode = 0;
         };
 
         template <>
@@ -2005,6 +2024,12 @@ namespace Tensile
             return std::isinf(static_cast<float>(value));
         }
 
+        template <>
+        inline bool DataInitialization::isBadOutput<BFloat8>(BFloat8 value)
+        {
+            return std::isinf(static_cast<float>(value));
+        }
+
         template <>
         inline bool DataInitialization::isBadOutput<BFloat16>(BFloat16 value)
         {
