@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -31,6 +31,9 @@
 #include "TypedId.hpp"
 
 #include <cstddef>
+#include <omp.h>
+
+#define MAX_OMP_THREADS 64
 
 namespace Tensile
 {
@@ -445,6 +448,11 @@ namespace Tensile
                 float tmp  = 0.5 * tanh(xx) + x1 * x2 + 0.5;
                 return static_cast<T>(tmp);
             }
+            else if(new_type == ActivationType::Silu)
+            {
+                auto castedVal = static_cast<castT>(val);
+                return static_cast<T>(castedVal / (1.f + static_cast<castT>(exp(-castedVal))));
+            }
             return val;
         }
 
@@ -525,6 +533,7 @@ namespace Tensile
             // For 2D bias reduction, d batch = 1
             if((tensor.dimensions() == 3 && tensor.sizes()[2] == 1) || tensor.dimensions() == 2)
             {
+omp_set_num_threads(MAX_OMP_THREADS);
 #pragma omp parallel for
                 for(size_t bNum = 0; bNum < biasTensor.totalLogicalElements();
                     bNum += validationStride)
@@ -670,7 +679,13 @@ namespace Tensile
                 }
             }
 
+            Accumulator    amaxD(0);
+            Accumulator    negOne(-1);
+            constexpr bool notCmplxAmaxD = !std::is_same<Accumulator, std::complex<double>>()
+                                           && !std::is_same<Accumulator, std::complex<float>>();
+
             // gemm
+omp_set_num_threads(MAX_OMP_THREADS);
 #pragma omp parallel for
             for(size_t dNum = 0; dNum < d.totalLogicalElements(); dNum += validationStrideGemm)
             {
@@ -761,34 +776,55 @@ namespace Tensile
                                          && sizeof(typename Inputs::BType)
                                                 > sizeof(typename Inputs::ComputeInputType))
                             {
-                                typename Inputs::ComputeInputType aValCast, bValCast;
-                                if(problem.useScaleAB())
+                                if(std::is_same<Float8BFloat8,
+                                                typename Inputs::ComputeInputType>::value)
                                 {
-                                    Accumulator scaleA = GetValue<Accumulator>(
-                                        problem.alphaType(), inputs.scaleA, 0, aConjugate);
-                                    auto tmp = div<Accumulator>(aVal, scaleA);
-                                    aValCast = static_cast<typename Inputs::ComputeInputType>(tmp);
-                                    Accumulator scaleB = GetValue<Accumulator>(
-                                        problem.alphaType(), inputs.scaleB, 0, aConjugate);
-                                    tmp      = div<Accumulator>(bVal, scaleB);
-                                    bValCast = static_cast<typename Inputs::ComputeInputType>(tmp);
+                                    auto aValCast = static_cast<Tensile::Float8>(aVal);
+                                    auto bValCast = static_cast<Tensile::BFloat8>(bVal);
+                                    value += multiply<Accumulator, MathOpAccum>(aValCast, bValCast);
+                                }
+                                else if(std::is_same<BFloat8Float8,
+                                                     typename Inputs::ComputeInputType>::value)
+                                {
+                                    auto aValCast = static_cast<Tensile::BFloat8>(aVal);
+                                    auto bValCast = static_cast<Tensile::Float8>(bVal);
+                                    value += multiply<Accumulator, MathOpAccum>(aValCast, bValCast);
                                 }
                                 else
                                 {
-                                    aValCast = static_cast<typename Inputs::ComputeInputType>(aVal);
-                                    bValCast = static_cast<typename Inputs::ComputeInputType>(bVal);
+                                    typename Inputs::ComputeInputType aValCast, bValCast;
+                                    if(problem.useScaleAB() == "Scalar")
+                                    {
+                                        Accumulator scaleA = GetValue<Accumulator>(
+                                            problem.alphaType(), inputs.scaleA, 0, aConjugate);
+                                        auto tmp = multiply<Accumulator>(aVal, scaleA);
+                                        aValCast
+                                            = static_cast<typename Inputs::ComputeInputType>(tmp);
+                                        Accumulator scaleB = GetValue<Accumulator>(
+                                            problem.alphaType(), inputs.scaleB, 0, aConjugate);
+                                        tmp = multiply<Accumulator>(bVal, scaleB);
+                                        bValCast
+                                            = static_cast<typename Inputs::ComputeInputType>(tmp);
+                                    }
+                                    else
+                                    {
+                                        aValCast
+                                            = static_cast<typename Inputs::ComputeInputType>(aVal);
+                                        bValCast
+                                            = static_cast<typename Inputs::ComputeInputType>(bVal);
+                                    }
+                                    value += multiply<Accumulator, MathOpAccum>(aValCast, bValCast);
                                 }
-                                value += multiply<Accumulator, MathOpAccum>(aValCast, bValCast);
                             }
                             else if constexpr(sizeof(typename Inputs::AType)
                                               > sizeof(typename Inputs::ComputeInputType))
                             {
                                 typename Inputs::ComputeInputType aValCast;
-                                if(problem.useScaleAB())
+                                if(problem.useScaleAB() == "Scalar")
                                 {
                                     Accumulator scaleA = GetValue<Accumulator>(
                                         problem.alphaType(), inputs.scaleA, 0, aConjugate);
-                                    auto tmp = div<Accumulator>(aVal, scaleA);
+                                    auto tmp = multiply<Accumulator>(aVal, scaleA);
                                     aValCast = static_cast<typename Inputs::ComputeInputType>(tmp);
                                 }
                                 else
@@ -801,11 +837,11 @@ namespace Tensile
                                               > sizeof(typename Inputs::ComputeInputType))
                             {
                                 typename Inputs::ComputeInputType bValCast;
-                                if(problem.useScaleAB())
+                                if(problem.useScaleAB() == "Scalar")
                                 {
                                     Accumulator scaleB = GetValue<Accumulator>(
                                         problem.alphaType(), inputs.scaleB, 0, aConjugate);
-                                    auto tmp = div<Accumulator>(bVal, scaleB);
+                                    auto tmp = multiply<Accumulator>(bVal, scaleB);
                                     bValCast = static_cast<typename Inputs::ComputeInputType>(tmp);
                                 }
                                 else
@@ -830,20 +866,46 @@ namespace Tensile
                 Accumulator beta  = constVariantCast<Accumulator>(inputs.beta);
                 auto        zero  = static_cast<Accumulator>(0);
 
-                if(problem.useScaleAB())
+                if(problem.useScaleAB() == "Scalar")
                 {
                     Accumulator scaleA
                         = GetValue<Accumulator>(problem.alphaType(), inputs.scaleA, 0, aConjugate);
                     Accumulator scaleB
                         = GetValue<Accumulator>(problem.alphaType(), inputs.scaleB, 0, aConjugate);
-                    alpha *= scaleA * scaleB;
+                    if constexpr(sizeof(typename Inputs::AType)
+                                 <= sizeof(typename Inputs::ComputeInputType))
+                        alpha *= scaleA;
+
+                    if constexpr(sizeof(typename Inputs::BType)
+                                 <= sizeof(typename Inputs::ComputeInputType))
+                        alpha *= scaleB;
+                }
+                else if(problem.useScaleAB() == "Vector")
+                {
+                    auto posB = int(int(dNum / problem.d().sizes()[0]) % problem.d().sizes()[1]);
+                    auto posA = int(dNum % problem.d().sizes()[0]);
+                    Accumulator scaleA = GetValue<Accumulator>(
+                        problem.alphaType(), inputs.scaleA, posA, aConjugate);
+                    Accumulator scaleB = GetValue<Accumulator>(
+                        problem.alphaType(), inputs.scaleB, posB, aConjugate);
+                    if constexpr(sizeof(typename Inputs::AType)
+                                 <= sizeof(typename Inputs::ComputeInputType))
+                        alpha *= scaleA;
+
+                    if constexpr(sizeof(typename Inputs::BType)
+                                 <= sizeof(typename Inputs::ComputeInputType))
+                        alpha *= scaleB;
                 }
 
                 auto resultD = multiply<Accumulator>(alpha, value);
 
                 if(problem.useScaleAlphaVec())
                 {
-                    int         pos           = int(dNum % problem.d().sizes()[0]);
+                    int pos = 0;
+                    if(problem.getParams().factorDim())
+                        pos = int(int(dNum / problem.d().sizes()[0]) % problem.d().sizes()[1]);
+                    else
+                        pos = int(dNum % problem.d().sizes()[0]);
                     Accumulator scaleAlphaVec = GetValue<Accumulator>(
                         problem.alphaType(), inputs.scaleAlphaVec, pos, aConjugate);
                     resultD *= scaleAlphaVec;
@@ -865,9 +927,14 @@ namespace Tensile
                 // bias
                 if(problem.useBias() && inputs.bias && !problem.useGradient())
                 {
-                    auto        biasIndex = problem.bias().index(biasCoord);
-                    int         pos       = int(dNum % problem.d().sizes()[0]) + biasIndex;
-                    Accumulator bias      = GetValue<Accumulator>(
+                    auto biasIndex = problem.bias().index(biasCoord);
+                    int  pos       = 0;
+                    if(problem.getParams().factorDim())
+                        pos = int(int(dNum / problem.d().sizes()[0]) % problem.d().sizes()[1])
+                              + biasIndex;
+                    else
+                        pos = int(dNum % problem.d().sizes()[0]) + biasIndex;
+                    Accumulator bias = GetValue<Accumulator>(
                         problem.bias().dataType(), inputs.bias, pos, aConjugate);
                     resultD += bias;
                 }
@@ -914,6 +981,20 @@ namespace Tensile
                                          actArgs);
                 }
 
+omp_set_num_threads(MAX_OMP_THREADS);
+#pragma omp critical
+                {
+                    if constexpr(notCmplxAmaxD)
+                    {
+                        if(problem.outputAmaxD())
+                        {
+                            Accumulator absResultD = (resultD > zero) ? resultD : resultD * negOne;
+                            if(absResultD > amaxD)
+                                amaxD = absResultD;
+                        }
+                    }
+                }
+
                 if(problem.useScaleCD())
                 {
                     Accumulator scaleD
@@ -928,6 +1009,15 @@ namespace Tensile
                 dPtr[dIndex] = SaturateCast<typename Inputs::DType>(resultD);
             }
 
+            if(problem.outputAmaxD())
+            {
+                SetValue<Accumulator>(
+                    problem.tensors()[ContractionProblemGemm::TENSOR::AMAXD].dataType(),
+                    amaxD,
+                    inputs.amaxD,
+                    0);
+            }
+
             if(problem.useGradient() && problem.useBias())
             {
                 auto& biasTensor = problem.tensor(ContractionProblemGemm::TENSOR::BIAS);
@@ -1150,6 +1240,24 @@ namespace Tensile
                 return ReferenceSolution<TypedGemm_B_S_S>::SolveCPU(
                     problem, inputs, elementsToValidate);
             }
+            case TypedGemm_H_B_H_S::TypeId():
+            {
+                if(isHPA)
+                {
+                    return ReferenceSolution<TypedGemm_H_B_H_S, float>::SolveCPU(
+                        problem, inputs, elementsToValidate);
+                }
+                else
+                {
+                    return ReferenceSolution<TypedGemm_H_B_H_S>::SolveCPU(
+                        problem, inputs, elementsToValidate);
+                }
+            }
+            case TypedGemm_I8_B_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_I8_B_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
 #endif // TENSILE_USE_BF16
 #ifdef TENSILE_USE_FP8_BF8
             case TypedGemm_F8_S_S::TypeId():
@@ -1162,16 +1270,41 @@ namespace Tensile
                 return ReferenceSolution<TypedGemm_F8_H_S, float>::SolveCPU(
                     problem, inputs, elementsToValidate);
             }
+            case TypedGemm_F8_B_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8_B_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
             case TypedGemm_F8_F8_S::TypeId():
             {
                 return ReferenceSolution<TypedGemm_F8_F8_S, float>::SolveCPU(
                     problem, inputs, elementsToValidate);
             }
+            case TypedGemm_F8_B8_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8_B8_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
             case TypedGemm_B8_S_S::TypeId():
             {
                 return ReferenceSolution<TypedGemm_B8_S_S, float>::SolveCPU(
                     problem, inputs, elementsToValidate);
             }
+            case TypedGemm_B8_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8_B_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8_B_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_B8_F8_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8_F8_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
             case TypedGemm_B8_B8_S::TypeId():
             {
                 return ReferenceSolution<TypedGemm_B8_B8_S, float>::SolveCPU(
@@ -1188,11 +1321,21 @@ namespace Tensile
                 return ReferenceSolution<TypedGemm_F8B8_H_S, float>::SolveCPU(
                     problem, inputs, elementsToValidate);
             }
+            case TypedGemm_F8B8_B_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8B8_B_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
             case TypedGemm_F8B8_F8_S::TypeId():
             {
                 return ReferenceSolution<TypedGemm_F8B8_F8_S, float>::SolveCPU(
                     problem, inputs, elementsToValidate);
             }
+            case TypedGemm_F8B8_B8_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_F8B8_B8_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
             case TypedGemm_B8F8_S_S::TypeId():
             {
                 return ReferenceSolution<TypedGemm_B8F8_S_S, float>::SolveCPU(
@@ -1203,14 +1346,9 @@ namespace Tensile
                 return ReferenceSolution<TypedGemm_B8F8_H_S, float>::SolveCPU(
                     problem, inputs, elementsToValidate);
             }
-            case TypedGemm_B8F8_B8_S::TypeId():
+            case TypedGemm_B8F8_B_S::TypeId():
             {
-                return ReferenceSolution<TypedGemm_B8F8_B8_S, float>::SolveCPU(
-                    problem, inputs, elementsToValidate);
-            }
-            case TypedGemm_F8B8_B8_S::TypeId():
-            {
-                return ReferenceSolution<TypedGemm_F8B8_B8_S, float>::SolveCPU(
+                return ReferenceSolution<TypedGemm_B8F8_B_S, float>::SolveCPU(
                     problem, inputs, elementsToValidate);
             }
             case TypedGemm_B8F8_F8_S::TypeId():
@@ -1218,7 +1356,32 @@ namespace Tensile
                 return ReferenceSolution<TypedGemm_B8F8_F8_S, float>::SolveCPU(
                     problem, inputs, elementsToValidate);
             }
+            case TypedGemm_B8F8_B8_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_B8F8_B8_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_H_F8B8_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_H_F8B8_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_H_B8F8_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_H_B8F8_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
 #ifdef TENSILE_USE_HALF
+            case TypedGemm_H_F8_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_H_F8_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
+            case TypedGemm_H_B8_H_S::TypeId():
+            {
+                return ReferenceSolution<TypedGemm_H_B8_H_S, float>::SolveCPU(
+                    problem, inputs, elementsToValidate);
+            }
             case TypedGemm_HF8_H_S_S::TypeId():
             {
                 return ReferenceSolution<TypedGemm_HF8_H_S_S, float>::SolveCPU(
