@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -47,15 +47,16 @@ namespace Tensile
             m_printValids        = args["print-valids"].as<bool>();
             m_printMax           = args["print-max"].as<int>();
 
-            m_printTensorA    = args["print-tensor-a"].as<bool>();
-            m_printTensorB    = args["print-tensor-b"].as<bool>();
-            m_printTensorC    = args["print-tensor-c"].as<bool>();
-            m_printTensorD    = args["print-tensor-d"].as<bool>();
-            m_printTensorRef  = args["print-tensor-ref"].as<bool>();
-            m_printTensorBias = args["print-tensor-bias"].as<bool>();
+            m_printTensorA     = args["print-tensor-a"].as<bool>();
+            m_printTensorB     = args["print-tensor-b"].as<bool>();
+            m_printTensorC     = args["print-tensor-c"].as<bool>();
+            m_printTensorD     = args["print-tensor-d"].as<bool>();
+            m_printTensorRef   = args["print-tensor-ref"].as<bool>();
+            m_printTensorBias  = args["print-tensor-bias"].as<bool>();
+            m_printTensorAmaxD = args["print-tensor-amaxd"].as<bool>();
 
             m_printAny = m_printTensorA || m_printTensorB || m_printTensorC || m_printTensorD
-                         || m_printTensorRef || m_printTensorBias;
+                         || m_printTensorRef || m_printTensorBias || m_printTensorAmaxD;
 
             m_enabled = m_elementsToValidate != 0 || m_printAny;
         }
@@ -229,6 +230,16 @@ namespace Tensile
                                        validationStride);
             }
             break;
+            case DataType::BFloat8:
+            {
+                rv = checkResultsTyped(tensor,
+                                       (BFloat8 const*)refPtr,
+                                       (BFloat8 const*)resPtr,
+                                       maxElements,
+                                       isgpu,
+                                       validationStride);
+            }
+            break;
             case DataType::Int8x4:
             {
                 throw std::runtime_error("Unsupported validator data type Int8x4 for output.");
@@ -366,6 +377,17 @@ namespace Tensile
                     refPtr = reference.scaleAlphaVec;
                     resPtr = result.scaleAlphaVec;
                 }
+                case ContractionProblemGemm::TENSOR::Synchronizer:
+                {
+                    refPtr = reference.Synchronizer;
+                    resPtr = result.Synchronizer;
+                }
+                case ContractionProblemGemm::TENSOR::AMAXD:
+                {
+                    refPtr = reference.amaxD;
+                    resPtr = result.amaxD;
+                }
+
                 break;
                 default:
                     throw std::runtime_error("Unrecognized output tensor.");
@@ -423,6 +445,9 @@ namespace Tensile
             if(m_printTensorBias)
                 requiredBufferSize
                     = std::max(requiredBufferSize, problem.bias().totalAllocatedBytes());
+            if(m_printTensorAmaxD)
+                requiredBufferSize
+                    = std::max(requiredBufferSize, problem.amaxd().totalAllocatedBytes());
 
             if(m_cpuResultBufferSize < requiredBufferSize)
                 allocateResultBuffer(requiredBufferSize);
@@ -529,6 +554,24 @@ namespace Tensile
                                       problem.bias(),
                                       result.bias);
             }
+
+            if(m_printTensorAmaxD)
+            {
+                HIP_CHECK_EXC(hipMemcpy(m_cpuResultBuffer.get(),
+                                        result.amaxD,
+                                        problem.amaxd().totalAllocatedBytes(),
+                                        hipMemcpyDeviceToHost));
+                m_reporter->logTensor(LogLevel::Verbose,
+                                      "AmaxD Ref",
+                                      reference.amaxD,
+                                      problem.amaxd(),
+                                      reference.amaxD);
+                m_reporter->logTensor(LogLevel::Verbose,
+                                      "AmaxD GPU",
+                                      m_cpuResultBuffer.get(),
+                                      problem.amaxd(),
+                                      result.amaxD);
+            }
         }
 
         template <typename ValidType>
