@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -230,8 +230,8 @@ namespace Tensile
                     // M/MT0 x N/MT1 x NumElementsPerThread/StoreVectorWidth x x Wavenumbers
                     bool ret = (std::ceil(static_cast<float>(problem.freeSizeA(0)) / value[0])
                                 * std::ceil(static_cast<float>(problem.freeSizeB(0)) / value[1]))
-                                   * (value[2] / value[3]) * (value[4] / 64)
-                               <= 1024;
+                                   * (value[2]) * (value[4] / 64) * value[3]
+                               <= 40960;
                     if(problem.groupedGemm())
                         ret = ret && (problem.groupedGemmCount() <= 16);
 
@@ -247,10 +247,10 @@ namespace Tensile
                         "prob",
                         (std::ceil(static_cast<float>(problem.freeSizeA(0)) / value[0])
                          * std::ceil(static_cast<float>(problem.freeSizeB(0)) / value[1]))
-                            * (value[2] / value[3]) * (value[4] / 64),
+                            * (value[2]) * (value[4] / 64) * value[3],
                         "==",
                         "sol",
-                        1024);
+                        40960);
                 }
             };
 
@@ -845,6 +845,63 @@ namespace Tensile
                 }
             };
 
+            struct AmaxDCheck : public Predicate_CRTP<AmaxDCheck, ContractionProblemGemm>
+            {
+                enum
+                {
+                    HasIndex = false,
+                    HasValue = true
+                };
+                bool value;
+
+                AmaxDCheck() = default;
+                AmaxDCheck(bool value)
+                    : value(value)
+                {
+                }
+
+                static std::string Type()
+                {
+                    return "AmaxDCheck";
+                }
+
+                virtual bool operator()(ContractionProblemGemm const& problem) const override
+                {
+                    bool amaxDStatusEqual = (problem.outputAmaxD() == value);
+
+                    // if value is true, then we also need to check gsu
+                    // otherwise we just check outputAmaxD
+                    if(value)
+                        return amaxDStatusEqual && problem.getParams().gsu() <= 1;
+                    else
+                        return amaxDStatusEqual;
+                }
+
+                virtual bool debugEval(ContractionProblemGemm const& problem,
+                                       std::ostream&                 stream) const override
+                {
+                    return (value) ? debugEvalCmp(problem,
+                                                  stream,
+                                                  "prob_amaxD",
+                                                  problem.outputAmaxD(),
+                                                  "==",
+                                                  "sol_amaxD",
+                                                  value,
+                                                  "prob_gsu",
+                                                  (int)(problem.getParams().gsu()),
+                                                  "<=",
+                                                  "sol_gsu",
+                                                  1)
+                                   : debugEvalCmp(problem,
+                                                  stream,
+                                                  "prob_amaxD",
+                                                  problem.outputAmaxD(),
+                                                  "==",
+                                                  "sol_amaxD",
+                                                  value);
+                }
+            };
+
             struct BetaZero : public Predicate_CRTP<BetaZero, ContractionProblemGemm>
             {
                 enum
@@ -1730,35 +1787,6 @@ namespace Tensile
                 }
             };
 
-            struct Fp16AltImpl : public Predicate_CRTP<Fp16AltImpl, ContractionProblemGemm>
-            {
-                enum
-                {
-                    HasIndex = false,
-                    HasValue = false
-                };
-
-                Fp16AltImpl() = default;
-
-                static std::string Type()
-                {
-                    return "Fp16AltImpl";
-                }
-
-                virtual bool operator()(ContractionProblemGemm const& problem) const override
-                {
-                    return problem.fp16AltImpl();
-                }
-
-                virtual bool debugEval(ContractionProblemGemm const& problem,
-                                       std::ostream&                 stream) const override
-                {
-                    bool rv = (*this)(problem);
-                    stream << rv << ": " << this->type() << std::endl;
-                    return rv;
-                }
-            };
-
             struct EqualityMatching
                 : public Predicate_CRTP<EqualityMatching, ContractionProblemGemm>
             {
@@ -1854,7 +1882,7 @@ namespace Tensile
             };
 
             // Activation
-            struct ActivationEqual : public Predicate_CRTP<ActivationEqual, ContractionProblemGemm>
+            struct ActivationCheck : public Predicate_CRTP<ActivationCheck, ContractionProblemGemm>
             {
                 enum
                 {
@@ -1863,8 +1891,8 @@ namespace Tensile
                 };
                 ActivationType value;
 
-                ActivationEqual() = default;
-                ActivationEqual(ActivationType value)
+                ActivationCheck() = default;
+                ActivationCheck(ActivationType value)
                     : value(value)
                 {
                 }
@@ -1876,7 +1904,7 @@ namespace Tensile
 
                 virtual bool operator()(ContractionProblemGemm const& problem) const override
                 {
-                    return problem.activationType() == value;
+                    return problem.activationType() == value || value == ActivationType::All;
                 }
 
                 virtual bool debugEval(ContractionProblemGemm const& problem,
@@ -2015,17 +2043,17 @@ namespace Tensile
                 }
             };
 
-            struct UseBiasEqual : public Predicate_CRTP<UseBiasEqual, ContractionProblemGemm>
+            struct UseBiasCheck : public Predicate_CRTP<UseBiasCheck, ContractionProblemGemm>
             {
                 enum
                 {
                     HasIndex = false,
                     HasValue = true
                 };
-                bool value;
+                int value;
 
-                UseBiasEqual() = default;
-                UseBiasEqual(bool value)
+                UseBiasCheck() = default;
+                UseBiasCheck(int value)
                     : value(value)
                 {
                 }
@@ -2037,14 +2065,16 @@ namespace Tensile
 
                 virtual bool operator()(ContractionProblemGemm const& problem) const override
                 {
-                    return problem.useBias() == value;
+                    return !problem.useBias() || value;
                 }
 
                 virtual bool debugEval(ContractionProblemGemm const& problem,
                                        std::ostream&                 stream) const override
                 {
-                    return debugEvalCmp(
-                        problem, stream, "prob", problem.useBias(), "==", "sol", value);
+                    bool rv = (*this)(problem);
+                    stream << *this << ": prob: " << problem.useBias()
+                           << ", Is sol support: " << value << std::endl;
+                    return rv;
                 }
             };
 
@@ -2081,17 +2111,17 @@ namespace Tensile
                 }
             };
 
-            struct UseScaleABEqual : public Predicate_CRTP<UseScaleABEqual, ContractionProblemGemm>
+            struct UseScaleABCheck : public Predicate_CRTP<UseScaleABCheck, ContractionProblemGemm>
             {
                 enum
                 {
                     HasIndex = false,
                     HasValue = true
                 };
-                bool value;
+                std::string value;
 
-                UseScaleABEqual() = default;
-                UseScaleABEqual(bool value)
+                UseScaleABCheck() = default;
+                UseScaleABCheck(std::string value)
                     : value(value)
                 {
                 }
@@ -2103,18 +2133,21 @@ namespace Tensile
 
                 virtual bool operator()(ContractionProblemGemm const& problem) const override
                 {
-                    return problem.useScaleAB() == value;
+                    return problem.useScaleAB().empty() || (problem.useScaleAB() == value);
                 }
 
                 virtual bool debugEval(ContractionProblemGemm const& problem,
                                        std::ostream&                 stream) const override
                 {
-                    return debugEvalCmp(
-                        problem, stream, "prob", problem.useScaleAB(), "==", "sol", value);
+                    bool rv = (*this)(problem);
+
+                    stream << *this << ": prob: " << problem.useScaleAB()
+                           << ", Is sol support: " << value << std::endl;
+                    return rv;
                 }
             };
 
-            struct UseScaleCDEqual : public Predicate_CRTP<UseScaleCDEqual, ContractionProblemGemm>
+            struct UseScaleCDCheck : public Predicate_CRTP<UseScaleCDCheck, ContractionProblemGemm>
             {
                 enum
                 {
@@ -2123,8 +2156,8 @@ namespace Tensile
                 };
                 bool value;
 
-                UseScaleCDEqual() = default;
-                UseScaleCDEqual(bool value)
+                UseScaleCDCheck() = default;
+                UseScaleCDCheck(bool value)
                     : value(value)
                 {
                 }
@@ -2136,29 +2169,32 @@ namespace Tensile
 
                 virtual bool operator()(ContractionProblemGemm const& problem) const override
                 {
-                    return problem.useScaleCD() == value;
+                    return !problem.useScaleCD() || value;
                 }
 
                 virtual bool debugEval(ContractionProblemGemm const& problem,
                                        std::ostream&                 stream) const override
                 {
-                    return debugEvalCmp(
-                        problem, stream, "prob", problem.useScaleCD(), "==", "sol", value);
+                    bool rv = (*this)(problem);
+
+                    stream << *this << ": prob: " << problem.useScaleCD()
+                           << ", Is sol support: " << value << std::endl;
+                    return rv;
                 }
             };
 
-            struct UseScaleAlphaVecEqual
-                : public Predicate_CRTP<UseScaleAlphaVecEqual, ContractionProblemGemm>
+            struct UseScaleAlphaVecCheck
+                : public Predicate_CRTP<UseScaleAlphaVecCheck, ContractionProblemGemm>
             {
                 enum
                 {
                     HasIndex = false,
                     HasValue = true
                 };
-                bool value;
+                int value;
 
-                UseScaleAlphaVecEqual() = default;
-                UseScaleAlphaVecEqual(bool value)
+                UseScaleAlphaVecCheck() = default;
+                UseScaleAlphaVecCheck(int value)
                     : value(value)
                 {
                 }
@@ -2170,14 +2206,17 @@ namespace Tensile
 
                 virtual bool operator()(ContractionProblemGemm const& problem) const override
                 {
-                    return problem.useScaleAlphaVec() == value;
+                    return !problem.useScaleAlphaVec() || (problem.useScaleAlphaVec() & value);
                 }
 
                 virtual bool debugEval(ContractionProblemGemm const& problem,
                                        std::ostream&                 stream) const override
                 {
-                    return debugEvalCmp(
-                        problem, stream, "prob", problem.useScaleAlphaVec(), "==", "sol", value);
+                    bool rv = (*this)(problem);
+
+                    stream << *this << ": prob: " << problem.useScaleAlphaVec()
+                           << ", Is sol support: " << value << std::endl;
+                    return rv;
                 }
             };
 
@@ -2251,12 +2290,20 @@ namespace Tensile
 
                 virtual bool operator()(ContractionProblemGemm const& problem) const override
                 {
+                    if(problem.useBias() && problem.useScaleAlphaVec()
+                       && problem.useBias() != problem.useScaleAlphaVec())
+                        return false;
+
+                    int factorDim = (problem.useBias() == 1) ? 0
+                                    : problem.useBias() == 2 ? 1
+                                    : problem.useBias() == 3 ? problem.getParams().factorDim()
+                                                             : 0;
+
                     if(problem.useBias())
                     {
                         auto& tensor = problem.tensor(ContractionProblemGemm::TENSOR::BIAS);
                         if(tensor.sizes().size() == 0)
                             return false;
-
                         for(size_t i = 0; i < value.size(); i++)
                         {
                             if(value[i] == static_cast<int>(problem.biasSrc()))
@@ -2266,7 +2313,14 @@ namespace Tensile
                                 if(problem.biasSrc() == ContractionProblemGemm::TENSOR::A
                                    || problem.biasSrc() == ContractionProblemGemm::TENSOR::D)
                                 {
-                                    if(length != problem.d().sizes()[0])
+                                    auto eLength = (problem.useBias() == 1
+                                                    || problem.biasSrc()
+                                                           != ContractionProblemGemm::TENSOR::D)
+                                                       ? problem.d().sizes()[0]
+                                                   : (problem.useBias() <= 3)
+                                                       ? problem.d().sizes()[factorDim]
+                                                       : -1;
+                                    if(length < eLength)
                                         return false;
                                 }
                                 else if(problem.biasSrc() == ContractionProblemGemm::TENSOR::B)
@@ -2399,7 +2453,8 @@ namespace Tensile
 
                 virtual bool operator()(ContractionProblemGemm const& problem) const override
                 {
-                    if(problem.getUseDeviceUserArguments())
+                    // Currently some solutions in gemm cannot enable this feature because lack of sgpr.
+                    if(problem.getUseDeviceUserArguments() && problem.groupedGemm())
                         return problem.getUseDeviceUserArguments() == value;
                     return true;
                 }
