@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (C) 2022-2023 Advanced Micro Devices, Inc. All rights reserved.
+ * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -40,6 +40,8 @@
 #include <Tensile/Predicates.hpp>
 #include <Tensile/Utils.hpp>
 
+#define TENSILE_COMMON_KERNEL_ARGS_SIZE 16
+
 namespace Tensile
 {
     template <typename TAct>
@@ -117,6 +119,15 @@ namespace Tensile
         using Inputs        = ContractionInputs;
         using GroupedInputs = ContractionGroupedInputs;
 
+        /**
+  * Indicate a solution is equally or estimatedly matched.
+  */
+        enum class MatchingTag
+        {
+            Equal,
+            Estimated
+        };
+
         static std::string Type()
         {
             return "Contraction";
@@ -307,9 +318,13 @@ namespace Tensile
                             uint32_t const&          workspaceOffsetInByte,
                             KA&                      args) const;
 
-        // Kernel related arguments (e.g. MT, GSU...)
-        template <bool T_Debug, typename KA>
-        void kernelArgs(KA& args, const ContractionProblemParameters& param) const;
+        // Common kernel related arguments (e.g. gemm_count, arg type, MT, GSU...)
+        template <bool T_Debug, bool Legacy, typename KA>
+        void kernelArgs(uint32_t                            gemmCount,
+                        uint32_t                            argType,
+                        KA&                                 args,
+                        uint32_t                            numWorkGroups,
+                        const ContractionProblemParameters& param) const;
 
         template <typename KA>
         inline void calculateSingleCallWorkGroupItems(std::vector<Problem> const& problems,
@@ -397,7 +412,14 @@ namespace Tensile
             dim3 threadTile;
             dim3 macroTile;
 
+            std::array<int, 4> matrixInstruction;
+            size_t             grvwA = 1;
+            size_t             grvwB = 1;
+            size_t             gwvwC = 1;
+            size_t             gwvwD = 1;
+
             size_t staggerU           = 0;
+            size_t staggerUMapping    = 0;
             size_t depthU             = 0;
             size_t globalSplitUPGR    = 0;
             size_t globalSplitU       = 0;
@@ -419,12 +441,19 @@ namespace Tensile
             bool activationFused = true;
 
             std::string customKernelName;
+
+            int workGroupMappingXCC = 1;
+            bool globalSplitUCoalesced = false;
+            bool globalSplitUWorkGroupMappingRoundRobin = false;
         };
 
         struct InternalArgsSupport
         {
-            bool gsu = true;
-            bool wgm = true;
+            int  version          = 0;
+            bool gsu              = true;
+            bool wgm              = true;
+            bool staggerU         = true;
+            bool useUniversalArgs = true;
         };
 
         struct ProblemType
@@ -444,22 +473,23 @@ namespace Tensile
             bool                  highPrecisionAccumulate   = false;
             bool                  useBeta                   = true;
             bool                  useGradient               = false;
-            bool                  useBias                   = false;
+            int                   useBias                   = 0;
             bool                  useE                      = false;
-            bool                  useScaleAB                = false;
+            std::string           useScaleAB                = "";
             bool                  useScaleCD                = false;
-            bool                  useScaleAlphaVec          = false;
+            int                   useScaleAlphaVec          = 0;
             bool                  useInitialStridesAB       = false;
             bool                  useInitialStridesCD       = false;
             bool                  stridedBatched            = true;
+            bool                  outputAmaxD               = false;
             bool                  groupedGemm               = false;
-            bool                  fp16AltImpl               = false;
             ActivationType        activationType            = ActivationType::None;
             int                   activationArgLength       = 0;
             bool                  activationNoGuard         = false;
             std::vector<int>      biasSrcWhiteList;
             std::vector<DataType> biasDataTypeWhiteList;
             int                   sparse                     = 0;
+            bool                  stochasticRounding         = false;
             bool                  supportDeviceUserArguments = false;
         };
 
@@ -496,6 +526,7 @@ namespace Tensile
         int32_t               libraryLogicIndex = -1;
         std::map<int, double> ideals;
         LinearModel           linearModel;
+        MatchingTag           tag{MatchingTag::Estimated};
 
         uint32_t magicNumberAlg1(uint32_t x, uint32_t* magicShift) const;
         uint32_t magicNumberAlg2(uint32_t x, uint32_t* magicShift) const;
